@@ -1,11 +1,12 @@
 """Represent the AsusWrt router."""
 from __future__ import annotations
 
+from collections.abc import Callable
 from datetime import datetime, timedelta
 import logging
 from typing import Any
 
-from aioasuswrt.asuswrt import AsusWrt
+from aioasuswrt.asuswrt import AsusWrt, Device as WrtDevice
 
 from homeassistant.components.device_tracker.const import (
     CONF_CONSIDER_HOME,
@@ -23,6 +24,8 @@ from homeassistant.const import (
 )
 from homeassistant.core import CALLBACK_TYPE, HomeAssistant, callback
 from homeassistant.exceptions import ConfigEntryNotReady
+from homeassistant.helpers import entity_registry as er
+from homeassistant.helpers.device_registry import format_mac
 from homeassistant.helpers.dispatcher import async_dispatcher_send
 from homeassistant.helpers.entity import DeviceInfo
 from homeassistant.helpers.event import async_track_time_interval
@@ -44,6 +47,7 @@ from .const import (
     SENSORS_CONNECTED_DEVICE,
     SENSORS_LOAD_AVG,
     SENSORS_RATES,
+    SENSORS_TEMPERATURES,
 )
 
 CONF_REQ_RELOAD = [CONF_DNSMASQ, CONF_INTERFACE, CONF_REQUIRE_IP]
@@ -57,6 +61,7 @@ SENSORS_TYPE_BYTES = "sensors_bytes"
 SENSORS_TYPE_COUNT = "sensors_count"
 SENSORS_TYPE_LOAD_AVG = "sensors_load_avg"
 SENSORS_TYPE_RATES = "sensors_rates"
+SENSORS_TYPE_TEMPERATURES = "sensors_temperatures"
 
 _LOGGER = logging.getLogger(__name__)
 
@@ -74,17 +79,17 @@ def _get_dict(keys: list, values: list) -> dict[str, Any]:
 class AsusWrtSensorDataHandler:
     """Data handler for AsusWrt sensor."""
 
-    def __init__(self, hass, api):
+    def __init__(self, hass: HomeAssistant, api: AsusWrt) -> None:
         """Initialize a AsusWrt sensor data handler."""
         self._hass = hass
         self._api = api
         self._connected_devices = 0
 
-    async def _get_connected_devices(self):
+    async def _get_connected_devices(self) -> dict[str, int]:
         """Return number of connected devices."""
         return {SENSORS_CONNECTED_DEVICE[0]: self._connected_devices}
 
-    async def _get_bytes(self):
+    async def _get_bytes(self) -> dict[str, Any]:
         """Fetch byte information from the router."""
         try:
             datas = await self._api.async_get_bytes_total()
@@ -93,7 +98,7 @@ class AsusWrtSensorDataHandler:
 
         return _get_dict(SENSORS_BYTES, datas)
 
-    async def _get_rates(self):
+    async def _get_rates(self) -> dict[str, Any]:
         """Fetch rates information from the router."""
         try:
             rates = await self._api.async_get_current_transfer_rates()
@@ -102,7 +107,7 @@ class AsusWrtSensorDataHandler:
 
         return _get_dict(SENSORS_RATES, rates)
 
-    async def _get_load_avg(self):
+    async def _get_load_avg(self) -> dict[str, Any]:
         """Fetch load average information from the router."""
         try:
             avg = await self._api.async_get_loadavg()
@@ -111,14 +116,25 @@ class AsusWrtSensorDataHandler:
 
         return _get_dict(SENSORS_LOAD_AVG, avg)
 
-    def update_device_count(self, conn_devices: int):
+    async def _get_temperatures(self) -> dict[str, Any]:
+        """Fetch temperatures information from the router."""
+        try:
+            temperatures: dict[str, Any] = await self._api.async_get_temperature()
+        except (OSError, ValueError) as exc:
+            raise UpdateFailed(exc) from exc
+
+        return temperatures
+
+    def update_device_count(self, conn_devices: int) -> bool:
         """Update connected devices attribute."""
         if self._connected_devices == conn_devices:
             return False
         self._connected_devices = conn_devices
         return True
 
-    async def get_coordinator(self, sensor_type: str, should_poll=True):
+    async def get_coordinator(
+        self, sensor_type: str, should_poll: bool = True
+    ) -> DataUpdateCoordinator:
         """Get the coordinator for a specific sensor type."""
         if sensor_type == SENSORS_TYPE_COUNT:
             method = self._get_connected_devices
@@ -128,6 +144,8 @@ class AsusWrtSensorDataHandler:
             method = self._get_load_avg
         elif sensor_type == SENSORS_TYPE_RATES:
             method = self._get_rates
+        elif sensor_type == SENSORS_TYPE_TEMPERATURES:
+            method = self._get_temperatures
         else:
             raise RuntimeError(f"Invalid sensor type: {sensor_type}")
 
@@ -147,15 +165,15 @@ class AsusWrtSensorDataHandler:
 class AsusWrtDevInfo:
     """Representation of a AsusWrt device info."""
 
-    def __init__(self, mac, name=None):
+    def __init__(self, mac: str, name: str | None = None) -> None:
         """Initialize a AsusWrt device info."""
         self._mac = mac
         self._name = name
-        self._ip_address = None
-        self._last_activity = None
+        self._ip_address: str | None = None
+        self._last_activity: datetime | None = None
         self._connected = False
 
-    def update(self, dev_info=None, consider_home=0):
+    def update(self, dev_info: WrtDevice | None = None, consider_home: int = 0) -> None:
         """Update AsusWrt device info."""
         utc_point_in_time = dt_util.utcnow()
         if dev_info:
@@ -167,32 +185,34 @@ class AsusWrtDevInfo:
 
         elif self._connected:
             self._connected = (
-                utc_point_in_time - self._last_activity
-            ).total_seconds() < consider_home
+                self._last_activity is not None
+                and (utc_point_in_time - self._last_activity).total_seconds()
+                < consider_home
+            )
             self._ip_address = None
 
     @property
-    def is_connected(self):
+    def is_connected(self) -> bool:
         """Return connected status."""
         return self._connected
 
     @property
-    def mac(self):
+    def mac(self) -> str:
         """Return device mac address."""
         return self._mac
 
     @property
-    def name(self):
+    def name(self) -> str | None:
         """Return device name."""
         return self._name
 
     @property
-    def ip_address(self):
+    def ip_address(self) -> str | None:
         """Return device ip address."""
         return self._ip_address
 
     @property
-    def last_activity(self):
+    def last_activity(self) -> datetime | None:
         """Return device last activity."""
         return self._last_activity
 
@@ -206,21 +226,21 @@ class AsusWrtRouter:
         self._entry = entry
 
         self._api: AsusWrt = None
-        self._protocol = entry.data[CONF_PROTOCOL]
-        self._host = entry.data[CONF_HOST]
-        self._model = "Asus Router"
-        self._sw_v = None
+        self._protocol: str = entry.data[CONF_PROTOCOL]
+        self._host: str = entry.data[CONF_HOST]
+        self._model: str = "Asus Router"
+        self._sw_v: str | None = None
 
-        self._devices: dict[str, Any] = {}
-        self._connected_devices = 0
-        self._connect_error = False
+        self._devices: dict[str, AsusWrtDevInfo] = {}
+        self._connected_devices: int = 0
+        self._connect_error: bool = False
 
-        self._sensors_data_handler: AsusWrtSensorDataHandler = None
+        self._sensors_data_handler: AsusWrtSensorDataHandler | None = None
         self._sensors_coordinator: dict[str, Any] = {}
 
-        self._on_close = []
+        self._on_close: list[Callable] = []
 
-        self._options = {
+        self._options: dict[str, Any] = {
             CONF_DNSMASQ: DEFAULT_DNSMASQ,
             CONF_INTERFACE: DEFAULT_INTERFACE,
             CONF_REQUIRE_IP: True,
@@ -229,7 +249,7 @@ class AsusWrtRouter:
 
     async def setup(self) -> None:
         """Set up a AsusWrt router."""
-        self._api = get_api(self._entry.data, self._options)
+        self._api = get_api(dict(self._entry.data), self._options)
 
         try:
             await self._api.connection.async_connect()
@@ -248,17 +268,32 @@ class AsusWrtRouter:
             self._sw_v = f"{firmware['firmver']} (build {firmware['buildno']})"
 
         # Load tracked entities from registry
-        entity_registry = await self.hass.helpers.entity_registry.async_get_registry()
-        track_entries = (
-            self.hass.helpers.entity_registry.async_entries_for_config_entry(
-                entity_registry, self._entry.entry_id
-            )
+        entity_reg = er.async_get(self.hass)
+        track_entries = er.async_entries_for_config_entry(
+            entity_reg, self._entry.entry_id
         )
         for entry in track_entries:
-            if entry.domain == TRACKER_DOMAIN:
-                self._devices[entry.unique_id] = AsusWrtDevInfo(
-                    entry.unique_id, entry.original_name
+
+            if entry.domain != TRACKER_DOMAIN:
+                continue
+            device_mac = format_mac(entry.unique_id)
+
+            # migrate entity unique ID if wrong formatted
+            if device_mac != entry.unique_id:
+                existing_entity_id = entity_reg.async_get_entity_id(
+                    TRACKER_DOMAIN, DOMAIN, device_mac
                 )
+                if existing_entity_id:
+                    # entity with uniqueid properly formatted already
+                    # exists in the registry, we delete this duplicate
+                    entity_reg.async_remove(entry.entity_id)
+                    continue
+
+                entity_reg.async_update_entity(
+                    entry.entity_id, new_unique_id=device_mac
+                )
+
+            self._devices[device_mac] = AsusWrtDevInfo(device_mac, entry.original_name)
 
         # Update devices
         await self.update_devices()
@@ -279,7 +314,7 @@ class AsusWrtRouter:
         new_device = False
         _LOGGER.debug("Checking devices for ASUS router %s", self._host)
         try:
-            wrt_devices = await self._api.async_get_connected_devices()
+            api_devices = await self._api.async_get_connected_devices()
         except OSError as exc:
             if not self._connect_error:
                 self._connect_error = True
@@ -294,18 +329,20 @@ class AsusWrtRouter:
             self._connect_error = False
             _LOGGER.info("Reconnected to ASUS router %s", self._host)
 
-        consider_home = self._options.get(
+        self._connected_devices = len(api_devices)
+        consider_home: int = self._options.get(
             CONF_CONSIDER_HOME, DEFAULT_CONSIDER_HOME.total_seconds()
         )
-        track_unknown = self._options.get(CONF_TRACK_UNKNOWN, DEFAULT_TRACK_UNKNOWN)
+        track_unknown: bool = self._options.get(
+            CONF_TRACK_UNKNOWN, DEFAULT_TRACK_UNKNOWN
+        )
 
+        wrt_devices = {format_mac(mac): dev for mac, dev in api_devices.items()}
         for device_mac, device in self._devices.items():
-            dev_info = wrt_devices.get(device_mac)
+            dev_info = wrt_devices.pop(device_mac, None)
             device.update(dev_info, consider_home)
 
         for device_mac, dev_info in wrt_devices.items():
-            if device_mac in self._devices:
-                continue
             if not track_unknown and not dev_info.name:
                 continue
             new_device = True
@@ -316,8 +353,6 @@ class AsusWrtRouter:
         async_dispatcher_send(self.hass, self.signal_device_update)
         if new_device:
             async_dispatcher_send(self.hass, self.signal_device_new)
-
-        self._connected_devices = len(wrt_devices)
         await self._update_unpolled_sensors()
 
     async def init_sensors_coordinator(self) -> None:
@@ -328,14 +363,17 @@ class AsusWrtRouter:
         self._sensors_data_handler = AsusWrtSensorDataHandler(self.hass, self._api)
         self._sensors_data_handler.update_device_count(self._connected_devices)
 
-        sensors_types = {
+        sensors_types: dict[str, list[str]] = {
             SENSORS_TYPE_BYTES: SENSORS_BYTES,
             SENSORS_TYPE_COUNT: SENSORS_CONNECTED_DEVICE,
             SENSORS_TYPE_LOAD_AVG: SENSORS_LOAD_AVG,
             SENSORS_TYPE_RATES: SENSORS_RATES,
+            SENSORS_TYPE_TEMPERATURES: await self._get_available_temperature_sensors(),
         }
 
         for sensor_type, sensor_names in sensors_types.items():
+            if not sensor_names:
+                continue
             coordinator = await self._sensors_data_handler.get_coordinator(
                 sensor_type, sensor_type != SENSORS_TYPE_COUNT
             )
@@ -354,6 +392,23 @@ class AsusWrtRouter:
             if self._sensors_data_handler.update_device_count(self._connected_devices):
                 await coordinator.async_refresh()
 
+    async def _get_available_temperature_sensors(self) -> list[str]:
+        """Check which temperature information is available on the router."""
+        try:
+            availability = await self._api.async_find_temperature_commands()
+            available_sensors = [
+                SENSORS_TEMPERATURES[i] for i in range(3) if availability[i]
+            ]
+        except Exception as exc:  # pylint: disable=broad-except
+            _LOGGER.debug(
+                "Failed checking temperature sensor availability for ASUS router %s. Exception: %s",
+                self._host,
+                exc,
+            )
+            return []
+
+        return available_sensors
+
     async def close(self) -> None:
         """Close the connection."""
         if self._api is not None and self._protocol == PROTOCOL_TELNET:
@@ -369,13 +424,13 @@ class AsusWrtRouter:
         """Add a function to call when router is closed."""
         self._on_close.append(func)
 
-    def update_options(self, new_options: dict) -> bool:
+    def update_options(self, new_options: dict[str, Any]) -> bool:
         """Update router options."""
         req_reload = False
         for name, new_opt in new_options.items():
-            if name in (CONF_REQ_RELOAD):
+            if name in CONF_REQ_RELOAD:
                 old_opt = self._options.get(name)
-                if not old_opt or old_opt != new_opt:
+                if old_opt is None or old_opt != new_opt:
                     req_reload = True
                     break
 
@@ -385,13 +440,14 @@ class AsusWrtRouter:
     @property
     def device_info(self) -> DeviceInfo:
         """Return the device information."""
-        return {
-            "identifiers": {(DOMAIN, "AsusWRT")},
-            "name": self._host,
-            "model": self._model,
-            "manufacturer": "Asus",
-            "sw_version": self._sw_v,
-        }
+        return DeviceInfo(
+            identifiers={(DOMAIN, "AsusWRT")},
+            name=self._host,
+            model=self._model,
+            manufacturer="Asus",
+            sw_version=self._sw_v,
+            configuration_url=f"http://{self._host}",
+        )
 
     @property
     def signal_device_new(self) -> str:
@@ -409,7 +465,7 @@ class AsusWrtRouter:
         return self._host
 
     @property
-    def devices(self) -> dict[str, Any]:
+    def devices(self) -> dict[str, AsusWrtDevInfo]:
         """Return devices."""
         return self._devices
 
@@ -418,30 +474,25 @@ class AsusWrtRouter:
         """Return sensors coordinators."""
         return self._sensors_coordinator
 
-    @property
-    def api(self) -> AsusWrt:
-        """Return router API."""
-        return self._api
-
 
 async def _get_nvram_info(api: AsusWrt, info_type: str) -> dict[str, Any]:
     """Get AsusWrt router info from nvram."""
     info = {}
     try:
         info = await api.async_get_nvram(info_type)
-    except (OSError, UnicodeDecodeError) as exc:
+    except OSError as exc:
         _LOGGER.warning("Error calling method async_get_nvram(%s): %s", info_type, exc)
 
     return info
 
 
-def get_api(conf: dict, options: dict | None = None) -> AsusWrt:
+def get_api(conf: dict[str, Any], options: dict[str, Any] | None = None) -> AsusWrt:
     """Get the AsusWrt API."""
     opt = options or {}
 
     return AsusWrt(
         conf[CONF_HOST],
-        conf[CONF_PORT],
+        conf.get(CONF_PORT),
         conf[CONF_PROTOCOL] == PROTOCOL_TELNET,
         conf[CONF_USERNAME],
         conf.get(CONF_PASSWORD, ""),
