@@ -2,11 +2,11 @@
 from __future__ import annotations
 
 import asyncio
-from collections.abc import Sequence
-from datetime import timedelta
+from collections.abc import Callable, Coroutine, Sequence
+from datetime import datetime, timedelta
 import hashlib
 from types import ModuleType
-from typing import Any, Callable, final
+from typing import Any, Final, final
 
 import attr
 import voluptuous as vol
@@ -28,7 +28,7 @@ from homeassistant.const import (
     STATE_HOME,
     STATE_NOT_HOME,
 )
-from homeassistant.core import HomeAssistant, callback
+from homeassistant.core import HomeAssistant, ServiceCall, callback
 from homeassistant.exceptions import HomeAssistantError
 from homeassistant.helpers import config_per_platform, discovery
 import homeassistant.helpers.config_validation as cv
@@ -38,7 +38,7 @@ from homeassistant.helpers.event import (
     async_track_utc_time_change,
 )
 from homeassistant.helpers.restore_state import RestoreEntity
-from homeassistant.helpers.typing import ConfigType, GPSType
+from homeassistant.helpers.typing import ConfigType, GPSType, StateType
 from homeassistant.setup import async_prepare_setup_platform, async_start_setup
 from homeassistant.util import dt as dt_util
 from homeassistant.util.yaml import dump
@@ -69,9 +69,9 @@ from .const import (
     SOURCE_TYPE_ROUTER,
 )
 
-SERVICE_SEE = "see"
+SERVICE_SEE: Final = "see"
 
-SOURCE_TYPES = (
+SOURCE_TYPES: Final[tuple[str, ...]] = (
     SOURCE_TYPE_GPS,
     SOURCE_TYPE_ROUTER,
     SOURCE_TYPE_BLUETOOTH,
@@ -82,7 +82,7 @@ NEW_DEVICE_DEFAULTS_SCHEMA = vol.Any(
     None,
     vol.Schema({vol.Optional(CONF_TRACK_NEW, default=DEFAULT_TRACK_NEW): cv.boolean}),
 )
-PLATFORM_SCHEMA = cv.PLATFORM_SCHEMA.extend(
+PLATFORM_SCHEMA: Final = cv.PLATFORM_SCHEMA.extend(
     {
         vol.Optional(CONF_SCAN_INTERVAL): cv.time_period,
         vol.Optional(CONF_TRACK_NEW): cv.boolean,
@@ -92,9 +92,11 @@ PLATFORM_SCHEMA = cv.PLATFORM_SCHEMA.extend(
         vol.Optional(CONF_NEW_DEVICE_DEFAULTS, default={}): NEW_DEVICE_DEFAULTS_SCHEMA,
     }
 )
-PLATFORM_SCHEMA_BASE = cv.PLATFORM_SCHEMA_BASE.extend(PLATFORM_SCHEMA.schema)
+PLATFORM_SCHEMA_BASE: Final[vol.Schema] = cv.PLATFORM_SCHEMA_BASE.extend(
+    PLATFORM_SCHEMA.schema
+)
 
-SERVICE_SEE_PAYLOAD_SCHEMA = vol.Schema(
+SERVICE_SEE_PAYLOAD_SCHEMA: Final[vol.Schema] = vol.Schema(
     vol.All(
         cv.has_at_least_one_key(ATTR_MAC, ATTR_DEV_ID),
         {
@@ -115,23 +117,23 @@ SERVICE_SEE_PAYLOAD_SCHEMA = vol.Schema(
     )
 )
 
-YAML_DEVICES = "known_devices.yaml"
-EVENT_NEW_DEVICE = "device_tracker_new_device"
+YAML_DEVICES: Final = "known_devices.yaml"
+EVENT_NEW_DEVICE: Final = "device_tracker_new_device"
 
 
 def see(
     hass: HomeAssistant,
-    mac: str = None,
-    dev_id: str = None,
-    host_name: str = None,
-    location_name: str = None,
-    gps: GPSType = None,
-    gps_accuracy=None,
-    battery: int = None,
-    attributes: dict = None,
-):
+    mac: str | None = None,
+    dev_id: str | None = None,
+    host_name: str | None = None,
+    location_name: str | None = None,
+    gps: GPSType | None = None,
+    gps_accuracy: int | None = None,
+    battery: int | None = None,
+    attributes: dict | None = None,
+) -> None:
     """Call service to notify you see device."""
-    data = {
+    data: dict[str, Any] = {
         key: value
         for key, value in (
             (ATTR_MAC, mac),
@@ -144,7 +146,7 @@ def see(
         )
         if value is not None
     }
-    if attributes:
+    if attributes is not None:
         data[ATTR_ATTRIBUTES] = attributes
     hass.services.call(DOMAIN, SERVICE_SEE, data)
 
@@ -163,7 +165,9 @@ async def async_setup_integration(hass: HomeAssistant, config: ConfigType) -> No
     if setup_tasks:
         await asyncio.wait(setup_tasks)
 
-    async def async_platform_discovered(p_type, info):
+    async def async_platform_discovered(
+        p_type: str, info: dict[str, Any] | None
+    ) -> None:
         """Load a platform."""
         platform = await async_create_platform_type(hass, config, p_type, {})
 
@@ -179,7 +183,7 @@ async def async_setup_integration(hass: HomeAssistant, config: ConfigType) -> No
         hass, tracker.async_update_stale, second=range(0, 60, 5)
     )
 
-    async def async_see_service(call):
+    async def async_see_service(call: ServiceCall) -> None:
         """Service to see a device."""
         # Temp workaround for iOS, introduced in 0.65
         data = dict(call.data)
@@ -199,7 +203,7 @@ async def async_setup_integration(hass: HomeAssistant, config: ConfigType) -> No
 class DeviceTrackerPlatform:
     """Class to hold platform information."""
 
-    LEGACY_SETUP = (
+    LEGACY_SETUP: Final[tuple[str, ...]] = (
         "async_get_scanner",
         "get_scanner",
         "async_setup_scanner",
@@ -211,30 +215,37 @@ class DeviceTrackerPlatform:
     config: dict = attr.ib()
 
     @property
-    def type(self):
+    def type(self) -> str | None:
         """Return platform type."""
-        for methods, platform_type in ((self.LEGACY_SETUP, PLATFORM_TYPE_LEGACY),):
-            for meth in methods:
-                if hasattr(self.platform, meth):
-                    return platform_type
-
+        methods, platform_type = self.LEGACY_SETUP, PLATFORM_TYPE_LEGACY
+        for method in methods:
+            if hasattr(self.platform, method):
+                return platform_type
         return None
 
-    async def async_setup_legacy(self, hass, tracker, discovery_info=None):
+    async def async_setup_legacy(
+        self,
+        hass: HomeAssistant,
+        tracker: DeviceTracker,
+        discovery_info: dict[str, Any] | None = None,
+    ) -> None:
         """Set up a legacy platform."""
+        assert self.type == PLATFORM_TYPE_LEGACY
         full_name = f"{DOMAIN}.{self.name}"
         LOGGER.info("Setting up %s", full_name)
         with async_start_setup(hass, [full_name]):
             try:
                 scanner = None
-                setup = None
+                setup: bool | None = None
                 if hasattr(self.platform, "async_get_scanner"):
                     scanner = await self.platform.async_get_scanner(
                         hass, {DOMAIN: self.config}
                     )
                 elif hasattr(self.platform, "get_scanner"):
                     scanner = await hass.async_add_executor_job(
-                        self.platform.get_scanner, hass, {DOMAIN: self.config}
+                        self.platform.get_scanner,
+                        hass,
+                        {DOMAIN: self.config},
                     )
                 elif hasattr(self.platform, "async_setup_scanner"):
                     setup = await self.platform.async_setup_scanner(
@@ -251,12 +262,12 @@ class DeviceTrackerPlatform:
                 else:
                     raise HomeAssistantError("Invalid legacy device_tracker platform.")
 
-                if scanner:
+                if scanner is not None:
                     async_setup_scanner_platform(
                         hass, self.config, scanner, tracker.async_see, self.type
                     )
 
-                if not setup and not scanner:
+                if not setup and scanner is None:
                     LOGGER.error(
                         "Error setting up platform %s %s", self.type, self.name
                     )
@@ -270,14 +281,17 @@ class DeviceTrackerPlatform:
                 )
 
 
-async def async_extract_config(hass, config):
+async def async_extract_config(
+    hass: HomeAssistant, config: ConfigType
+) -> list[DeviceTrackerPlatform]:
     """Extract device tracker config and split between legacy and modern."""
-    legacy = []
+    legacy: list[DeviceTrackerPlatform] = []
 
     for platform in await asyncio.gather(
         *(
             async_create_platform_type(hass, config, p_type, p_config)
             for p_type, p_config in config_per_platform(config, DOMAIN)
+            if p_type is not None
         )
     ):
         if platform is None:
@@ -294,7 +308,7 @@ async def async_extract_config(hass, config):
 
 
 async def async_create_platform_type(
-    hass, config, p_type, p_config
+    hass: HomeAssistant, config: ConfigType, p_type: str, p_config: dict
 ) -> DeviceTrackerPlatform | None:
     """Determine type of platform."""
     platform = await async_prepare_setup_platform(hass, config, DOMAIN, p_type)
@@ -309,10 +323,10 @@ async def async_create_platform_type(
 def async_setup_scanner_platform(
     hass: HomeAssistant,
     config: ConfigType,
-    scanner: Any,
-    async_see_device: Callable,
+    scanner: DeviceScanner,
+    async_see_device: Callable[..., Coroutine[None, None, None]],
     platform: str,
-):
+) -> None:
     """Set up the connect scanner-based platform to device tracker.
 
     This method must be run in the event loop.
@@ -324,7 +338,7 @@ def async_setup_scanner_platform(
     # Initial scan of each mac we also tell about host name for config
     seen: Any = set()
 
-    async def async_device_tracker_scan(now: dt_util.dt.datetime):
+    async def async_device_tracker_scan(now: datetime | None) -> None:
         """Handle interval matches."""
         if update_lock.locked():
             LOGGER.warning(
@@ -350,7 +364,7 @@ def async_setup_scanner_platform(
             except NotImplementedError:
                 extra_attributes = {}
 
-            kwargs = {
+            kwargs: dict[str, Any] = {
                 "mac": mac,
                 "host_name": host_name,
                 "source_type": SOURCE_TYPE_ROUTER,
@@ -361,7 +375,7 @@ def async_setup_scanner_platform(
             }
 
             zone_home = hass.states.get(hass.components.zone.ENTITY_ID_HOME)
-            if zone_home:
+            if zone_home is not None:
                 kwargs["gps"] = [
                     zone_home.attributes[ATTR_LATITUDE],
                     zone_home.attributes[ATTR_LONGITUDE],
@@ -374,7 +388,7 @@ def async_setup_scanner_platform(
     hass.async_create_task(async_device_tracker_scan(None))
 
 
-async def get_tracker(hass, config):
+async def get_tracker(hass: HomeAssistant, config: ConfigType) -> DeviceTracker:
     """Create a tracker."""
     yaml_path = hass.config.path(YAML_DEVICES)
 
@@ -383,8 +397,7 @@ async def get_tracker(hass, config):
     consider_home = conf.get(CONF_CONSIDER_HOME, DEFAULT_CONSIDER_HOME)
 
     defaults = conf.get(CONF_NEW_DEVICE_DEFAULTS, {})
-    track_new = conf.get(CONF_TRACK_NEW)
-    if track_new is None:
+    if (track_new := conf.get(CONF_TRACK_NEW)) is None:
         track_new = defaults.get(CONF_TRACK_NEW, DEFAULT_TRACK_NEW)
 
     devices = await async_load_config(yaml_path, hass, consider_home)
@@ -400,12 +413,12 @@ class DeviceTracker:
         hass: HomeAssistant,
         consider_home: timedelta,
         track_new: bool,
-        defaults: dict,
-        devices: Sequence,
+        defaults: dict[str, Any],
+        devices: Sequence[Device],
     ) -> None:
         """Initialize a device tracker."""
         self.hass = hass
-        self.devices = {dev.dev_id: dev for dev in devices}
+        self.devices: dict[str, Device] = {dev.dev_id: dev for dev in devices}
         self.mac_to_dev = {dev.mac: dev for dev in devices if dev.mac}
         self.consider_home = consider_home
         self.track_new = (
@@ -424,21 +437,21 @@ class DeviceTracker:
 
     def see(
         self,
-        mac: str = None,
-        dev_id: str = None,
-        host_name: str = None,
-        location_name: str = None,
-        gps: GPSType = None,
-        gps_accuracy: int = None,
-        battery: int = None,
-        attributes: dict = None,
+        mac: str | None = None,
+        dev_id: str | None = None,
+        host_name: str | None = None,
+        location_name: str | None = None,
+        gps: GPSType | None = None,
+        gps_accuracy: int | None = None,
+        battery: int | None = None,
+        attributes: dict | None = None,
         source_type: str = SOURCE_TYPE_GPS,
-        picture: str = None,
-        icon: str = None,
-        consider_home: timedelta = None,
-    ):
+        picture: str | None = None,
+        icon: str | None = None,
+        consider_home: timedelta | None = None,
+    ) -> None:
         """Notify the device tracker that you see a device."""
-        self.hass.add_job(
+        self.hass.create_task(
             self.async_see(
                 mac,
                 dev_id,
@@ -457,19 +470,19 @@ class DeviceTracker:
 
     async def async_see(
         self,
-        mac: str = None,
-        dev_id: str = None,
-        host_name: str = None,
-        location_name: str = None,
-        gps: GPSType = None,
-        gps_accuracy: int = None,
-        battery: int = None,
-        attributes: dict = None,
+        mac: str | None = None,
+        dev_id: str | None = None,
+        host_name: str | None = None,
+        location_name: str | None = None,
+        gps: GPSType | None = None,
+        gps_accuracy: int | None = None,
+        battery: int | None = None,
+        attributes: dict | None = None,
         source_type: str = SOURCE_TYPE_GPS,
-        picture: str = None,
-        icon: str = None,
-        consider_home: timedelta = None,
-    ):
+        picture: str | None = None,
+        icon: str | None = None,
+        consider_home: timedelta | None = None,
+    ) -> None:
         """Notify the device tracker that you see a device.
 
         This method is a coroutine.
@@ -479,14 +492,13 @@ class DeviceTracker:
             raise HomeAssistantError("Neither mac or device id passed in")
         if mac is not None:
             mac = str(mac).upper()
-            device = self.mac_to_dev.get(mac)
-            if not device:
+            if (device := self.mac_to_dev.get(mac)) is None:
                 dev_id = util.slugify(host_name or "") or util.slugify(mac)
         else:
             dev_id = cv.slug(str(dev_id).lower())
             device = self.devices.get(dev_id)
 
-        if device:
+        if device is not None:
             await device.async_seen(
                 host_name,
                 location_name,
@@ -501,6 +513,9 @@ class DeviceTracker:
                 device.async_write_ha_state()
             return
 
+        # If it's None then device is not None and we can't get here.
+        assert dev_id is not None
+
         # Guard from calling see on entity registry entities.
         entity_id = f"{DOMAIN}.{dev_id}"
         if registry.async_is_registered(entity_id):
@@ -553,7 +568,7 @@ class DeviceTracker:
             )
         )
 
-    async def async_update_config(self, path, dev_id, device):
+    async def async_update_config(self, path: str, dev_id: str, device: Device) -> None:
         """Add device to YAML configuration file.
 
         This method is a coroutine.
@@ -564,7 +579,7 @@ class DeviceTracker:
             )
 
     @callback
-    def async_update_stale(self, now: dt_util.dt.datetime):
+    def async_update_stale(self, now: datetime) -> None:
         """Update stale devices.
 
         This method must be run in the event loop.
@@ -573,18 +588,18 @@ class DeviceTracker:
             if (device.track and device.last_update_home) and device.stale(now):
                 self.hass.async_create_task(device.async_update_ha_state(True))
 
-    async def async_setup_tracked_device(self):
+    async def async_setup_tracked_device(self) -> None:
         """Set up all not exists tracked devices.
 
         This method is a coroutine.
         """
 
-        async def async_init_single_device(dev):
+        async def async_init_single_device(dev: Device) -> None:
             """Init a single device_tracker entity."""
             await dev.async_added_to_hass()
             dev.async_write_ha_state()
 
-        tasks = []
+        tasks: list[asyncio.Task] = []
         for device in self.devices.values():
             if device.track and not device.last_seen:
                 tasks.append(
@@ -598,19 +613,17 @@ class DeviceTracker:
 class Device(RestoreEntity):
     """Base class for a tracked device."""
 
-    host_name: str = None
-    location_name: str = None
-    gps: GPSType = None
+    host_name: str | None = None
+    location_name: str | None = None
+    gps: GPSType | None = None
     gps_accuracy: int = 0
-    last_seen: dt_util.dt.datetime = None
-    consider_home: dt_util.dt.timedelta = None
-    battery: int = None
-    attributes: dict = None
-    icon: str = None
+    last_seen: datetime | None = None
+    battery: int | None = None
+    attributes: dict | None = None
 
     # Track if the last update of this device was HOME.
-    last_update_home = False
-    _state = STATE_NOT_HOME
+    last_update_home: bool = False
+    _state: str = STATE_NOT_HOME
 
     def __init__(
         self,
@@ -618,11 +631,11 @@ class Device(RestoreEntity):
         consider_home: timedelta,
         track: bool,
         dev_id: str,
-        mac: str,
-        name: str = None,
-        picture: str = None,
-        gravatar: str = None,
-        icon: str = None,
+        mac: str | None,
+        name: str | None = None,
+        picture: str | None = None,
+        gravatar: str | None = None,
+        icon: str | None = None,
     ) -> None:
         """Initialize a device."""
         self.hass = hass
@@ -643,64 +656,70 @@ class Device(RestoreEntity):
         self.config_name = name
 
         # Configured picture
+        self.config_picture: str | None
         if gravatar is not None:
             self.config_picture = get_gravatar_for_email(gravatar)
         else:
             self.config_picture = picture
 
-        self.icon = icon
+        self._icon = icon
 
-        self.source_type = None
+        self.source_type: str | None = None
 
-        self._attributes = {}
+        self._attributes: dict[str, Any] = {}
 
     @property
-    def name(self):
+    def name(self) -> str:
         """Return the name of the entity."""
         return self.config_name or self.host_name or self.dev_id or DEVICE_DEFAULT_NAME
 
     @property
-    def state(self):
+    def state(self) -> str:
         """Return the state of the device."""
         return self._state
 
     @property
-    def entity_picture(self):
+    def entity_picture(self) -> str | None:
         """Return the picture of the device."""
         return self.config_picture
 
     @final
     @property
-    def state_attributes(self):
+    def state_attributes(self) -> dict[str, StateType]:
         """Return the device state attributes."""
-        attributes = {ATTR_SOURCE_TYPE: self.source_type}
+        attributes: dict[str, StateType] = {ATTR_SOURCE_TYPE: self.source_type}
 
-        if self.gps:
+        if self.gps is not None:
             attributes[ATTR_LATITUDE] = self.gps[0]
             attributes[ATTR_LONGITUDE] = self.gps[1]
             attributes[ATTR_GPS_ACCURACY] = self.gps_accuracy
 
-        if self.battery:
+        if self.battery is not None:
             attributes[ATTR_BATTERY] = self.battery
 
         return attributes
 
     @property
-    def extra_state_attributes(self):
+    def extra_state_attributes(self) -> dict[str, Any]:
         """Return device state attributes."""
         return self._attributes
 
+    @property
+    def icon(self) -> str | None:
+        """Return device icon."""
+        return self._icon
+
     async def async_seen(
         self,
-        host_name: str = None,
-        location_name: str = None,
-        gps: GPSType = None,
-        gps_accuracy=0,
-        battery: int = None,
-        attributes: dict = None,
+        host_name: str | None = None,
+        location_name: str | None = None,
+        gps: GPSType | None = None,
+        gps_accuracy: int | None = None,
+        battery: int | None = None,
+        attributes: dict[str, Any] | None = None,
         source_type: str = SOURCE_TYPE_GPS,
-        consider_home: timedelta = None,
-    ):
+        consider_home: timedelta | None = None,
+    ) -> None:
         """Mark the device as seen."""
         self.source_type = source_type
         self.last_seen = dt_util.utcnow()
@@ -708,9 +727,9 @@ class Device(RestoreEntity):
         self.location_name = location_name
         self.consider_home = consider_home or self.consider_home
 
-        if battery:
+        if battery is not None:
             self.battery = battery
-        if attributes:
+        if attributes is not None:
             self._attributes.update(attributes)
 
         self.gps = None
@@ -726,7 +745,7 @@ class Device(RestoreEntity):
 
         await self.async_update()
 
-    def stale(self, now: dt_util.dt.datetime = None):
+    def stale(self, now: datetime | None = None) -> bool:
         """Return if device state is stale.
 
         Async friendly.
@@ -736,13 +755,13 @@ class Device(RestoreEntity):
             or (now or dt_util.utcnow()) - self.last_seen > self.consider_home
         )
 
-    def mark_stale(self):
+    def mark_stale(self) -> None:
         """Mark the device state as stale."""
         self._state = STATE_NOT_HOME
         self.gps = None
         self.last_update_home = False
 
-    async def async_update(self):
+    async def async_update(self) -> None:
         """Update state of entity.
 
         This method is a coroutine.
@@ -767,11 +786,10 @@ class Device(RestoreEntity):
             self._state = STATE_HOME
             self.last_update_home = True
 
-    async def async_added_to_hass(self):
+    async def async_added_to_hass(self) -> None:
         """Add an entity."""
         await super().async_added_to_hass()
-        state = await self.async_get_last_state()
-        if not state:
+        if not (state := await self.async_get_last_state()):
             return
         self._state = state.state
         self.last_update_home = state.state == STATE_HOME
@@ -795,34 +813,45 @@ class Device(RestoreEntity):
 class DeviceScanner:
     """Device scanner object."""
 
-    hass: HomeAssistant = None
+    hass: HomeAssistant | None = None
 
     def scan_devices(self) -> list[str]:
         """Scan for devices."""
         raise NotImplementedError()
 
-    async def async_scan_devices(self) -> Any:
+    async def async_scan_devices(self) -> list[str]:
         """Scan for devices."""
+        assert (
+            self.hass is not None
+        ), "hass should be set by async_setup_scanner_platform"
         return await self.hass.async_add_executor_job(self.scan_devices)
 
-    def get_device_name(self, device: str) -> str:
+    def get_device_name(self, device: str) -> str | None:
         """Get the name of a device."""
         raise NotImplementedError()
 
-    async def async_get_device_name(self, device: str) -> Any:
+    async def async_get_device_name(self, device: str) -> str | None:
         """Get the name of a device."""
+        assert (
+            self.hass is not None
+        ), "hass should be set by async_setup_scanner_platform"
         return await self.hass.async_add_executor_job(self.get_device_name, device)
 
     def get_extra_attributes(self, device: str) -> dict:
         """Get the extra attributes of a device."""
         raise NotImplementedError()
 
-    async def async_get_extra_attributes(self, device: str) -> Any:
+    async def async_get_extra_attributes(self, device: str) -> dict:
         """Get the extra attributes of a device."""
+        assert (
+            self.hass is not None
+        ), "hass should be set by async_setup_scanner_platform"
         return await self.hass.async_add_executor_job(self.get_extra_attributes, device)
 
 
-async def async_load_config(path: str, hass: HomeAssistant, consider_home: timedelta):
+async def async_load_config(
+    path: str, hass: HomeAssistant, consider_home: timedelta
+) -> list[Device]:
     """Load devices from YAML configuration file.
 
     This method is a coroutine.
@@ -842,7 +871,7 @@ async def async_load_config(path: str, hass: HomeAssistant, consider_home: timed
             ),
         }
     )
-    result = []
+    result: list[Device] = []
     try:
         devices = await hass.async_add_executor_job(load_yaml_config_file, path)
     except HomeAssistantError as err:
@@ -865,10 +894,10 @@ async def async_load_config(path: str, hass: HomeAssistant, consider_home: timed
     return result
 
 
-def update_config(path: str, dev_id: str, device: Device):
+def update_config(path: str, dev_id: str, device: Device) -> None:
     """Add device to YAML configuration file."""
-    with open(path, "a") as out:
-        device = {
+    with open(path, "a", encoding="utf8") as out:
+        device_config = {
             device.dev_id: {
                 ATTR_NAME: device.name,
                 ATTR_MAC: device.mac,
@@ -878,10 +907,10 @@ def update_config(path: str, dev_id: str, device: Device):
             }
         }
         out.write("\n")
-        out.write(dump(device))
+        out.write(dump(device_config))
 
 
-def get_gravatar_for_email(email: str):
+def get_gravatar_for_email(email: str) -> str:
     """Return an 80px Gravatar for the given email address.
 
     Async friendly.
