@@ -1,28 +1,26 @@
 """Support for ESPHome covers."""
 from __future__ import annotations
 
+from typing import Any
+
 from aioesphomeapi import CoverInfo, CoverOperation, CoverState
 
 from homeassistant.components.cover import (
     ATTR_POSITION,
     ATTR_TILT_POSITION,
-    SUPPORT_CLOSE,
-    SUPPORT_CLOSE_TILT,
-    SUPPORT_OPEN,
-    SUPPORT_OPEN_TILT,
-    SUPPORT_SET_POSITION,
-    SUPPORT_SET_TILT_POSITION,
-    SUPPORT_STOP,
+    DEVICE_CLASSES,
     CoverEntity,
+    CoverEntityFeature,
 )
 from homeassistant.config_entries import ConfigEntry
 from homeassistant.core import HomeAssistant
+from homeassistant.helpers.entity_platform import AddEntitiesCallback
 
 from . import EsphomeEntity, esphome_state_property, platform_async_setup_entry
 
 
 async def async_setup_entry(
-    hass: HomeAssistant, entry: ConfigEntry, async_add_entities
+    hass: HomeAssistant, entry: ConfigEntry, async_add_entities: AddEntitiesCallback
 ) -> None:
     """Set up ESPHome covers based on a config entry."""
     await platform_async_setup_entry(
@@ -36,26 +34,34 @@ async def async_setup_entry(
     )
 
 
-class EsphomeCover(EsphomeEntity, CoverEntity):
-    """A cover implementation for ESPHome."""
+# https://github.com/PyCQA/pylint/issues/3150 for all @esphome_state_property
+# pylint: disable=invalid-overridden-method
 
-    @property
-    def _static_info(self) -> CoverInfo:
-        return super()._static_info
+
+class EsphomeCover(EsphomeEntity[CoverInfo, CoverState], CoverEntity):
+    """A cover implementation for ESPHome."""
 
     @property
     def supported_features(self) -> int:
         """Flag supported features."""
-        flags = SUPPORT_OPEN | SUPPORT_CLOSE | SUPPORT_STOP
+        flags = (
+            CoverEntityFeature.OPEN | CoverEntityFeature.CLOSE | CoverEntityFeature.STOP
+        )
         if self._static_info.supports_position:
-            flags |= SUPPORT_SET_POSITION
+            flags |= CoverEntityFeature.SET_POSITION
         if self._static_info.supports_tilt:
-            flags |= SUPPORT_OPEN_TILT | SUPPORT_CLOSE_TILT | SUPPORT_SET_TILT_POSITION
+            flags |= (
+                CoverEntityFeature.OPEN_TILT
+                | CoverEntityFeature.CLOSE_TILT
+                | CoverEntityFeature.SET_TILT_POSITION
+            )
         return flags
 
     @property
-    def device_class(self) -> str:
+    def device_class(self) -> str | None:
         """Return the class of this device, from component DEVICE_CLASSES."""
+        if self._static_info.device_class not in DEVICE_CLASSES:
+            return None
         return self._static_info.device_class
 
     @property
@@ -63,13 +69,6 @@ class EsphomeCover(EsphomeEntity, CoverEntity):
         """Return true if we do optimistic updates."""
         return self._static_info.assumed_state
 
-    @property
-    def _state(self) -> CoverState | None:
-        return super()._state
-
-    # https://github.com/PyCQA/pylint/issues/3150 for all @esphome_state_property
-    # pylint: disable=invalid-overridden-method
-
     @esphome_state_property
     def is_closed(self) -> bool | None:
         """Return if the cover is closed or not."""
@@ -94,39 +93,39 @@ class EsphomeCover(EsphomeEntity, CoverEntity):
         return round(self._state.position * 100.0)
 
     @esphome_state_property
-    def current_cover_tilt_position(self) -> float | None:
+    def current_cover_tilt_position(self) -> int | None:
         """Return current position of cover tilt. 0 is closed, 100 is open."""
         if not self._static_info.supports_tilt:
             return None
-        return self._state.tilt * 100.0
+        return round(self._state.tilt * 100.0)
 
-    async def async_open_cover(self, **kwargs) -> None:
+    async def async_open_cover(self, **kwargs: Any) -> None:
         """Open the cover."""
         await self._client.cover_command(key=self._static_info.key, position=1.0)
 
-    async def async_close_cover(self, **kwargs) -> None:
+    async def async_close_cover(self, **kwargs: Any) -> None:
         """Close cover."""
         await self._client.cover_command(key=self._static_info.key, position=0.0)
 
-    async def async_stop_cover(self, **kwargs) -> None:
+    async def async_stop_cover(self, **kwargs: Any) -> None:
         """Stop the cover."""
         await self._client.cover_command(key=self._static_info.key, stop=True)
 
-    async def async_set_cover_position(self, **kwargs) -> None:
+    async def async_set_cover_position(self, **kwargs: int) -> None:
         """Move the cover to a specific position."""
         await self._client.cover_command(
             key=self._static_info.key, position=kwargs[ATTR_POSITION] / 100
         )
 
-    async def async_open_cover_tilt(self, **kwargs) -> None:
+    async def async_open_cover_tilt(self, **kwargs: Any) -> None:
         """Open the cover tilt."""
         await self._client.cover_command(key=self._static_info.key, tilt=1.0)
 
-    async def async_close_cover_tilt(self, **kwargs) -> None:
+    async def async_close_cover_tilt(self, **kwargs: Any) -> None:
         """Close the cover tilt."""
         await self._client.cover_command(key=self._static_info.key, tilt=0.0)
 
-    async def async_set_cover_tilt_position(self, **kwargs) -> None:
+    async def async_set_cover_tilt_position(self, **kwargs: int) -> None:
         """Move the cover tilt to a specific position."""
         await self._client.cover_command(
             key=self._static_info.key, tilt=kwargs[ATTR_TILT_POSITION] / 100
