@@ -12,6 +12,7 @@ from homeassistant.components import (
     fan,
     group,
     input_boolean,
+    input_button,
     input_select,
     light,
     lock,
@@ -303,9 +304,7 @@ class CameraStreamTrait(_Trait):
 
     async def execute(self, command, data, params, challenge):
         """Execute a get camera stream command."""
-        url = await self.hass.components.camera.async_request_stream(
-            self.state.entity_id, "hls"
-        )
+        url = await camera.async_request_stream(self.hass, self.state.entity_id, "hls")
         self.stream_info = {
             "cameraStreamAccessUrl": f"{get_url(self.hass)}{url}",
             "cameraStreamReceiverAppId": CAST_APP_ID_HOMEASSISTANT_MEDIA,
@@ -514,7 +513,12 @@ class SceneTrait(_Trait):
     @staticmethod
     def supported(domain, features, device_class, _):
         """Test if state is supported."""
-        return domain in (button.DOMAIN, scene.DOMAIN, script.DOMAIN)
+        return domain in (
+            button.DOMAIN,
+            input_button.DOMAIN,
+            scene.DOMAIN,
+            script.DOMAIN,
+        )
 
     def sync_attributes(self):
         """Return scene attributes for a sync request."""
@@ -530,6 +534,8 @@ class SceneTrait(_Trait):
         service = SERVICE_TURN_ON
         if self.state.domain == button.DOMAIN:
             service = button.SERVICE_PRESS
+        elif self.state.domain == input_button.DOMAIN:
+            service = input_button.SERVICE_PRESS
 
         # Don't block for scripts or buttons, as they can be slow.
         await self.hass.services.async_call(
@@ -537,7 +543,8 @@ class SceneTrait(_Trait):
             service,
             {ATTR_ENTITY_ID: self.state.entity_id},
             blocking=(not self.config.should_report_state)
-            and self.state.domain not in (button.DOMAIN, script.DOMAIN),
+            and self.state.domain
+            not in (button.DOMAIN, input_button.DOMAIN, script.DOMAIN),
             context=data.context,
         )
 
@@ -767,14 +774,10 @@ class StartStopTrait(_Trait):
         """Execute a StartStop command."""
         if command == COMMAND_STARTSTOP:
             if params["start"] is False:
-                if (
-                    self.state.state
-                    in (
-                        cover.STATE_CLOSING,
-                        cover.STATE_OPENING,
-                    )
-                    or self.state.attributes.get(ATTR_ASSUMED_STATE)
-                ):
+                if self.state.state in (
+                    cover.STATE_CLOSING,
+                    cover.STATE_OPENING,
+                ) or self.state.attributes.get(ATTR_ASSUMED_STATE):
                     await self.hass.services.async_call(
                         self.state.domain,
                         cover.SERVICE_STOP_COVER,
@@ -809,7 +812,8 @@ class TemperatureControlTrait(_Trait):
     def supported(domain, features, device_class, _):
         """Test if state is supported."""
         return (
-            domain == sensor.DOMAIN and device_class == sensor.DEVICE_CLASS_TEMPERATURE
+            domain == sensor.DOMAIN
+            and device_class == sensor.SensorDeviceClass.TEMPERATURE
         )
 
     def sync_attributes(self):
@@ -1113,7 +1117,10 @@ class HumiditySettingTrait(_Trait):
         if domain == humidifier.DOMAIN:
             return True
 
-        return domain == sensor.DOMAIN and device_class == sensor.DEVICE_CLASS_HUMIDITY
+        return (
+            domain == sensor.DOMAIN
+            and device_class == sensor.SensorDeviceClass.HUMIDITY
+        )
 
     def sync_attributes(self):
         """Return humidity attributes for a sync request."""
@@ -1123,7 +1130,7 @@ class HumiditySettingTrait(_Trait):
 
         if domain == sensor.DOMAIN:
             device_class = attrs.get(ATTR_DEVICE_CLASS)
-            if device_class == sensor.DEVICE_CLASS_HUMIDITY:
+            if device_class == sensor.SensorDeviceClass.HUMIDITY:
                 response["queryOnlyHumiditySetting"] = True
 
         elif domain == humidifier.DOMAIN:
@@ -1146,7 +1153,7 @@ class HumiditySettingTrait(_Trait):
 
         if domain == sensor.DOMAIN:
             device_class = attrs.get(ATTR_DEVICE_CLASS)
-            if device_class == sensor.DEVICE_CLASS_HUMIDITY:
+            if device_class == sensor.SensorDeviceClass.HUMIDITY:
                 current_humidity = self.state.state
                 if current_humidity not in (STATE_UNKNOWN, STATE_UNAVAILABLE):
                     response["humidityAmbientPercent"] = round(float(current_humidity))
@@ -1761,9 +1768,9 @@ class OpenCloseTrait(_Trait):
 
     # Cover device classes that require 2FA
     COVER_2FA = (
-        cover.DEVICE_CLASS_DOOR,
-        cover.DEVICE_CLASS_GARAGE,
-        cover.DEVICE_CLASS_GATE,
+        cover.CoverDeviceClass.DOOR,
+        cover.CoverDeviceClass.GARAGE,
+        cover.CoverDeviceClass.GATE,
     )
 
     name = TRAIT_OPENCLOSE
@@ -1776,11 +1783,11 @@ class OpenCloseTrait(_Trait):
             return True
 
         return domain == binary_sensor.DOMAIN and device_class in (
-            binary_sensor.DEVICE_CLASS_DOOR,
-            binary_sensor.DEVICE_CLASS_GARAGE_DOOR,
-            binary_sensor.DEVICE_CLASS_LOCK,
-            binary_sensor.DEVICE_CLASS_OPENING,
-            binary_sensor.DEVICE_CLASS_WINDOW,
+            binary_sensor.BinarySensorDeviceClass.DOOR,
+            binary_sensor.BinarySensorDeviceClass.GARAGE_DOOR,
+            binary_sensor.BinarySensorDeviceClass.LOCK,
+            binary_sensor.BinarySensorDeviceClass.OPENING,
+            binary_sensor.BinarySensorDeviceClass.WINDOW,
         )
 
     @staticmethod
@@ -2247,7 +2254,7 @@ class ChannelTrait(_Trait):
         if (
             domain == media_player.DOMAIN
             and (features & media_player.SUPPORT_PLAY_MEDIA)
-            and device_class == media_player.DEVICE_CLASS_TV
+            and device_class == media_player.MediaPlayerDeviceClass.TV
         ):
             return True
 
@@ -2295,12 +2302,12 @@ class SensorStateTrait(_Trait):
     """
 
     sensor_types = {
-        sensor.DEVICE_CLASS_AQI: ("AirQuality", "AQI"),
-        sensor.DEVICE_CLASS_CO: ("CarbonDioxideLevel", "PARTS_PER_MILLION"),
-        sensor.DEVICE_CLASS_CO2: ("CarbonMonoxideLevel", "PARTS_PER_MILLION"),
-        sensor.DEVICE_CLASS_PM25: ("PM2.5", "MICROGRAMS_PER_CUBIC_METER"),
-        sensor.DEVICE_CLASS_PM10: ("PM10", "MICROGRAMS_PER_CUBIC_METER"),
-        sensor.DEVICE_CLASS_VOLATILE_ORGANIC_COMPOUNDS: (
+        sensor.SensorDeviceClass.AQI: ("AirQuality", "AQI"),
+        sensor.SensorDeviceClass.CO: ("CarbonMonoxideLevel", "PARTS_PER_MILLION"),
+        sensor.SensorDeviceClass.CO2: ("CarbonDioxideLevel", "PARTS_PER_MILLION"),
+        sensor.SensorDeviceClass.PM25: ("PM2.5", "MICROGRAMS_PER_CUBIC_METER"),
+        sensor.SensorDeviceClass.PM10: ("PM10", "MICROGRAMS_PER_CUBIC_METER"),
+        sensor.SensorDeviceClass.VOLATILE_ORGANIC_COMPOUNDS: (
             "VolatileOrganicCompounds",
             "PARTS_PER_MILLION",
         ),
