@@ -5,51 +5,28 @@ import datetime
 from typing import Any
 
 import somecomfort
-import voluptuous as vol
 
-from homeassistant.components.climate import PLATFORM_SCHEMA, ClimateEntity
+from homeassistant.components.climate import ClimateEntity
 from homeassistant.components.climate.const import (
     ATTR_TARGET_TEMP_HIGH,
     ATTR_TARGET_TEMP_LOW,
-    CURRENT_HVAC_COOL,
-    CURRENT_HVAC_FAN,
-    CURRENT_HVAC_HEAT,
-    CURRENT_HVAC_IDLE,
+    DEFAULT_MAX_TEMP,
+    DEFAULT_MIN_TEMP,
     FAN_AUTO,
     FAN_DIFFUSE,
     FAN_ON,
-    HVAC_MODE_COOL,
-    HVAC_MODE_HEAT,
-    HVAC_MODE_HEAT_COOL,
-    HVAC_MODE_OFF,
     PRESET_AWAY,
     PRESET_NONE,
-    SUPPORT_AUX_HEAT,
-    SUPPORT_FAN_MODE,
-    SUPPORT_PRESET_MODE,
-    SUPPORT_TARGET_HUMIDITY,
-    SUPPORT_TARGET_TEMPERATURE,
-    SUPPORT_TARGET_TEMPERATURE_RANGE,
+    ClimateEntityFeature,
+    HVACAction,
+    HVACMode,
 )
-from homeassistant.config_entries import SOURCE_IMPORT
-from homeassistant.const import (
-    ATTR_TEMPERATURE,
-    CONF_PASSWORD,
-    CONF_REGION,
-    CONF_USERNAME,
-    TEMP_CELSIUS,
-    TEMP_FAHRENHEIT,
-)
-import homeassistant.helpers.config_validation as cv
+from homeassistant.const import ATTR_TEMPERATURE, TEMP_CELSIUS, TEMP_FAHRENHEIT
 
 from .const import (
     _LOGGER,
     CONF_COOL_AWAY_TEMPERATURE,
-    CONF_DEV_ID,
     CONF_HEAT_AWAY_TEMPERATURE,
-    CONF_LOC_ID,
-    DEFAULT_COOL_AWAY_TEMPERATURE,
-    DEFAULT_HEAT_AWAY_TEMPERATURE,
     DOMAIN,
 )
 
@@ -59,43 +36,24 @@ ATTR_PERMANENT_HOLD = "permanent_hold"
 
 PRESET_HOLD = "Hold"
 
-PLATFORM_SCHEMA = vol.All(
-    cv.deprecated(CONF_REGION),
-    PLATFORM_SCHEMA.extend(
-        {
-            vol.Required(CONF_USERNAME): cv.string,
-            vol.Required(CONF_PASSWORD): cv.string,
-            vol.Optional(
-                CONF_COOL_AWAY_TEMPERATURE, default=DEFAULT_COOL_AWAY_TEMPERATURE
-            ): vol.Coerce(int),
-            vol.Optional(
-                CONF_HEAT_AWAY_TEMPERATURE, default=DEFAULT_HEAT_AWAY_TEMPERATURE
-            ): vol.Coerce(int),
-            vol.Optional(CONF_REGION): cv.string,
-            vol.Optional(CONF_DEV_ID): cv.string,
-            vol.Optional(CONF_LOC_ID): cv.string,
-        }
-    ),
-)
-
 HVAC_MODE_TO_HW_MODE = {
-    "SwitchOffAllowed": {HVAC_MODE_OFF: "off"},
-    "SwitchAutoAllowed": {HVAC_MODE_HEAT_COOL: "auto"},
-    "SwitchCoolAllowed": {HVAC_MODE_COOL: "cool"},
-    "SwitchHeatAllowed": {HVAC_MODE_HEAT: "heat"},
+    "SwitchOffAllowed": {HVACMode.OFF: "off"},
+    "SwitchAutoAllowed": {HVACMode.HEAT_COOL: "auto"},
+    "SwitchCoolAllowed": {HVACMode.COOL: "cool"},
+    "SwitchHeatAllowed": {HVACMode.HEAT: "heat"},
 }
 HW_MODE_TO_HVAC_MODE = {
-    "off": HVAC_MODE_OFF,
-    "emheat": HVAC_MODE_HEAT,
-    "heat": HVAC_MODE_HEAT,
-    "cool": HVAC_MODE_COOL,
-    "auto": HVAC_MODE_HEAT_COOL,
+    "off": HVACMode.OFF,
+    "emheat": HVACMode.HEAT,
+    "heat": HVACMode.HEAT,
+    "cool": HVACMode.COOL,
+    "auto": HVACMode.HEAT_COOL,
 }
 HW_MODE_TO_HA_HVAC_ACTION = {
-    "off": CURRENT_HVAC_IDLE,
-    "fan": CURRENT_HVAC_FAN,
-    "heat": CURRENT_HVAC_HEAT,
-    "cool": CURRENT_HVAC_COOL,
+    "off": HVACAction.IDLE,
+    "fan": HVACAction.FAN,
+    "heat": HVACAction.HEATING,
+    "cool": HVACAction.COOLING,
 }
 FAN_MODE_TO_HW = {
     "fanModeOnAllowed": {FAN_ON: "on"},
@@ -114,8 +72,8 @@ PARALLEL_UPDATES = 1
 
 async def async_setup_entry(hass, config, async_add_entities, discovery_info=None):
     """Set up the Honeywell thermostat."""
-    cool_away_temp = config.data.get(CONF_COOL_AWAY_TEMPERATURE)
-    heat_away_temp = config.data.get(CONF_HEAT_AWAY_TEMPERATURE)
+    cool_away_temp = config.options.get(CONF_COOL_AWAY_TEMPERATURE)
+    heat_away_temp = config.options.get(CONF_HEAT_AWAY_TEMPERATURE)
 
     data = hass.data[DOMAIN][config.entry_id]
 
@@ -127,26 +85,6 @@ async def async_setup_entry(hass, config, async_add_entities, discovery_info=Non
     )
 
 
-async def async_setup_platform(hass, config, add_entities, discovery_info=None):
-    """Set up the Honeywell climate platform.
-
-    Honeywell uses config flow for configuration now. If an entry exists in
-    configuration.yaml, the import flow will attempt to import it and create
-    a config entry.
-    """
-
-    if config["platform"] == "honeywell":
-        _LOGGER.warning(
-            "Loading honeywell via platform config is deprecated; The configuration"
-            " has been migrated to a config entry and can be safely removed"
-        )
-        # No config entry exists and configuration.yaml config exists, trigger the import flow.
-        if not hass.config_entries.async_entries(DOMAIN):
-            await hass.config_entries.flow.async_init(
-                DOMAIN, context={"source": SOURCE_IMPORT}, data=config
-            )
-
-
 class HoneywellUSThermostat(ClimateEntity):
     """Representation of a Honeywell US Thermostat."""
 
@@ -172,16 +110,16 @@ class HoneywellUSThermostat(ClimateEntity):
         self._attr_hvac_modes = list(self._hvac_mode_map)
 
         self._attr_supported_features = (
-            SUPPORT_PRESET_MODE
-            | SUPPORT_TARGET_TEMPERATURE
-            | SUPPORT_TARGET_TEMPERATURE_RANGE
+            ClimateEntityFeature.PRESET_MODE
+            | ClimateEntityFeature.TARGET_TEMPERATURE
+            | ClimateEntityFeature.TARGET_TEMPERATURE_RANGE
         )
 
         if device._data["canControlHumidification"]:
-            self._attr_supported_features |= SUPPORT_TARGET_HUMIDITY
+            self._attr_supported_features |= ClimateEntityFeature.TARGET_HUMIDITY
 
         if device.raw_ui_data["SwitchEmergencyHeatAllowed"]:
-            self._attr_supported_features |= SUPPORT_AUX_HEAT
+            self._attr_supported_features |= ClimateEntityFeature.AUX_HEAT
 
         if not device._data["hasFan"]:
             return
@@ -192,12 +130,12 @@ class HoneywellUSThermostat(ClimateEntity):
 
         self._attr_fan_modes = list(self._fan_mode_map)
 
-        self._attr_supported_features |= SUPPORT_FAN_MODE
+        self._attr_supported_features |= ClimateEntityFeature.FAN_MODE
 
     @property
     def extra_state_attributes(self) -> dict[str, Any]:
         """Return the device specific state attributes."""
-        data = {}
+        data: dict[str, Any] = {}
         data[ATTR_FAN_ACTION] = "running" if self._device.fan_running else "idle"
         data[ATTR_PERMANENT_HOLD] = self._is_permanent_hold()
         if self._device.raw_dr_data:
@@ -207,20 +145,20 @@ class HoneywellUSThermostat(ClimateEntity):
     @property
     def min_temp(self) -> float:
         """Return the minimum temperature."""
-        if self.hvac_mode in [HVAC_MODE_COOL, HVAC_MODE_HEAT_COOL]:
+        if self.hvac_mode in [HVACMode.COOL, HVACMode.HEAT_COOL]:
             return self._device.raw_ui_data["CoolLowerSetptLimit"]
-        if self.hvac_mode == HVAC_MODE_HEAT:
+        if self.hvac_mode == HVACMode.HEAT:
             return self._device.raw_ui_data["HeatLowerSetptLimit"]
-        return None
+        return DEFAULT_MIN_TEMP
 
     @property
     def max_temp(self) -> float:
         """Return the maximum temperature."""
-        if self.hvac_mode == HVAC_MODE_COOL:
+        if self.hvac_mode == HVACMode.COOL:
             return self._device.raw_ui_data["CoolUpperSetptLimit"]
-        if self.hvac_mode in [HVAC_MODE_HEAT, HVAC_MODE_HEAT_COOL]:
+        if self.hvac_mode in [HVACMode.HEAT, HVACMode.HEAT_COOL]:
             return self._device.raw_ui_data["HeatUpperSetptLimit"]
-        return None
+        return DEFAULT_MAX_TEMP
 
     @property
     def current_humidity(self) -> int | None:
@@ -228,14 +166,14 @@ class HoneywellUSThermostat(ClimateEntity):
         return self._device.current_humidity
 
     @property
-    def hvac_mode(self) -> str:
+    def hvac_mode(self) -> HVACMode:
         """Return hvac operation ie. heat, cool mode."""
         return HW_MODE_TO_HVAC_MODE[self._device.system_mode]
 
     @property
-    def hvac_action(self) -> str | None:
+    def hvac_action(self) -> HVACAction | None:
         """Return the current running hvac operation if supported."""
-        if self.hvac_mode == HVAC_MODE_OFF:
+        if self.hvac_mode == HVACMode.OFF:
             return None
         return HW_MODE_TO_HA_HVAC_ACTION[self._device.equipment_output_status]
 
@@ -247,23 +185,23 @@ class HoneywellUSThermostat(ClimateEntity):
     @property
     def target_temperature(self) -> float | None:
         """Return the temperature we try to reach."""
-        if self.hvac_mode == HVAC_MODE_COOL:
+        if self.hvac_mode == HVACMode.COOL:
             return self._device.setpoint_cool
-        if self.hvac_mode == HVAC_MODE_HEAT:
+        if self.hvac_mode == HVACMode.HEAT:
             return self._device.setpoint_heat
         return None
 
     @property
     def target_temperature_high(self) -> float | None:
         """Return the highbound target temperature we try to reach."""
-        if self.hvac_mode == HVAC_MODE_HEAT_COOL:
+        if self.hvac_mode == HVACMode.HEAT_COOL:
             return self._device.setpoint_cool
         return None
 
     @property
     def target_temperature_low(self) -> float | None:
         """Return the lowbound target temperature we try to reach."""
-        if self.hvac_mode == HVAC_MODE_HEAT_COOL:
+        if self.hvac_mode == HVACMode.HEAT_COOL:
             return self._device.setpoint_heat
         return None
 
@@ -295,7 +233,7 @@ class HoneywellUSThermostat(ClimateEntity):
             # Get current mode
             mode = self._device.system_mode
             # Set hold if this is not the case
-            if getattr(self._device, f"hold_{mode}") is False:
+            if getattr(self._device, f"hold_{mode}", None) is False:
                 # Get next period key
                 next_period_key = f"{mode.capitalize()}NextPeriod"
                 # Get next period raw value
@@ -311,11 +249,11 @@ class HoneywellUSThermostat(ClimateEntity):
 
     def set_temperature(self, **kwargs) -> None:
         """Set new target temperature."""
-        if {HVAC_MODE_COOL, HVAC_MODE_HEAT} & set(self._hvac_mode_map):
+        if {HVACMode.COOL, HVACMode.HEAT} & set(self._hvac_mode_map):
             self._set_temperature(**kwargs)
 
         try:
-            if HVAC_MODE_HEAT_COOL in self._hvac_mode_map:
+            if HVACMode.HEAT_COOL in self._hvac_mode_map:
                 if temperature := kwargs.get(ATTR_TARGET_TEMP_HIGH):
                     self._device.setpoint_cool = temperature
                 if temperature := kwargs.get(ATTR_TARGET_TEMP_LOW):
@@ -327,7 +265,7 @@ class HoneywellUSThermostat(ClimateEntity):
         """Set new target fan mode."""
         self._device.fan_mode = self._fan_mode_map[fan_mode]
 
-    def set_hvac_mode(self, hvac_mode: str) -> None:
+    def set_hvac_mode(self, hvac_mode: HVACMode) -> None:
         """Set new target hvac mode."""
         self._device.system_mode = self._hvac_mode_map[hvac_mode]
 
@@ -404,10 +342,10 @@ class HoneywellUSThermostat(ClimateEntity):
 
     def turn_aux_heat_off(self) -> None:
         """Turn auxiliary heater off."""
-        if HVAC_MODE_HEAT in self.hvac_modes:
-            self.set_hvac_mode(HVAC_MODE_HEAT)
+        if HVACMode.HEAT in self.hvac_modes:
+            self.set_hvac_mode(HVACMode.HEAT)
         else:
-            self.set_hvac_mode(HVAC_MODE_OFF)
+            self.set_hvac_mode(HVACMode.OFF)
 
     async def async_update(self):
         """Get the latest state from the service."""
