@@ -1,5 +1,6 @@
 """Support for hunter douglas shades."""
 import asyncio
+from contextlib import suppress
 import logging
 
 from aiopvapi.helpers.constants import ATTR_POSITION1, ATTR_POSITION_DATA
@@ -13,14 +14,13 @@ import async_timeout
 
 from homeassistant.components.cover import (
     ATTR_POSITION,
-    DEVICE_CLASS_SHADE,
-    SUPPORT_CLOSE,
-    SUPPORT_OPEN,
-    SUPPORT_SET_POSITION,
-    SUPPORT_STOP,
+    CoverDeviceClass,
     CoverEntity,
+    CoverEntityFeature,
 )
-from homeassistant.core import callback
+from homeassistant.config_entries import ConfigEntry
+from homeassistant.core import HomeAssistant, callback
+from homeassistant.helpers.entity_platform import AddEntitiesCallback
 from homeassistant.helpers.event import async_call_later
 
 from .const import (
@@ -43,10 +43,16 @@ _LOGGER = logging.getLogger(__name__)
 
 # Estimated time it takes to complete a transition
 # from one state to another
-TRANSITION_COMPLETE_DURATION = 30
+TRANSITION_COMPLETE_DURATION = 40
+
+PARALLEL_UPDATES = 1
+
+RESYNC_DELAY = 60
 
 
-async def async_setup_entry(hass, entry, async_add_entities):
+async def async_setup_entry(
+    hass: HomeAssistant, entry: ConfigEntry, async_add_entities: AddEntitiesCallback
+) -> None:
     """Set up the hunter douglas shades."""
 
     pv_data = hass.data[DOMAIN][entry.entry_id]
@@ -63,21 +69,21 @@ async def async_setup_entry(hass, entry, async_add_entities):
         # possible
         shade = PvShade(raw_shade, pv_request)
         name_before_refresh = shade.name
-        try:
+        with suppress(asyncio.TimeoutError):
             async with async_timeout.timeout(1):
                 await shade.refresh()
-        except asyncio.TimeoutError:
-            # Forced refresh is not required for setup
-            pass
+
         if ATTR_POSITION_DATA not in shade.raw_data:
             _LOGGER.info(
                 "The %s shade was skipped because it is missing position data",
                 name_before_refresh,
             )
             continue
+        room_id = shade.raw_data.get(ROOM_ID_IN_SHADE)
+        room_name = room_data.get(room_id, {}).get(ROOM_NAME_UNICODE, "")
         entities.append(
             PowerViewShade(
-                shade, name_before_refresh, room_data, coordinator, device_info
+                coordinator, device_info, room_name, shade, name_before_refresh
             )
         )
     async_add_entities(entities)
@@ -88,41 +94,40 @@ def hd_position_to_hass(hd_position):
     return round((hd_position / MAX_POSITION) * 100)
 
 
-def hass_position_to_hd(hass_positon):
+def hass_position_to_hd(hass_position):
     """Convert hass position to hunter douglas position."""
-    return int(hass_positon / 100 * MAX_POSITION)
+    return int(hass_position / 100 * MAX_POSITION)
 
 
 class PowerViewShade(ShadeEntity, CoverEntity):
     """Representation of a powerview shade."""
 
-    def __init__(self, shade, name, room_data, coordinator, device_info):
+    # The hub frequently reports stale states
+    _attr_assumed_state = True
+
+    def __init__(self, coordinator, device_info, room_name, shade, name):
         """Initialize the shade."""
-        room_id = shade.raw_data.get(ROOM_ID_IN_SHADE)
-        super().__init__(coordinator, device_info, shade, name)
+        super().__init__(coordinator, device_info, room_name, shade, name)
         self._shade = shade
-        self._device_info = device_info
         self._is_opening = False
         self._is_closing = False
         self._last_action_timestamp = 0
         self._scheduled_transition_update = None
-        self._room_name = room_data.get(room_id, {}).get(ROOM_NAME_UNICODE, "")
         self._current_cover_position = MIN_POSITION
-        self._coordinator = coordinator
+        self._attr_supported_features = (
+            CoverEntityFeature.OPEN
+            | CoverEntityFeature.CLOSE
+            | CoverEntityFeature.SET_POSITION
+        )
+        if self._device_info[DEVICE_MODEL] != LEGACY_DEVICE_MODEL:
+            self._attr_supported_features |= CoverEntityFeature.STOP
+        self._forced_resync = None
 
     @property
-    def device_state_attributes(self):
+    def extra_state_attributes(self):
         """Return the state attributes."""
         return {STATE_ATTRIBUTE_ROOM_NAME: self._room_name}
 
-    @property
-    def supported_features(self):
-        """Flag supported features."""
-        supported_features = SUPPORT_OPEN | SUPPORT_CLOSE | SUPPORT_SET_POSITION
-        if self._device_info[DEVICE_MODEL] != LEGACY_DEVICE_MODEL:
-            supported_features |= SUPPORT_STOP
-        return supported_features
-
     @property
     def is_closed(self):
         """Return if the cover is closed."""
@@ -146,7 +151,7 @@ class PowerViewShade(ShadeEntity, CoverEntity):
     @property
     def device_class(self):
         """Return device class."""
-        return DEVICE_CLASS_SHADE
+        return CoverDeviceClass.SHADE
 
     @property
     def name(self):
@@ -178,8 +183,6 @@ class PowerViewShade(ShadeEntity, CoverEntity):
         """Move the shade to a position."""
         current_hass_position = hd_position_to_hass(self._current_cover_position)
         steps_to_move = abs(current_hass_position - target_hass_position)
-        if not steps_to_move:
-            return
         self._async_schedule_update_for_transition(steps_to_move)
         self._async_update_from_command(
             await self._shade.move(
@@ -214,19 +217,21 @@ class PowerViewShade(ShadeEntity, CoverEntity):
     def _async_update_current_cover_position(self):
         """Update the current cover position from the data."""
         _LOGGER.debug("Raw data update: %s", self._shade.raw_data)
-        position_data = self._shade.raw_data[ATTR_POSITION_DATA]
+        position_data = self._shade.raw_data.get(ATTR_POSITION_DATA, {})
         if ATTR_POSITION1 in position_data:
-            self._current_cover_position = position_data[ATTR_POSITION1]
+            self._current_cover_position = int(position_data[ATTR_POSITION1])
         self._is_opening = False
         self._is_closing = False
 
     @callback
     def _async_cancel_scheduled_transition_update(self):
         """Cancel any previous updates."""
-        if not self._scheduled_transition_update:
-            return
-        self._scheduled_transition_update()
-        self._scheduled_transition_update = None
+        if self._scheduled_transition_update:
+            self._scheduled_transition_update()
+            self._scheduled_transition_update = None
+        if self._forced_resync:
+            self._forced_resync()
+            self._forced_resync = None
 
     @callback
     def _async_schedule_update_for_transition(self, steps):
@@ -259,6 +264,14 @@ class PowerViewShade(ShadeEntity, CoverEntity):
         _LOGGER.debug("Processing scheduled update for %s", self.name)
         self._scheduled_transition_update = None
         await self._async_force_refresh_state()
+        self._forced_resync = async_call_later(
+            self.hass, RESYNC_DELAY, self._async_force_resync
+        )
+
+    async def _async_force_resync(self, *_):
+        """Force a resync after an update since the hub may have stale state."""
+        self._forced_resync = None
+        await self._async_force_refresh_state()
 
     async def _async_force_refresh_state(self):
         """Refresh the cover state and force the device cache to be bypassed."""
@@ -270,15 +283,19 @@ class PowerViewShade(ShadeEntity, CoverEntity):
         """When entity is added to hass."""
         self._async_update_current_cover_position()
         self.async_on_remove(
-            self._coordinator.async_add_listener(self._async_update_shade_from_group)
+            self.coordinator.async_add_listener(self._async_update_shade_from_group)
         )
 
+    async def async_will_remove_from_hass(self):
+        """Cancel any pending refreshes."""
+        self._async_cancel_scheduled_transition_update()
+
     @callback
     def _async_update_shade_from_group(self):
         """Update with new data from the coordinator."""
-        if self._scheduled_transition_update:
+        if self._scheduled_transition_update or self._forced_resync:
             # If a transition in in progress
             # the data will be wrong
             return
-        self._async_process_new_shade_data(self._coordinator.data[self._shade.id])
+        self._async_process_new_shade_data(self.coordinator.data[self._shade.id])
         self.async_write_ha_state()
