@@ -1,15 +1,22 @@
 """Config flow for Universal Devices ISY994 integration."""
-import logging
-from urllib.parse import urlparse
+from __future__ import annotations
 
+import logging
+from typing import Any
+from urllib.parse import urlparse, urlunparse
+
+from aiohttp import CookieJar
+import async_timeout
+from pyisy import ISYConnectionError, ISYInvalidAuthError, ISYResponseParseError
 from pyisy.configuration import Configuration
 from pyisy.connection import Connection
 import voluptuous as vol
 
-from homeassistant import config_entries, core, exceptions
-from homeassistant.components import ssdp
+from homeassistant import config_entries, core, data_entry_flow, exceptions
+from homeassistant.components import dhcp, ssdp
 from homeassistant.const import CONF_HOST, CONF_NAME, CONF_PASSWORD, CONF_USERNAME
 from homeassistant.core import callback
+from homeassistant.helpers import aiohttp_client
 
 from .const import (
     CONF_IGNORE_STRING,
@@ -23,14 +30,18 @@ from .const import (
     DEFAULT_TLS_VERSION,
     DEFAULT_VAR_SENSOR_STRING,
     DOMAIN,
+    HTTP_PORT,
+    HTTPS_PORT,
     ISY_URL_POSTFIX,
+    SCHEME_HTTP,
+    SCHEME_HTTPS,
     UDN_UUID_PREFIX,
 )
 
 _LOGGER = logging.getLogger(__name__)
 
 
-def _data_schema(schema_input):
+def _data_schema(schema_input: dict[str, str]) -> vol.Schema:
     """Generate schema with defaults."""
     return vol.Schema(
         {
@@ -43,7 +54,9 @@ def _data_schema(schema_input):
     )
 
 
-async def validate_input(hass: core.HomeAssistant, data):
+async def validate_input(
+    hass: core.HomeAssistant, data: dict[str, Any]
+) -> dict[str, str]:
     """Validate the user input allows us to connect.
 
     Data has the keys from DATA_SCHEMA with values provided by the user.
@@ -53,28 +66,44 @@ async def validate_input(hass: core.HomeAssistant, data):
     host = urlparse(data[CONF_HOST])
     tls_version = data.get(CONF_TLS_VER)
 
-    if host.scheme == "http":
+    if host.scheme == SCHEME_HTTP:
         https = False
-        port = host.port or 80
-    elif host.scheme == "https":
+        port = host.port or HTTP_PORT
+        session = aiohttp_client.async_create_clientsession(
+            hass, verify_ssl=False, cookie_jar=CookieJar(unsafe=True)
+        )
+    elif host.scheme == SCHEME_HTTPS:
         https = True
-        port = host.port or 443
+        port = host.port or HTTPS_PORT
+        session = aiohttp_client.async_get_clientsession(hass)
     else:
         _LOGGER.error("The isy994 host value in configuration is invalid")
         raise InvalidHost
 
     # Connect to ISY controller.
-    isy_conf = await hass.async_add_executor_job(
-        _fetch_isy_configuration,
+    isy_conn = Connection(
         host.hostname,
         port,
         user,
         password,
-        https,
-        tls_version,
-        host.path,
+        use_https=https,
+        tls_ver=tls_version,
+        webroot=host.path,
+        websession=session,
     )
 
+    try:
+        async with async_timeout.timeout(30):
+            isy_conf_xml = await isy_conn.test_connection()
+    except ISYInvalidAuthError as error:
+        raise InvalidAuth from error
+    except ISYConnectionError as error:
+        raise CannotConnect from error
+
+    try:
+        isy_conf = Configuration(xml=isy_conf_xml)
+    except ISYResponseParseError as error:
+        raise CannotConnect from error
     if not isy_conf or "name" not in isy_conf or not isy_conf["name"]:
         raise CannotConnect
 
@@ -82,46 +111,29 @@ async def validate_input(hass: core.HomeAssistant, data):
     return {"title": f"{isy_conf['name']} ({host.hostname})", "uuid": isy_conf["uuid"]}
 
 
-def _fetch_isy_configuration(
-    address, port, username, password, use_https, tls_ver, webroot
-):
-    """Validate and fetch the configuration from the ISY."""
-    try:
-        isy_conn = Connection(
-            address,
-            port,
-            username,
-            password,
-            use_https,
-            tls_ver,
-            webroot=webroot,
-        )
-    except ValueError as err:
-        raise InvalidAuth(err.args[0]) from err
-
-    return Configuration(xml=isy_conn.get_config())
-
-
 class ConfigFlow(config_entries.ConfigFlow, domain=DOMAIN):
     """Handle a config flow for Universal Devices ISY994."""
 
     VERSION = 1
-    CONNECTION_CLASS = config_entries.CONN_CLASS_LOCAL_PUSH
 
-    def __init__(self):
+    def __init__(self) -> None:
         """Initialize the isy994 config flow."""
-        self.discovered_conf = {}
+        self.discovered_conf: dict[str, str] = {}
 
     @staticmethod
     @callback
-    def async_get_options_flow(config_entry):
+    def async_get_options_flow(
+        config_entry: config_entries.ConfigEntry,
+    ) -> config_entries.OptionsFlow:
         """Get the options flow for this handler."""
         return OptionsFlowHandler(config_entry)
 
-    async def async_step_user(self, user_input=None):
+    async def async_step_user(
+        self, user_input: dict[str, Any] | None = None
+    ) -> data_entry_flow.FlowResult:
         """Handle the initial step."""
         errors = {}
-        info = None
+        info: dict[str, str] = {}
         if user_input is not None:
             try:
                 info = await validate_input(self.hass, user_input)
@@ -146,22 +158,91 @@ class ConfigFlow(config_entries.ConfigFlow, domain=DOMAIN):
             errors=errors,
         )
 
-    async def async_step_import(self, user_input):
+    async def async_step_import(
+        self, user_input: dict[str, Any]
+    ) -> data_entry_flow.FlowResult:
         """Handle import."""
         return await self.async_step_user(user_input)
 
-    async def async_step_ssdp(self, discovery_info):
+    async def _async_set_unique_id_or_update(
+        self, isy_mac: str, ip_address: str, port: int | None
+    ) -> None:
+        """Abort and update the ip address on change."""
+        existing_entry = await self.async_set_unique_id(isy_mac)
+        if not existing_entry:
+            return
+        if existing_entry.source == config_entries.SOURCE_IGNORE:
+            raise data_entry_flow.AbortFlow("already_configured")
+        parsed_url = urlparse(existing_entry.data[CONF_HOST])
+        if parsed_url.hostname != ip_address:
+            new_netloc = ip_address
+            if port:
+                new_netloc = f"{ip_address}:{port}"
+            elif parsed_url.port:
+                new_netloc = f"{ip_address}:{parsed_url.port}"
+            self.hass.config_entries.async_update_entry(
+                existing_entry,
+                data={
+                    **existing_entry.data,
+                    CONF_HOST: urlunparse(
+                        (
+                            parsed_url.scheme,
+                            new_netloc,
+                            parsed_url.path,
+                            parsed_url.query,
+                            parsed_url.fragment,
+                            None,
+                        )
+                    ),
+                },
+            )
+        raise data_entry_flow.AbortFlow("already_configured")
+
+    async def async_step_dhcp(
+        self, discovery_info: dhcp.DhcpServiceInfo
+    ) -> data_entry_flow.FlowResult:
+        """Handle a discovered isy994 via dhcp."""
+        friendly_name = discovery_info.hostname
+        if friendly_name.startswith("polisy"):
+            url = f"http://{discovery_info.ip}:8080"
+        else:
+            url = f"http://{discovery_info.ip}"
+        mac = discovery_info.macaddress
+        isy_mac = (
+            f"{mac[0:2]}:{mac[2:4]}:{mac[4:6]}:{mac[6:8]}:{mac[8:10]}:{mac[10:12]}"
+        )
+        await self._async_set_unique_id_or_update(isy_mac, discovery_info.ip, None)
+
+        self.discovered_conf = {
+            CONF_NAME: friendly_name,
+            CONF_HOST: url,
+        }
+
+        self.context["title_placeholders"] = self.discovered_conf
+        return await self.async_step_user()
+
+    async def async_step_ssdp(
+        self, discovery_info: ssdp.SsdpServiceInfo
+    ) -> data_entry_flow.FlowResult:
         """Handle a discovered isy994."""
-        friendly_name = discovery_info[ssdp.ATTR_UPNP_FRIENDLY_NAME]
-        url = discovery_info[ssdp.ATTR_SSDP_LOCATION]
-        mac = discovery_info[ssdp.ATTR_UPNP_UDN]
+        friendly_name = discovery_info.upnp[ssdp.ATTR_UPNP_FRIENDLY_NAME]
+        url = discovery_info.ssdp_location
+        assert isinstance(url, str)
+        parsed_url = urlparse(url)
+        mac = discovery_info.upnp[ssdp.ATTR_UPNP_UDN]
         if mac.startswith(UDN_UUID_PREFIX):
             mac = mac[len(UDN_UUID_PREFIX) :]
         if url.endswith(ISY_URL_POSTFIX):
             url = url[: -len(ISY_URL_POSTFIX)]
 
-        await self.async_set_unique_id(mac)
-        self._abort_if_unique_id_configured()
+        port = HTTP_PORT
+        if parsed_url.port:
+            port = parsed_url.port
+        elif parsed_url.scheme == SCHEME_HTTPS:
+            port = HTTPS_PORT
+
+        assert isinstance(parsed_url.hostname, str)
+        await self._async_set_unique_id_or_update(mac, parsed_url.hostname, port)
 
         self.discovered_conf = {
             CONF_NAME: friendly_name,
@@ -175,11 +256,13 @@ class ConfigFlow(config_entries.ConfigFlow, domain=DOMAIN):
 class OptionsFlowHandler(config_entries.OptionsFlow):
     """Handle a option flow for isy994."""
 
-    def __init__(self, config_entry: config_entries.ConfigEntry):
+    def __init__(self, config_entry: config_entries.ConfigEntry) -> None:
         """Initialize options flow."""
         self.config_entry = config_entry
 
-    async def async_step_init(self, user_input=None):
+    async def async_step_init(
+        self, user_input: dict[str, Any] | None = None
+    ) -> data_entry_flow.FlowResult:
         """Handle options flow."""
         if user_input is not None:
             return self.async_create_entry(title="", data=user_input)
