@@ -1,14 +1,21 @@
 """Support for Lutron Caseta lights."""
+from datetime import timedelta
 import logging
 
 from homeassistant.components.light import (
     ATTR_BRIGHTNESS,
+    ATTR_TRANSITION,
     DOMAIN,
-    SUPPORT_BRIGHTNESS,
+    ColorMode,
     LightEntity,
+    LightEntityFeature,
 )
+from homeassistant.config_entries import ConfigEntry
+from homeassistant.core import HomeAssistant
+from homeassistant.helpers.entity_platform import AddEntitiesCallback
 
-from . import DOMAIN as CASETA_DOMAIN, LutronCasetaDevice
+from . import LutronCasetaDevice
+from .const import BRIDGE_DEVICE, BRIDGE_LEAP, DOMAIN as CASETA_DOMAIN
 
 _LOGGER = logging.getLogger(__name__)
 
@@ -23,19 +30,24 @@ def to_hass_level(level):
     return int((level * 255) // 100)
 
 
-async def async_setup_entry(hass, config_entry, async_add_entities):
+async def async_setup_entry(
+    hass: HomeAssistant,
+    config_entry: ConfigEntry,
+    async_add_entities: AddEntitiesCallback,
+) -> None:
     """Set up the Lutron Caseta light platform.
 
     Adds dimmers from the Caseta bridge associated with the config_entry as
     light entities.
     """
-
     entities = []
-    bridge = hass.data[CASETA_DOMAIN][config_entry.entry_id]
+    data = hass.data[CASETA_DOMAIN][config_entry.entry_id]
+    bridge = data[BRIDGE_LEAP]
+    bridge_device = data[BRIDGE_DEVICE]
     light_devices = bridge.get_devices_by_domain(DOMAIN)
 
     for light_device in light_devices:
-        entity = LutronCasetaLight(light_device, bridge)
+        entity = LutronCasetaLight(light_device, bridge, bridge_device)
         entities.append(entity)
 
     async_add_entities(entities, True)
@@ -44,24 +56,33 @@ async def async_setup_entry(hass, config_entry, async_add_entities):
 class LutronCasetaLight(LutronCasetaDevice, LightEntity):
     """Representation of a Lutron Light, including dimmable."""
 
-    @property
-    def supported_features(self):
-        """Flag supported features."""
-        return SUPPORT_BRIGHTNESS
+    _attr_color_mode = ColorMode.BRIGHTNESS
+    _attr_supported_color_modes = {ColorMode.BRIGHTNESS}
+    _attr_supported_features = LightEntityFeature.TRANSITION
 
     @property
     def brightness(self):
         """Return the brightness of the light."""
         return to_hass_level(self._device["current_state"])
 
+    async def _set_brightness(self, brightness, **kwargs):
+        args = {}
+        if ATTR_TRANSITION in kwargs:
+            args["fade_time"] = timedelta(seconds=kwargs[ATTR_TRANSITION])
+
+        await self._smartbridge.set_value(
+            self.device_id, to_lutron_level(brightness), **args
+        )
+
     async def async_turn_on(self, **kwargs):
         """Turn the light on."""
-        brightness = kwargs.get(ATTR_BRIGHTNESS, 255)
-        self._smartbridge.set_value(self.device_id, to_lutron_level(brightness))
+        brightness = kwargs.pop(ATTR_BRIGHTNESS, 255)
+
+        await self._set_brightness(brightness, **kwargs)
 
     async def async_turn_off(self, **kwargs):
         """Turn the light off."""
-        self._smartbridge.set_value(self.device_id, 0)
+        await self._set_brightness(0, **kwargs)
 
     @property
     def is_on(self):
