@@ -18,12 +18,6 @@ from homeassistant.components.light import (
     ATTR_TRANSITION,
     ATTR_WHITE_VALUE,
     ATTR_XY_COLOR,
-    COLOR_MODE_COLOR_TEMP,
-    COLOR_MODE_HS,
-    COLOR_MODE_RGB,
-    COLOR_MODE_RGBW,
-    COLOR_MODE_RGBWW,
-    COLOR_MODE_XY,
     ENTITY_ID_FORMAT,
     FLASH_LONG,
     FLASH_SHORT,
@@ -35,6 +29,7 @@ from homeassistant.components.light import (
     SUPPORT_TRANSITION,
     SUPPORT_WHITE_VALUE,
     VALID_COLOR_MODES,
+    ColorMode,
     LightEntity,
     legacy_supported_features,
     valid_supported_color_modes,
@@ -267,39 +262,39 @@ class MqttLightJson(MqttEntity, LightEntity, RestoreEntity):
                 _LOGGER.warning("Invalid color mode received")
                 return
             try:
-                if color_mode == COLOR_MODE_COLOR_TEMP:
+                if color_mode == ColorMode.COLOR_TEMP:
                     self._color_temp = int(values["color_temp"])
-                    self._color_mode = COLOR_MODE_COLOR_TEMP
-                elif color_mode == COLOR_MODE_HS:
+                    self._color_mode = ColorMode.COLOR_TEMP
+                elif color_mode == ColorMode.HS:
                     hue = float(values["color"]["h"])
                     saturation = float(values["color"]["s"])
-                    self._color_mode = COLOR_MODE_HS
+                    self._color_mode = ColorMode.HS
                     self._hs = (hue, saturation)
-                elif color_mode == COLOR_MODE_RGB:
+                elif color_mode == ColorMode.RGB:
                     r = int(values["color"]["r"])  # pylint: disable=invalid-name
                     g = int(values["color"]["g"])  # pylint: disable=invalid-name
                     b = int(values["color"]["b"])  # pylint: disable=invalid-name
-                    self._color_mode = COLOR_MODE_RGB
+                    self._color_mode = ColorMode.RGB
                     self._rgb = (r, g, b)
-                elif color_mode == COLOR_MODE_RGBW:
+                elif color_mode == ColorMode.RGBW:
                     r = int(values["color"]["r"])  # pylint: disable=invalid-name
                     g = int(values["color"]["g"])  # pylint: disable=invalid-name
                     b = int(values["color"]["b"])  # pylint: disable=invalid-name
                     w = int(values["color"]["w"])  # pylint: disable=invalid-name
-                    self._color_mode = COLOR_MODE_RGBW
+                    self._color_mode = ColorMode.RGBW
                     self._rgbw = (r, g, b, w)
-                elif color_mode == COLOR_MODE_RGBWW:
+                elif color_mode == ColorMode.RGBWW:
                     r = int(values["color"]["r"])  # pylint: disable=invalid-name
                     g = int(values["color"]["g"])  # pylint: disable=invalid-name
                     b = int(values["color"]["b"])  # pylint: disable=invalid-name
                     c = int(values["color"]["c"])  # pylint: disable=invalid-name
                     w = int(values["color"]["w"])  # pylint: disable=invalid-name
-                    self._color_mode = COLOR_MODE_RGBWW
+                    self._color_mode = ColorMode.RGBWW
                     self._rgbww = (r, g, b, c, w)
-                elif color_mode == COLOR_MODE_XY:
+                elif color_mode == ColorMode.XY:
                     x = float(values["color"]["x"])  # pylint: disable=invalid-name
                     y = float(values["color"]["y"])  # pylint: disable=invalid-name
-                    self._color_mode = COLOR_MODE_XY
+                    self._color_mode = ColorMode.XY
                     self._xy = (x, y)
             except (KeyError, ValueError):
                 _LOGGER.warning("Invalid or incomplete color value received")
@@ -553,31 +548,31 @@ class MqttLightJson(MqttEntity, LightEntity, RestoreEntity):
                 self._hs = kwargs[ATTR_HS_COLOR]
                 should_update = True
 
-        if ATTR_HS_COLOR in kwargs and self._supports_color_mode(COLOR_MODE_HS):
+        if ATTR_HS_COLOR in kwargs and self._supports_color_mode(ColorMode.HS):
             hs_color = kwargs[ATTR_HS_COLOR]
             message["color"] = {"h": hs_color[0], "s": hs_color[1]}
             if self._optimistic:
-                self._color_mode = COLOR_MODE_HS
+                self._color_mode = ColorMode.HS
                 self._hs = hs_color
                 should_update = True
 
-        if ATTR_RGB_COLOR in kwargs and self._supports_color_mode(COLOR_MODE_RGB):
+        if ATTR_RGB_COLOR in kwargs and self._supports_color_mode(ColorMode.RGB):
             rgb = self._scale_rgbxx(kwargs[ATTR_RGB_COLOR], kwargs)
             message["color"] = {"r": rgb[0], "g": rgb[1], "b": rgb[2]}
             if self._optimistic:
-                self._color_mode = COLOR_MODE_RGB
+                self._color_mode = ColorMode.RGB
                 self._rgb = rgb
                 should_update = True
 
-        if ATTR_RGBW_COLOR in kwargs and self._supports_color_mode(COLOR_MODE_RGBW):
+        if ATTR_RGBW_COLOR in kwargs and self._supports_color_mode(ColorMode.RGBW):
             rgb = self._scale_rgbxx(kwargs[ATTR_RGBW_COLOR], kwargs)
             message["color"] = {"r": rgb[0], "g": rgb[1], "b": rgb[2], "w": rgb[3]}
             if self._optimistic:
-                self._color_mode = COLOR_MODE_RGBW
+                self._color_mode = ColorMode.RGBW
                 self._rgbw = rgb
                 should_update = True
 
-        if ATTR_RGBWW_COLOR in kwargs and self._supports_color_mode(COLOR_MODE_RGBWW):
+        if ATTR_RGBWW_COLOR in kwargs and self._supports_color_mode(ColorMode.RGBWW):
             rgb = self._scale_rgbxx(kwargs[ATTR_RGBWW_COLOR], kwargs)
             message["color"] = {
                 "r": rgb[0],
@@ -587,15 +582,15 @@ class MqttLightJson(MqttEntity, LightEntity, RestoreEntity):
                 "w": rgb[4],
             }
             if self._optimistic:
-                self._color_mode = COLOR_MODE_RGBWW
+                self._color_mode = ColorMode.RGBWW
                 self._rgbww = rgb
                 should_update = True
 
-        if ATTR_XY_COLOR in kwargs and self._supports_color_mode(COLOR_MODE_XY):
+        if ATTR_XY_COLOR in kwargs and self._supports_color_mode(ColorMode.XY):
             xy = kwargs[ATTR_XY_COLOR]  # pylint: disable=invalid-name
             message["color"] = {"x": xy[0], "y": xy[1]}
             if self._optimistic:
-                self._color_mode = COLOR_MODE_XY
+                self._color_mode = ColorMode.XY
                 self._xy = xy
                 should_update = True
 
