@@ -5,14 +5,12 @@ LOGGER = logging.getLogger(__package__)
 
 DOMAIN = "onvif"
 
-DEFAULT_NAME = "ONVIF Camera"
-DEFAULT_PORT = 5000
-DEFAULT_USERNAME = "admin"
-DEFAULT_PASSWORD = "888888"
+DEFAULT_PORT = 80
 DEFAULT_ARGUMENTS = "-pred 1"
 
 CONF_DEVICE_ID = "deviceid"
 CONF_RTSP_TRANSPORT = "rtsp_transport"
+CONF_SNAPSHOT_AUTH = "snapshot_auth"
 
 RTSP_TRANS_PROTOCOLS = ["tcp", "udp", "udp_multicast", "http"]
 
@@ -38,5 +36,6 @@ CONTINUOUS_MOVE = "ContinuousMove"
 RELATIVE_MOVE = "RelativeMove"
 ABSOLUTE_MOVE = "AbsoluteMove"
 GOTOPRESET_MOVE = "GotoPreset"
+STOP_MOVE = "Stop"
 
 SERVICE_PTZ = "ptz"
