@@ -1,20 +1,19 @@
 """Support for OpenTherm Gateway climate devices."""
+from __future__ import annotations
+
 import logging
 
 from pyotgw import vars as gw_vars
 
 from homeassistant.components.climate import ENTITY_ID_FORMAT, ClimateEntity
 from homeassistant.components.climate.const import (
-    CURRENT_HVAC_COOL,
-    CURRENT_HVAC_HEAT,
-    CURRENT_HVAC_IDLE,
-    HVAC_MODE_COOL,
-    HVAC_MODE_HEAT,
     PRESET_AWAY,
     PRESET_NONE,
-    SUPPORT_PRESET_MODE,
-    SUPPORT_TARGET_TEMPERATURE,
+    ClimateEntityFeature,
+    HVACAction,
+    HVACMode,
 )
+from homeassistant.config_entries import ConfigEntry
 from homeassistant.const import (
     ATTR_TEMPERATURE,
     CONF_ID,
@@ -23,9 +22,10 @@ from homeassistant.const import (
     PRECISION_WHOLE,
     TEMP_CELSIUS,
 )
-from homeassistant.core import callback
+from homeassistant.core import HomeAssistant, callback
 from homeassistant.helpers.dispatcher import async_dispatcher_connect
-from homeassistant.helpers.entity import async_generate_entity_id
+from homeassistant.helpers.entity import DeviceInfo, async_generate_entity_id
+from homeassistant.helpers.entity_platform import AddEntitiesCallback
 
 from . import DOMAIN
 from .const import (
@@ -41,10 +41,12 @@ _LOGGER = logging.getLogger(__name__)
 
 DEFAULT_FLOOR_TEMP = False
 
-SUPPORT_FLAGS = SUPPORT_TARGET_TEMPERATURE | SUPPORT_PRESET_MODE
 
-
-async def async_setup_entry(hass, config_entry, async_add_entities):
+async def async_setup_entry(
+    hass: HomeAssistant,
+    config_entry: ConfigEntry,
+    async_add_entities: AddEntitiesCallback,
+) -> None:
     """Set up an OpenTherm Gateway climate entity."""
     ents = []
     ents.append(
@@ -60,6 +62,10 @@ async def async_setup_entry(hass, config_entry, async_add_entities):
 class OpenThermClimate(ClimateEntity):
     """Representation of a climate device."""
 
+    _attr_supported_features = (
+        ClimateEntityFeature.TARGET_TEMPERATURE | ClimateEntityFeature.PRESET_MODE
+    )
+
     def __init__(self, gw_dev, options):
         """Initialize the device."""
         self._gateway = gw_dev
@@ -72,9 +78,9 @@ class OpenThermClimate(ClimateEntity):
         self.temp_set_precision = options.get(CONF_SET_PRECISION)
         self.temporary_ovrd_mode = options.get(CONF_TEMPORARY_OVRD_MODE, True)
         self._available = False
-        self._current_operation = None
+        self._current_operation: HVACAction | None = None
         self._current_temperature = None
-        self._hvac_mode = HVAC_MODE_HEAT
+        self._hvac_mode = HVACMode.HEAT
         self._new_target_temperature = None
         self._target_temperature = None
         self._away_mode_a = None
@@ -117,13 +123,13 @@ class OpenThermClimate(ClimateEntity):
         flame_on = status[gw_vars.BOILER].get(gw_vars.DATA_SLAVE_FLAME_ON)
         cooling_active = status[gw_vars.BOILER].get(gw_vars.DATA_SLAVE_COOLING_ACTIVE)
         if ch_active and flame_on:
-            self._current_operation = CURRENT_HVAC_HEAT
-            self._hvac_mode = HVAC_MODE_HEAT
+            self._current_operation = HVACAction.HEATING
+            self._hvac_mode = HVACMode.HEAT
         elif cooling_active:
-            self._current_operation = CURRENT_HVAC_COOL
-            self._hvac_mode = HVAC_MODE_COOL
+            self._current_operation = HVACAction.COOLING
+            self._hvac_mode = HVACMode.COOL
         else:
-            self._current_operation = CURRENT_HVAC_IDLE
+            self._current_operation = HVACAction.IDLE
 
         self._current_temperature = status[gw_vars.THERMOSTAT].get(
             gw_vars.DATA_ROOM_TEMP
@@ -171,15 +177,15 @@ class OpenThermClimate(ClimateEntity):
         return self.friendly_name
 
     @property
-    def device_info(self):
+    def device_info(self) -> DeviceInfo:
         """Return device info."""
-        return {
-            "identifiers": {(DOMAIN, self._gateway.gw_id)},
-            "name": self._gateway.name,
-            "manufacturer": "Schelte Bron",
-            "model": "OpenTherm Gateway",
-            "sw_version": self._gateway.gw_version,
-        }
+        return DeviceInfo(
+            identifiers={(DOMAIN, self._gateway.gw_id)},
+            manufacturer="Schelte Bron",
+            model="OpenTherm Gateway",
+            name=self._gateway.name,
+            sw_version=self._gateway.gw_version,
+        )
 
     @property
     def unique_id(self):
@@ -206,21 +212,21 @@ class OpenThermClimate(ClimateEntity):
         return TEMP_CELSIUS
 
     @property
-    def hvac_action(self):
+    def hvac_action(self) -> HVACAction | None:
         """Return current HVAC operation."""
         return self._current_operation
 
     @property
-    def hvac_mode(self):
+    def hvac_mode(self) -> HVACMode:
         """Return current HVAC mode."""
         return self._hvac_mode
 
     @property
-    def hvac_modes(self):
+    def hvac_modes(self) -> list[HVACMode]:
         """Return available HVAC modes."""
         return []
 
-    def set_hvac_mode(self, hvac_mode):
+    def set_hvac_mode(self, hvac_mode: HVACMode) -> None:
         """Set the HVAC mode."""
         _LOGGER.warning("Changing HVAC mode is not supported")
 
@@ -278,11 +284,6 @@ class OpenThermClimate(ClimateEntity):
             )
             self.async_write_ha_state()
 
-    @property
-    def supported_features(self):
-        """Return the list of supported features."""
-        return SUPPORT_FLAGS
-
     @property
     def min_temp(self):
         """Return the minimum temperature."""
