@@ -2,12 +2,12 @@
 from __future__ import annotations
 
 from collections import defaultdict
-from collections.abc import Iterable, MutableMapping
+from collections.abc import Iterable, Iterator, MutableMapping
 from datetime import datetime
 from itertools import groupby
 import logging
 import time
-from typing import Any
+from typing import Any, cast
 
 from sqlalchemy import Column, Text, and_, bindparam, func, or_
 from sqlalchemy.ext import baked
@@ -23,6 +23,7 @@ from .models import (
     RecorderRuns,
     StateAttributes,
     States,
+    process_timestamp,
     process_timestamp_to_utc_isoformat,
 )
 from .util import execute, session_scope
@@ -58,11 +59,22 @@ BASE_STATES = [
     States.last_changed,
     States.last_updated,
 ]
+BASE_STATES_NO_LAST_UPDATED = [
+    States.entity_id,
+    States.state,
+    States.last_changed,
+    literal(value=None, type_=Text).label("last_updated"),
+]
 QUERY_STATE_NO_ATTR = [
     *BASE_STATES,
     literal(value=None, type_=Text).label("attributes"),
     literal(value=None, type_=Text).label("shared_attrs"),
 ]
+QUERY_STATE_NO_ATTR_NO_LAST_UPDATED = [
+    *BASE_STATES_NO_LAST_UPDATED,
+    literal(value=None, type_=Text).label("attributes"),
+    literal(value=None, type_=Text).label("shared_attrs"),
+]
 # Remove QUERY_STATES_PRE_SCHEMA_25
 # and the migration_in_progress check
 # once schema 26 is created
@@ -71,12 +83,23 @@ QUERY_STATES_PRE_SCHEMA_25 = [
     States.attributes,
     literal(value=None, type_=Text).label("shared_attrs"),
 ]
+QUERY_STATES_PRE_SCHEMA_25_NO_LAST_UPDATED = [
+    *BASE_STATES_NO_LAST_UPDATED,
+    States.attributes,
+    literal(value=None, type_=Text).label("shared_attrs"),
+]
 QUERY_STATES = [
     *BASE_STATES,
     # Remove States.attributes once all attributes are in StateAttributes.shared_attrs
     States.attributes,
     StateAttributes.shared_attrs,
 ]
+QUERY_STATES_NO_LAST_UPDATED = [
+    *BASE_STATES_NO_LAST_UPDATED,
+    # Remove States.attributes once all attributes are in StateAttributes.shared_attrs
+    States.attributes,
+    StateAttributes.shared_attrs,
+]
 
 HISTORY_BAKERY = "recorder_history_bakery"
 
@@ -93,7 +116,7 @@ def query_and_join_attributes(
     # If we in the process of migrating schema we do
     # not want to join the state_attributes table as we
     # do not know if it will be there yet
-    if recorder.get_instance(hass).migration_in_progress:
+    if recorder.get_instance(hass).schema_version < 25:
         return QUERY_STATES_PRE_SCHEMA_25, False
     # Finally if no migration is in progress and no_attributes
     # was not requested, we query both attributes columns and
@@ -102,7 +125,7 @@ def query_and_join_attributes(
 
 
 def bake_query_and_join_attributes(
-    hass: HomeAssistant, no_attributes: bool
+    hass: HomeAssistant, no_attributes: bool, include_last_updated: bool = True
 ) -> tuple[Any, bool]:
     """Return the initial backed query and if StateAttributes should be joined.
 
@@ -114,16 +137,35 @@ def bake_query_and_join_attributes(
     # without the attributes fields and do not join the
     # state_attributes table
     if no_attributes:
-        return bakery(lambda session: session.query(*QUERY_STATE_NO_ATTR)), False
+        if include_last_updated:
+            return bakery(lambda session: session.query(*QUERY_STATE_NO_ATTR)), False
+        return (
+            bakery(lambda session: session.query(*QUERY_STATE_NO_ATTR_NO_LAST_UPDATED)),
+            False,
+        )
     # If we in the process of migrating schema we do
     # not want to join the state_attributes table as we
     # do not know if it will be there yet
-    if recorder.get_instance(hass).migration_in_progress:
-        return bakery(lambda session: session.query(*QUERY_STATES_PRE_SCHEMA_25)), False
+    if recorder.get_instance(hass).schema_version < 25:
+        if include_last_updated:
+            return (
+                bakery(lambda session: session.query(*QUERY_STATES_PRE_SCHEMA_25)),
+                False,
+            )
+        return (
+            bakery(
+                lambda session: session.query(
+                    *QUERY_STATES_PRE_SCHEMA_25_NO_LAST_UPDATED
+                )
+            ),
+            False,
+        )
     # Finally if no migration is in progress and no_attributes
     # was not requested, we query both attributes columns and
     # join state_attributes
-    return bakery(lambda session: session.query(*QUERY_STATES)), True
+    if include_last_updated:
+        return bakery(lambda session: session.query(*QUERY_STATES)), True
+    return bakery(lambda session: session.query(*QUERY_STATES_NO_LAST_UPDATED)), True
 
 
 def async_setup(hass: HomeAssistant) -> None:
@@ -141,7 +183,7 @@ def get_significant_states(
     significant_changes_only: bool = True,
     minimal_response: bool = False,
     no_attributes: bool = False,
-) -> MutableMapping[str, Iterable[LazyState | State | dict[str, Any]]]:
+) -> MutableMapping[str, list[State | dict[str, Any]]]:
     """Wrap get_significant_states_with_session with an sql session."""
     with session_scope(hass=hass) as session:
         return get_significant_states_with_session(
@@ -158,31 +200,20 @@ def get_significant_states(
         )
 
 
-def get_significant_states_with_session(
+def _query_significant_states_with_session(
     hass: HomeAssistant,
     session: Session,
     start_time: datetime,
     end_time: datetime | None = None,
     entity_ids: list[str] | None = None,
     filters: Any = None,
-    include_start_time_state: bool = True,
     significant_changes_only: bool = True,
-    minimal_response: bool = False,
     no_attributes: bool = False,
-) -> MutableMapping[str, Iterable[LazyState | State | dict[str, Any]]]:
-    """
-    Return states changes during UTC period start_time - end_time.
+) -> list[States]:
+    """Query the database for significant state changes."""
+    if _LOGGER.isEnabledFor(logging.DEBUG):
+        timer_start = time.perf_counter()
 
-    entity_ids is an optional iterable of entities to include in the results.
-
-    filters is an optional SQLAlchemy filter which will be applied to the database
-    queries unless entity_ids is given, in which case its ignored.
-
-    Significant states are all states where there is a state change,
-    as well as all states from certain domains (for instance
-    thermostat so that we get current temperature in our graphs).
-    """
-    timer_start = time.perf_counter()
     baked_query, join_attributes = bake_query_and_join_attributes(hass, no_attributes)
 
     if entity_ids is not None and len(entity_ids) == 1:
@@ -190,6 +221,9 @@ def get_significant_states_with_session(
             significant_changes_only
             and split_entity_id(entity_ids[0])[0] not in SIGNIFICANT_DOMAINS
         ):
+            baked_query, join_attributes = bake_query_and_join_attributes(
+                hass, no_attributes, include_last_updated=False
+            )
             baked_query += lambda q: q.filter(
                 States.last_changed == States.last_updated
             )
@@ -240,6 +274,43 @@ def get_significant_states_with_session(
         elapsed = time.perf_counter() - timer_start
         _LOGGER.debug("get_significant_states took %fs", elapsed)
 
+    return states
+
+
+def get_significant_states_with_session(
+    hass: HomeAssistant,
+    session: Session,
+    start_time: datetime,
+    end_time: datetime | None = None,
+    entity_ids: list[str] | None = None,
+    filters: Any = None,
+    include_start_time_state: bool = True,
+    significant_changes_only: bool = True,
+    minimal_response: bool = False,
+    no_attributes: bool = False,
+) -> MutableMapping[str, list[State | dict[str, Any]]]:
+    """
+    Return states changes during UTC period start_time - end_time.
+
+    entity_ids is an optional iterable of entities to include in the results.
+
+    filters is an optional SQLAlchemy filter which will be applied to the database
+    queries unless entity_ids is given, in which case its ignored.
+
+    Significant states are all states where there is a state change,
+    as well as all states from certain domains (for instance
+    thermostat so that we get current temperature in our graphs).
+    """
+    states = _query_significant_states_with_session(
+        hass,
+        session,
+        start_time,
+        end_time,
+        entity_ids,
+        filters,
+        significant_changes_only,
+        no_attributes,
+    )
     return _sorted_states_to_dict(
         hass,
         session,
@@ -253,6 +324,35 @@ def get_significant_states_with_session(
     )
 
 
+def get_full_significant_states_with_session(
+    hass: HomeAssistant,
+    session: Session,
+    start_time: datetime,
+    end_time: datetime | None = None,
+    entity_ids: list[str] | None = None,
+    filters: Any = None,
+    include_start_time_state: bool = True,
+    significant_changes_only: bool = True,
+    no_attributes: bool = False,
+) -> MutableMapping[str, list[State]]:
+    """Variant of get_significant_states_with_session that does not return minimal responses."""
+    return cast(
+        MutableMapping[str, list[State]],
+        get_significant_states_with_session(
+            hass=hass,
+            session=session,
+            start_time=start_time,
+            end_time=end_time,
+            entity_ids=entity_ids,
+            filters=filters,
+            include_start_time_state=include_start_time_state,
+            significant_changes_only=significant_changes_only,
+            minimal_response=False,
+            no_attributes=no_attributes,
+        ),
+    )
+
+
 def state_changes_during_period(
     hass: HomeAssistant,
     start_time: datetime,
@@ -262,11 +362,11 @@ def state_changes_during_period(
     descending: bool = False,
     limit: int | None = None,
     include_start_time_state: bool = True,
-) -> MutableMapping[str, Iterable[LazyState | State | dict[str, Any]]]:
+) -> MutableMapping[str, list[State]]:
     """Return states changes during UTC period start_time - end_time."""
     with session_scope(hass=hass) as session:
         baked_query, join_attributes = bake_query_and_join_attributes(
-            hass, no_attributes
+            hass, no_attributes, include_last_updated=False
         )
 
         baked_query += lambda q: q.filter(
@@ -288,38 +388,50 @@ def state_changes_during_period(
                 StateAttributes, States.attributes_id == StateAttributes.attributes_id
             )
 
-        last_updated = States.last_updated.desc() if descending else States.last_updated
-        baked_query += lambda q: q.order_by(States.entity_id, last_updated)
+        if descending:
+            baked_query += lambda q: q.order_by(
+                States.entity_id, States.last_updated.desc()
+            )
+        else:
+            baked_query += lambda q: q.order_by(States.entity_id, States.last_updated)
 
         if limit:
-            baked_query += lambda q: q.limit(limit)
+            baked_query += lambda q: q.limit(bindparam("limit"))
 
         states = execute(
             baked_query(session).params(
-                start_time=start_time, end_time=end_time, entity_id=entity_id
+                start_time=start_time,
+                end_time=end_time,
+                entity_id=entity_id,
+                limit=limit,
             )
         )
 
         entity_ids = [entity_id] if entity_id is not None else None
 
-        return _sorted_states_to_dict(
-            hass,
-            session,
-            states,
-            start_time,
-            entity_ids,
-            include_start_time_state=include_start_time_state,
+        return cast(
+            MutableMapping[str, list[State]],
+            _sorted_states_to_dict(
+                hass,
+                session,
+                states,
+                start_time,
+                entity_ids,
+                include_start_time_state=include_start_time_state,
+            ),
         )
 
 
 def get_last_state_changes(
     hass: HomeAssistant, number_of_states: int, entity_id: str
-) -> MutableMapping[str, Iterable[LazyState | State | dict[str, Any]]]:
+) -> MutableMapping[str, list[State]]:
     """Return the last number_of_states."""
     start_time = dt_util.utcnow()
 
     with session_scope(hass=hass) as session:
-        baked_query, join_attributes = bake_query_and_join_attributes(hass, False)
+        baked_query, join_attributes = bake_query_and_join_attributes(
+            hass, False, include_last_updated=False
+        )
 
         baked_query += lambda q: q.filter(States.last_changed == States.last_updated)
 
@@ -345,36 +457,16 @@ def get_last_state_changes(
 
         entity_ids = [entity_id] if entity_id is not None else None
 
-        return _sorted_states_to_dict(
-            hass,
-            session,
-            reversed(states),
-            start_time,
-            entity_ids,
-            include_start_time_state=False,
-        )
-
-
-def get_states(
-    hass: HomeAssistant,
-    utc_point_in_time: datetime,
-    entity_ids: list[str] | None = None,
-    run: RecorderRuns | None = None,
-    filters: Any = None,
-    no_attributes: bool = False,
-) -> list[LazyState]:
-    """Return the states at a specific point in time."""
-    if (
-        run is None
-        and (run := (recorder.run_information_from_instance(hass, utc_point_in_time)))
-        is None
-    ):
-        # History did not run before utc_point_in_time
-        return []
-
-    with session_scope(hass=hass) as session:
-        return _get_states_with_session(
-            hass, session, utc_point_in_time, entity_ids, run, filters, no_attributes
+        return cast(
+            MutableMapping[str, list[State]],
+            _sorted_states_to_dict(
+                hass,
+                session,
+                reversed(states),
+                start_time,
+                entity_ids,
+                include_start_time_state=False,
+            ),
         )
 
 
@@ -386,18 +478,17 @@ def _get_states_with_session(
     run: RecorderRuns | None = None,
     filters: Any | None = None,
     no_attributes: bool = False,
-) -> list[LazyState]:
+) -> list[State]:
     """Return the states at a specific point in time."""
     if entity_ids and len(entity_ids) == 1:
         return _get_single_entity_states_with_session(
             hass, session, utc_point_in_time, entity_ids[0], no_attributes
         )
 
-    if (
-        run is None
-        and (run := (recorder.run_information_with_session(session, utc_point_in_time)))
-        is None
-    ):
+    if run is None:
+        run = recorder.get_instance(hass).run_history.get(utc_point_in_time)
+
+    if run is None or process_timestamp(run.start) > utc_point_in_time:
         # History did not run before utc_point_in_time
         return []
 
@@ -482,7 +573,7 @@ def _get_single_entity_states_with_session(
     utc_point_in_time: datetime,
     entity_id: str,
     no_attributes: bool = False,
-) -> list[LazyState]:
+) -> list[State]:
     # Use an entirely different (and extremely fast) query if we only
     # have a single entity id
     baked_query, join_attributes = bake_query_and_join_attributes(hass, no_attributes)
@@ -514,7 +605,7 @@ def _sorted_states_to_dict(
     include_start_time_state: bool = True,
     minimal_response: bool = False,
     no_attributes: bool = False,
-) -> MutableMapping[str, Iterable[LazyState | State | dict[str, Any]]]:
+) -> MutableMapping[str, list[State | dict[str, Any]]]:
     """Convert SQL results into JSON friendly data structure.
 
     This takes our state list and turns it into a JSON friendly data
@@ -526,7 +617,7 @@ def _sorted_states_to_dict(
     each list of states, otherwise our graphs won't start on the Y
     axis correctly.
     """
-    result: dict[str, list[LazyState | dict[str, Any]]] = defaultdict(list)
+    result: dict[str, list[State | dict[str, Any]]] = defaultdict(list)
     # Set all entity IDs to empty lists in result set to maintain the order
     if entity_ids is not None:
         for ent_id in entity_ids:
@@ -535,13 +626,11 @@ def _sorted_states_to_dict(
     # Get the states at the start time
     timer_start = time.perf_counter()
     if include_start_time_state:
-        run = recorder.run_information_from_instance(hass, start_time)
         for state in _get_states_with_session(
             hass,
             session,
             start_time,
             entity_ids,
-            run=run,
             filters=filters,
             no_attributes=no_attributes,
         ):
@@ -557,21 +646,30 @@ def _sorted_states_to_dict(
     # here
     _process_timestamp_to_utc_isoformat = process_timestamp_to_utc_isoformat
 
+    if entity_ids and len(entity_ids) == 1:
+        states_iter: Iterable[tuple[str | Column, Iterator[States]]] = (
+            (entity_ids[0], iter(states)),
+        )
+    else:
+        states_iter = groupby(states, lambda state: state.entity_id)
+
     # Append all changes to it
-    for ent_id, group in groupby(states, lambda state: state.entity_id):  # type: ignore[no-any-return]
-        domain = split_entity_id(ent_id)[0]
+    for ent_id, group in states_iter:
         ent_results = result[ent_id]
         attr_cache: dict[str, dict[str, Any]] = {}
 
-        if not minimal_response or domain in NEED_ATTRIBUTE_DOMAINS:
+        if not minimal_response or split_entity_id(ent_id)[0] in NEED_ATTRIBUTE_DOMAINS:
             ent_results.extend(LazyState(db_state, attr_cache) for db_state in group)
+            continue
 
         # With minimal response we only provide a native
         # State for the first and last response. All the states
         # in-between only provide the "state" and the
         # "last_changed".
         if not ent_results:
-            ent_results.append(LazyState(next(group), attr_cache))
+            if (first_state := next(group, None)) is None:
+                continue
+            ent_results.append(LazyState(first_state, attr_cache))
 
         prev_state = ent_results[-1]
         assert isinstance(prev_state, LazyState)
@@ -601,15 +699,3 @@ def _sorted_states_to_dict(
 
     # Filter out the empty lists if some states had 0 results.
     return {key: val for key, val in result.items() if val}
-
-
-def get_state(
-    hass: HomeAssistant,
-    utc_point_in_time: datetime,
-    entity_id: str,
-    run: RecorderRuns | None = None,
-    no_attributes: bool = False,
-) -> LazyState | None:
-    """Return a state at a specific point in time."""
-    states = get_states(hass, utc_point_in_time, [entity_id], run, None, no_attributes)
-    return states[0] if states else None
