@@ -3,7 +3,6 @@ from __future__ import annotations
 
 import asyncio
 from http import HTTPStatus
-import logging
 from typing import Any, Final
 
 import aiohttp
@@ -14,13 +13,13 @@ import async_timeout
 import voluptuous as vol
 
 from homeassistant import config_entries
+from homeassistant.components import zeroconf
 from homeassistant.const import CONF_HOST, CONF_PASSWORD, CONF_USERNAME
 from homeassistant.core import HomeAssistant
 from homeassistant.data_entry_flow import FlowResult
 from homeassistant.helpers import aiohttp_client
-from homeassistant.helpers.typing import DiscoveryInfoType
 
-from .const import AIOSHELLY_DEVICE_TIMEOUT_SEC, DOMAIN
+from .const import AIOSHELLY_DEVICE_TIMEOUT_SEC, CONF_SLEEP_PERIOD, DOMAIN, LOGGER
 from .utils import (
     get_block_device_name,
     get_block_device_sleep_period,
@@ -31,8 +30,6 @@ from .utils import (
     get_rpc_device_name,
 )
 
-_LOGGER: Final = logging.getLogger(__name__)
-
 HOST_SCHEMA: Final = vol.Schema({vol.Required(CONF_HOST): str})
 
 HTTP_CONNECT_ERRORS: Final = (asyncio.TimeoutError, aiohttp.ClientError)
@@ -63,7 +60,7 @@ async def validate_input(
             await rpc_device.shutdown()
             return {
                 "title": get_rpc_device_name(rpc_device),
-                "sleep_period": 0,
+                CONF_SLEEP_PERIOD: 0,
                 "model": rpc_device.model,
                 "gen": 2,
             }
@@ -78,7 +75,7 @@ async def validate_input(
         block_device.shutdown()
         return {
             "title": get_block_device_name(block_device),
-            "sleep_period": get_block_device_sleep_period(block_device.settings),
+            CONF_SLEEP_PERIOD: get_block_device_sleep_period(block_device.settings),
             "model": block_device.model,
             "gen": 1,
         }
@@ -107,7 +104,7 @@ class ConfigFlow(config_entries.ConfigFlow, domain=DOMAIN):
             except aioshelly.exceptions.FirmwareUnsupported:
                 return self.async_abort(reason="unsupported_firmware")
             except Exception:  # pylint: disable=broad-except
-                _LOGGER.exception("Unexpected exception")
+                LOGGER.exception("Unexpected exception")
                 errors["base"] = "unknown"
             else:
                 await self.async_set_unique_id(self.info["mac"])
@@ -123,14 +120,14 @@ class ConfigFlow(config_entries.ConfigFlow, domain=DOMAIN):
                 except HTTP_CONNECT_ERRORS:
                     errors["base"] = "cannot_connect"
                 except Exception:  # pylint: disable=broad-except
-                    _LOGGER.exception("Unexpected exception")
+                    LOGGER.exception("Unexpected exception")
                     errors["base"] = "unknown"
                 else:
                     return self.async_create_entry(
                         title=device_info["title"],
                         data={
                             **user_input,
-                            "sleep_period": device_info["sleep_period"],
+                            CONF_SLEEP_PERIOD: device_info[CONF_SLEEP_PERIOD],
                             "model": device_info["model"],
                             "gen": device_info["gen"],
                         },
@@ -146,6 +143,8 @@ class ConfigFlow(config_entries.ConfigFlow, domain=DOMAIN):
         """Handle the credentials step."""
         errors: dict[str, str] = {}
         if user_input is not None:
+            if get_info_gen(self.info) == 2:
+                user_input[CONF_USERNAME] = "admin"
             try:
                 device_info = await validate_input(
                     self.hass, self.host, self.info, user_input
@@ -155,10 +154,14 @@ class ConfigFlow(config_entries.ConfigFlow, domain=DOMAIN):
                     errors["base"] = "invalid_auth"
                 else:
                     errors["base"] = "cannot_connect"
+            except aioshelly.exceptions.InvalidAuthError:
+                errors["base"] = "invalid_auth"
             except HTTP_CONNECT_ERRORS:
                 errors["base"] = "cannot_connect"
+            except aioshelly.exceptions.JSONRPCError:
+                errors["base"] = "cannot_connect"
             except Exception:  # pylint: disable=broad-except
-                _LOGGER.exception("Unexpected exception")
+                LOGGER.exception("Unexpected exception")
                 errors["base"] = "unknown"
             else:
                 return self.async_create_entry(
@@ -166,7 +169,7 @@ class ConfigFlow(config_entries.ConfigFlow, domain=DOMAIN):
                     data={
                         **user_input,
                         CONF_HOST: self.host,
-                        "sleep_period": device_info["sleep_period"],
+                        CONF_SLEEP_PERIOD: device_info[CONF_SLEEP_PERIOD],
                         "model": device_info["model"],
                         "gen": device_info["gen"],
                     },
@@ -174,35 +177,42 @@ class ConfigFlow(config_entries.ConfigFlow, domain=DOMAIN):
         else:
             user_input = {}
 
-        schema = vol.Schema(
-            {
+        if get_info_gen(self.info) == 2:
+            schema = {
+                vol.Required(CONF_PASSWORD, default=user_input.get(CONF_PASSWORD)): str,
+            }
+        else:
+            schema = {
                 vol.Required(CONF_USERNAME, default=user_input.get(CONF_USERNAME)): str,
                 vol.Required(CONF_PASSWORD, default=user_input.get(CONF_PASSWORD)): str,
             }
-        )
 
         return self.async_show_form(
-            step_id="credentials", data_schema=schema, errors=errors
+            step_id="credentials", data_schema=vol.Schema(schema), errors=errors
         )
 
     async def async_step_zeroconf(
-        self, discovery_info: DiscoveryInfoType
+        self, discovery_info: zeroconf.ZeroconfServiceInfo
     ) -> FlowResult:
         """Handle zeroconf discovery."""
+        host = discovery_info.host
         try:
-            self.info = await self._async_get_info(discovery_info["host"])
+            self.info = await self._async_get_info(host)
         except HTTP_CONNECT_ERRORS:
             return self.async_abort(reason="cannot_connect")
         except aioshelly.exceptions.FirmwareUnsupported:
             return self.async_abort(reason="unsupported_firmware")
 
         await self.async_set_unique_id(self.info["mac"])
-        self._abort_if_unique_id_configured({CONF_HOST: discovery_info["host"]})
-        self.host = discovery_info["host"]
+        self._abort_if_unique_id_configured({CONF_HOST: host})
+        self.host = host
 
-        self.context["title_placeholders"] = {
-            "name": discovery_info.get("name", "").split(".")[0]
-        }
+        self.context.update(
+            {
+                "title_placeholders": {"name": discovery_info.name.split(".")[0]},
+                "configuration_url": f"http://{discovery_info.host}",
+            }
+        )
 
         if get_info_auth(self.info):
             return await self.async_step_credentials()
@@ -224,7 +234,7 @@ class ConfigFlow(config_entries.ConfigFlow, domain=DOMAIN):
                 title=self.device_info["title"],
                 data={
                     "host": self.host,
-                    "sleep_period": self.device_info["sleep_period"],
+                    CONF_SLEEP_PERIOD: self.device_info[CONF_SLEEP_PERIOD],
                     "model": self.device_info["model"],
                     "gen": self.device_info["gen"],
                 },
