@@ -1,22 +1,22 @@
 """Support for interface with a Bose Soundtouch."""
+from __future__ import annotations
+
+from functools import partial
 import logging
 import re
 
 from libsoundtouch import soundtouch_device
+from libsoundtouch.utils import Source
 import voluptuous as vol
 
-from homeassistant.components.media_player import PLATFORM_SCHEMA, MediaPlayerDevice
-from homeassistant.components.media_player.const import (
-    SUPPORT_NEXT_TRACK,
-    SUPPORT_PAUSE,
-    SUPPORT_PLAY,
-    SUPPORT_PLAY_MEDIA,
-    SUPPORT_PREVIOUS_TRACK,
-    SUPPORT_TURN_OFF,
-    SUPPORT_TURN_ON,
-    SUPPORT_VOLUME_MUTE,
-    SUPPORT_VOLUME_SET,
-    SUPPORT_VOLUME_STEP,
+from homeassistant.components import media_source
+from homeassistant.components.media_player import (
+    PLATFORM_SCHEMA,
+    MediaPlayerEntity,
+    MediaPlayerEntityFeature,
+)
+from homeassistant.components.media_player.browse_media import (
+    async_process_play_media_url,
 )
 from homeassistant.const import (
     CONF_HOST,
@@ -28,8 +28,10 @@ from homeassistant.const import (
     STATE_PLAYING,
     STATE_UNAVAILABLE,
 )
-from homeassistant.core import callback
+from homeassistant.core import HomeAssistant, ServiceCall, callback
 import homeassistant.helpers.config_validation as cv
+from homeassistant.helpers.entity_platform import AddEntitiesCallback
+from homeassistant.helpers.typing import ConfigType, DiscoveryInfoType
 
 from .const import (
     DOMAIN,
@@ -69,19 +71,6 @@ SOUNDTOUCH_REMOVE_ZONE_SCHEMA = vol.Schema(
 DEFAULT_NAME = "Bose Soundtouch"
 DEFAULT_PORT = 8090
 
-SUPPORT_SOUNDTOUCH = (
-    SUPPORT_PAUSE
-    | SUPPORT_VOLUME_STEP
-    | SUPPORT_VOLUME_MUTE
-    | SUPPORT_PREVIOUS_TRACK
-    | SUPPORT_NEXT_TRACK
-    | SUPPORT_TURN_OFF
-    | SUPPORT_VOLUME_SET
-    | SUPPORT_TURN_ON
-    | SUPPORT_PLAY
-    | SUPPORT_PLAY_MEDIA
-)
-
 PLATFORM_SCHEMA = PLATFORM_SCHEMA.extend(
     {
         vol.Required(CONF_HOST): cv.string,
@@ -91,7 +80,12 @@ PLATFORM_SCHEMA = PLATFORM_SCHEMA.extend(
 )
 
 
-def setup_platform(hass, config, add_entities, discovery_info=None):
+def setup_platform(
+    hass: HomeAssistant,
+    config: ConfigType,
+    add_entities: AddEntitiesCallback,
+    discovery_info: DiscoveryInfoType | None = None,
+) -> None:
     """Set up the Bose Soundtouch platform."""
     if DATA_SOUNDTOUCH not in hass.data:
         hass.data[DATA_SOUNDTOUCH] = []
@@ -119,7 +113,7 @@ def setup_platform(hass, config, add_entities, discovery_info=None):
         hass.data[DATA_SOUNDTOUCH].append(bose_soundtouch_entity)
         add_entities([bose_soundtouch_entity], True)
 
-    def service_handle(service):
+    def service_handle(service: ServiceCall) -> None:
         """Handle the applying of a service."""
         master_device_id = service.data.get("master")
         slaves_ids = service.data.get("slaves")
@@ -132,11 +126,13 @@ def setup_platform(hass, config, add_entities, discovery_info=None):
             ]
 
         master = next(
-            [
-                device
-                for device in hass.data[DATA_SOUNDTOUCH]
-                if device.entity_id == master_device_id
-            ].__iter__(),
+            iter(
+                [
+                    device
+                    for device in hass.data[DATA_SOUNDTOUCH]
+                    if device.entity_id == master_device_id
+                ]
+            ),
             None,
         )
 
@@ -184,9 +180,24 @@ def setup_platform(hass, config, add_entities, discovery_info=None):
     )
 
 
-class SoundTouchDevice(MediaPlayerDevice):
+class SoundTouchDevice(MediaPlayerEntity):
     """Representation of a SoundTouch Bose device."""
 
+    _attr_supported_features = (
+        MediaPlayerEntityFeature.PAUSE
+        | MediaPlayerEntityFeature.VOLUME_STEP
+        | MediaPlayerEntityFeature.VOLUME_MUTE
+        | MediaPlayerEntityFeature.PREVIOUS_TRACK
+        | MediaPlayerEntityFeature.NEXT_TRACK
+        | MediaPlayerEntityFeature.TURN_OFF
+        | MediaPlayerEntityFeature.VOLUME_SET
+        | MediaPlayerEntityFeature.TURN_ON
+        | MediaPlayerEntityFeature.PLAY
+        | MediaPlayerEntityFeature.PLAY_MEDIA
+        | MediaPlayerEntityFeature.SELECT_SOURCE
+        | MediaPlayerEntityFeature.BROWSE_MEDIA
+    )
+
     def __init__(self, name, config):
         """Create Soundtouch Entity."""
 
@@ -234,16 +245,24 @@ class SoundTouchDevice(MediaPlayerDevice):
 
         return MAP_STATUS.get(self._status.play_status, STATE_UNAVAILABLE)
 
+    @property
+    def source(self):
+        """Name of the current input source."""
+        return self._status.source
+
+    @property
+    def source_list(self):
+        """List of available input sources."""
+        return [
+            Source.AUX.value,
+            Source.BLUETOOTH.value,
+        ]
+
     @property
     def is_volume_muted(self):
         """Boolean if volume is currently muted."""
         return self._volume.muted
 
-    @property
-    def supported_features(self):
-        """Flag media player features that are supported."""
-        return SUPPORT_SOUNDTOUCH
-
     def turn_off(self):
         """Turn off media player."""
         self._device.power_off()
@@ -335,6 +354,16 @@ class SoundTouchDevice(MediaPlayerDevice):
             EVENT_HOMEASSISTANT_START, async_update_on_start
         )
 
+    async def async_play_media(self, media_type, media_id, **kwargs):
+        """Play a piece of media."""
+        if media_source.is_media_source_id(media_id):
+            play_item = await media_source.async_resolve_media(self.hass, media_id)
+            media_id = async_process_play_media_url(self.hass, play_item.url)
+
+        await self.hass.async_add_executor_job(
+            partial(self.play_media, media_type, media_id, **kwargs)
+        )
+
     def play_media(self, media_type, media_id, **kwargs):
         """Play a piece of media."""
         _LOGGER.debug("Starting media with media_id: %s", media_id)
@@ -346,9 +375,9 @@ class SoundTouchDevice(MediaPlayerDevice):
             # Preset
             presets = self._device.presets()
             preset = next(
-                [
-                    preset for preset in presets if preset.preset_id == str(media_id)
-                ].__iter__(),
+                iter(
+                    [preset for preset in presets if preset.preset_id == str(media_id)]
+                ),
                 None,
             )
             if preset is not None:
@@ -357,6 +386,17 @@ class SoundTouchDevice(MediaPlayerDevice):
             else:
                 _LOGGER.warning("Unable to find preset with id %s", media_id)
 
+    def select_source(self, source):
+        """Select input source."""
+        if source == Source.AUX.value:
+            _LOGGER.debug("Selecting source AUX")
+            self._device.select_source_aux()
+        elif source == Source.BLUETOOTH.value:
+            _LOGGER.debug("Selecting source Bluetooth")
+            self._device.select_source_bluetooth()
+        else:
+            _LOGGER.warning("Source %s is not supported", source)
+
     def create_zone(self, slaves):
         """
         Create a zone (multi-room)  and play on selected devices.
@@ -413,7 +453,7 @@ class SoundTouchDevice(MediaPlayerDevice):
             self._device.add_zone_slave([slave.device for slave in slaves])
 
     @property
-    def device_state_attributes(self):
+    def extra_state_attributes(self):
         """Return entity specific state attributes."""
         attributes = {}
 
@@ -426,6 +466,10 @@ class SoundTouchDevice(MediaPlayerDevice):
 
         return attributes
 
+    async def async_browse_media(self, media_content_type=None, media_content_id=None):
+        """Implement the websocket media browsing helper."""
+        return await media_source.async_browse_media(self.hass, media_content_id)
+
     def get_zone_info(self):
         """Return the current zone info."""
         zone_status = self._device.zone_status()
