@@ -8,7 +8,7 @@ from datetime import timedelta
 from ipaddress import ip_address
 from typing import Any
 
-from async_upnp_client.exceptions import UpnpConnectionError
+from async_upnp_client.exceptions import UpnpCommunicationError, UpnpConnectionError
 import voluptuous as vol
 
 from homeassistant import config_entries
@@ -26,20 +26,20 @@ from homeassistant.helpers.typing import ConfigType
 from homeassistant.helpers.update_coordinator import (
     CoordinatorEntity,
     DataUpdateCoordinator,
+    UpdateFailed,
 )
 
 from .const import (
     CONF_LOCAL_IP,
-    CONFIG_ENTRY_HOSTNAME,
-    CONFIG_ENTRY_SCAN_INTERVAL,
+    CONFIG_ENTRY_MAC_ADDRESS,
+    CONFIG_ENTRY_ORIGINAL_UDN,
     CONFIG_ENTRY_ST,
     CONFIG_ENTRY_UDN,
     DEFAULT_SCAN_INTERVAL,
     DOMAIN,
-    DOMAIN_DEVICES,
     LOGGER,
 )
-from .device import Device
+from .device import Device, async_get_mac_address_from_host
 
 NOTIFICATION_ID = "upnp_notification"
 NOTIFICATION_TITLE = "UPnP/IGD Setup"
@@ -66,9 +66,7 @@ CONFIG_SCHEMA = vol.Schema(
 
 async def async_setup(hass: HomeAssistant, config: ConfigType) -> bool:
     """Set up UPnP component."""
-    hass.data[DOMAIN] = {
-        DOMAIN_DEVICES: {},
-    }
+    hass.data[DOMAIN] = {}
 
     # Only start if set up via configuration.yaml.
     if DOMAIN in config:
@@ -83,7 +81,7 @@ async def async_setup(hass: HomeAssistant, config: ConfigType) -> bool:
 
 async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool:
     """Set up UPnP/IGD device from a config entry."""
-    LOGGER.debug("Setting up config entry: %s", entry.unique_id)
+    LOGGER.debug("Setting up config entry: %s", entry.entry_id)
 
     udn = entry.data[CONFIG_ENTRY_UDN]
     st = entry.data[CONFIG_ENTRY_ST]  # pylint: disable=invalid-name
@@ -121,75 +119,105 @@ async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool:
         cancel_discovered_callback()
 
     # Create device.
+    assert discovery_info is not None
+    assert discovery_info.ssdp_location is not None
     location = discovery_info.ssdp_location
     try:
         device = await Device.async_create_device(hass, location)
     except UpnpConnectionError as err:
-        LOGGER.debug("Error connecting to device %s", location)
+        LOGGER.debug(
+            "Error connecting to device at location: %s, err: %s", location, err
+        )
         raise ConfigEntryNotReady from err
 
-    # Ensure entry has a unique_id.
-    if not entry.unique_id:
-        LOGGER.debug(
-            "Setting unique_id: %s, for config_entry: %s",
-            device.unique_id,
-            entry,
-        )
+    # Track the original UDN such that existing sensors do not change their unique_id.
+    if CONFIG_ENTRY_ORIGINAL_UDN not in entry.data:
         hass.config_entries.async_update_entry(
             entry=entry,
-            unique_id=device.unique_id,
+            data={
+                **entry.data,
+                CONFIG_ENTRY_ORIGINAL_UDN: device.udn,
+            },
         )
+    device.original_udn = entry.data[CONFIG_ENTRY_ORIGINAL_UDN]
 
-    # Ensure entry has a hostname, for older entries.
-    if (
-        CONFIG_ENTRY_HOSTNAME not in entry.data
-        or entry.data[CONFIG_ENTRY_HOSTNAME] != device.hostname
-    ):
+    # Store mac address for changed UDN matching.
+    if device.host:
+        device.mac_address = await async_get_mac_address_from_host(hass, device.host)
+    if device.mac_address and not entry.data.get("CONFIG_ENTRY_MAC_ADDRESS"):
         hass.config_entries.async_update_entry(
             entry=entry,
-            data={CONFIG_ENTRY_HOSTNAME: device.hostname, **entry.data},
+            data={
+                **entry.data,
+                CONFIG_ENTRY_MAC_ADDRESS: device.mac_address,
+            },
         )
 
-    # Create device registry entry.
+    connections = {(dr.CONNECTION_UPNP, device.udn)}
+    if device.mac_address:
+        connections.add((dr.CONNECTION_NETWORK_MAC, device.mac_address))
+
     device_registry = dr.async_get(hass)
-    device_registry.async_get_or_create(
-        config_entry_id=entry.entry_id,
-        connections={(dr.CONNECTION_UPNP, device.udn)},
-        identifiers={(DOMAIN, device.udn)},
-        name=device.name,
-        manufacturer=device.manufacturer,
-        model=device.model_name,
+    device_entry = device_registry.async_get_device(
+        identifiers=set(), connections=connections
     )
+    if device_entry:
+        LOGGER.debug(
+            "Found device using connections: %s, device_entry: %s",
+            connections,
+            device_entry,
+        )
+    if not device_entry:
+        # No device found, create new device entry.
+        device_entry = device_registry.async_get_or_create(
+            config_entry_id=entry.entry_id,
+            connections=connections,
+            identifiers={(DOMAIN, device.usn)},
+            name=device.name,
+            manufacturer=device.manufacturer,
+            model=device.model_name,
+        )
+        LOGGER.debug(
+            "Created device using UDN '%s', device_entry: %s", device.udn, device_entry
+        )
+    else:
+        # Update identifier.
+        device_entry = device_registry.async_update_device(
+            device_entry.id,
+            new_identifiers={(DOMAIN, device.usn)},
+        )
 
-    update_interval_sec = entry.options.get(
-        CONFIG_ENTRY_SCAN_INTERVAL, DEFAULT_SCAN_INTERVAL
-    )
-    update_interval = timedelta(seconds=update_interval_sec)
-    LOGGER.debug("update_interval: %s", update_interval)
+    assert device_entry
+    update_interval = timedelta(seconds=DEFAULT_SCAN_INTERVAL)
     coordinator = UpnpDataUpdateCoordinator(
         hass,
         device=device,
+        device_entry=device_entry,
         update_interval=update_interval,
     )
 
+    # Try an initial refresh.
+    await coordinator.async_config_entry_first_refresh()
+
     # Save coordinator.
     hass.data[DOMAIN][entry.entry_id] = coordinator
 
-    await coordinator.async_config_entry_first_refresh()
-
-    # Create sensors.
-    LOGGER.debug("Enabling sensors")
+    # Setup platforms, creating sensors/binary_sensors.
     hass.config_entries.async_setup_platforms(entry, PLATFORMS)
 
     return True
 
 
-async def async_unload_entry(hass: HomeAssistant, config_entry: ConfigEntry) -> bool:
+async def async_unload_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool:
     """Unload a UPnP/IGD device from a config entry."""
-    LOGGER.debug("Unloading config entry: %s", config_entry.unique_id)
+    LOGGER.debug("Unloading config entry: %s", entry.entry_id)
 
-    LOGGER.debug("Deleting sensors")
-    return await hass.config_entries.async_unload_platforms(config_entry, PLATFORMS)
+    # Unload platforms.
+    unload_ok = await hass.config_entries.async_unload_platforms(entry, PLATFORMS)
+    if unload_ok:
+        del hass.data[DOMAIN][entry.entry_id]
+
+    return unload_ok
 
 
 @dataclass
@@ -212,32 +240,50 @@ class UpnpDataUpdateCoordinator(DataUpdateCoordinator):
     """Define an object to update data from UPNP device."""
 
     def __init__(
-        self, hass: HomeAssistant, device: Device, update_interval: timedelta
+        self,
+        hass: HomeAssistant,
+        device: Device,
+        device_entry: dr.DeviceEntry,
+        update_interval: timedelta,
     ) -> None:
         """Initialize."""
         self.device = device
+        self.device_entry = device_entry
 
         super().__init__(
-            hass, LOGGER, name=device.name, update_interval=update_interval
+            hass,
+            LOGGER,
+            name=device.name,
+            update_interval=update_interval,
+            update_method=self._async_fetch_data,
         )
 
-    async def _async_update_data(self) -> Mapping[str, Any]:
+    async def _async_fetch_data(self) -> Mapping[str, Any]:
         """Update data."""
-        update_values = await asyncio.gather(
-            self.device.async_get_traffic_data(),
-            self.device.async_get_status(),
-        )
+        try:
+            update_values = await asyncio.gather(
+                self.device.async_get_traffic_data(),
+                self.device.async_get_status(),
+            )
 
-        return {
-            **update_values[0],
-            **update_values[1],
-        }
+            return {
+                **update_values[0],
+                **update_values[1],
+            }
+        except UpnpCommunicationError as exception:
+            LOGGER.debug(
+                "Caught exception when updating device: %s, exception: %s",
+                self.device,
+                exception,
+            )
+            raise UpdateFailed(
+                f"Unable to communicate with IGD at: {self.device.device_url}"
+            ) from exception
 
 
-class UpnpEntity(CoordinatorEntity):
+class UpnpEntity(CoordinatorEntity[UpnpDataUpdateCoordinator]):
     """Base class for UPnP/IGD entities."""
 
-    coordinator: UpnpDataUpdateCoordinator
     entity_description: UpnpSensorEntityDescription | UpnpBinarySensorEntityDescription
 
     def __init__(
@@ -251,13 +297,13 @@ class UpnpEntity(CoordinatorEntity):
         self._device = coordinator.device
         self.entity_description = entity_description
         self._attr_name = f"{coordinator.device.name} {entity_description.name}"
-        self._attr_unique_id = f"{coordinator.device.udn}_{entity_description.unique_id or entity_description.key}"
+        self._attr_unique_id = f"{coordinator.device.original_udn}_{entity_description.unique_id or entity_description.key}"
         self._attr_device_info = DeviceInfo(
-            connections={(dr.CONNECTION_UPNP, coordinator.device.udn)},
-            name=coordinator.device.name,
-            manufacturer=coordinator.device.manufacturer,
-            model=coordinator.device.model_name,
-            configuration_url=f"http://{coordinator.device.hostname}",
+            connections=coordinator.device_entry.connections,
+            name=coordinator.device_entry.name,
+            manufacturer=coordinator.device_entry.manufacturer,
+            model=coordinator.device_entry.model,
+            configuration_url=coordinator.device_entry.configuration_url,
         )
 
     @property
