@@ -3,16 +3,19 @@ from __future__ import annotations
 
 import asyncio
 from collections.abc import Callable
+import datetime as dt
 import json
-from typing import Any
+from typing import Any, cast
 
 import voluptuous as vol
 
 from homeassistant.auth.permissions.const import CAT_ENTITIES, POLICY_READ
-from homeassistant.bootstrap import SIGNAL_BOOTSTRAP_INTEGRATONS
-from homeassistant.components.websocket_api.const import ERR_NOT_FOUND
-from homeassistant.const import EVENT_STATE_CHANGED, EVENT_TIME_CHANGED, MATCH_ALL
-from homeassistant.core import Context, Event, HomeAssistant, callback
+from homeassistant.const import (
+    EVENT_STATE_CHANGED,
+    MATCH_ALL,
+    SIGNAL_BOOTSTRAP_INTEGRATONS,
+)
+from homeassistant.core import Context, Event, HomeAssistant, State, callback
 from homeassistant.exceptions import (
     HomeAssistantError,
     ServiceNotFound,
@@ -30,9 +33,14 @@ from homeassistant.helpers.json import ExtendedJSONEncoder
 from homeassistant.helpers.service import async_get_all_descriptions
 from homeassistant.loader import IntegrationNotFound, async_get_integration
 from homeassistant.setup import DATA_SETUP_TIME, async_get_loaded_integrations
+from homeassistant.util.json import (
+    find_paths_unserializable_data,
+    format_unserializable_data,
+)
 
 from . import const, decorators, messages
 from .connection import ActiveConnection
+from .const import ERR_NOT_FOUND
 
 
 @callback
@@ -44,6 +52,7 @@ def async_register_commands(
     async_reg(hass, handle_call_service)
     async_reg(hass, handle_entity_source)
     async_reg(hass, handle_execute_script)
+    async_reg(hass, handle_fire_event)
     async_reg(hass, handle_get_config)
     async_reg(hass, handle_get_services)
     async_reg(hass, handle_get_states)
@@ -57,6 +66,8 @@ def async_register_commands(
     async_reg(hass, handle_subscribe_trigger)
     async_reg(hass, handle_test_condition)
     async_reg(hass, handle_unsubscribe_events)
+    async_reg(hass, handle_validate_config)
+    async_reg(hass, handle_subscribe_entities)
 
 
 def pong_message(iden: int) -> dict[str, Any]:
@@ -101,16 +112,13 @@ def handle_subscribe_events(
         @callback
         def forward_events(event: Event) -> None:
             """Forward events to websocket."""
-            if event.event_type == EVENT_TIME_CHANGED:
-                return
-
             connection.send_message(messages.cached_event_message(msg["id"], event))
 
     connection.subscriptions[msg["id"]] = hass.bus.async_listen(
         event_type, forward_events
     )
 
-    connection.send_message(messages.result_message(msg["id"]))
+    connection.send_result(msg["id"])
 
 
 @callback
@@ -133,7 +141,7 @@ def handle_subscribe_bootstrap_integrations(
         hass, SIGNAL_BOOTSTRAP_INTEGRATONS, forward_bootstrap_integrations
     )
 
-    connection.send_message(messages.result_message(msg["id"]))
+    connection.send_result(msg["id"])
 
 
 @callback
@@ -151,13 +159,9 @@ def handle_unsubscribe_events(
 
     if subscription in connection.subscriptions:
         connection.subscriptions.pop(subscription)()
-        connection.send_message(messages.result_message(msg["id"]))
+        connection.send_result(msg["id"])
     else:
-        connection.send_message(
-            messages.error_message(
-                msg["id"], const.ERR_NOT_FOUND, "Subscription not found."
-            )
-        )
+        connection.send_error(msg["id"], const.ERR_NOT_FOUND, "Subscription not found.")
 
 
 @decorators.websocket_command(
@@ -190,36 +194,34 @@ async def handle_call_service(
             context,
             target=target,
         )
-        connection.send_message(
-            messages.result_message(msg["id"], {"context": context})
-        )
+        connection.send_result(msg["id"], {"context": context})
     except ServiceNotFound as err:
         if err.domain == msg["domain"] and err.service == msg["service"]:
-            connection.send_message(
-                messages.error_message(
-                    msg["id"], const.ERR_NOT_FOUND, "Service not found."
-                )
-            )
+            connection.send_error(msg["id"], const.ERR_NOT_FOUND, "Service not found.")
         else:
-            connection.send_message(
-                messages.error_message(
-                    msg["id"], const.ERR_HOME_ASSISTANT_ERROR, str(err)
-                )
-            )
+            connection.send_error(msg["id"], const.ERR_HOME_ASSISTANT_ERROR, str(err))
     except vol.Invalid as err:
-        connection.send_message(
-            messages.error_message(msg["id"], const.ERR_INVALID_FORMAT, str(err))
-        )
+        connection.send_error(msg["id"], const.ERR_INVALID_FORMAT, str(err))
     except HomeAssistantError as err:
         connection.logger.exception(err)
-        connection.send_message(
-            messages.error_message(msg["id"], const.ERR_HOME_ASSISTANT_ERROR, str(err))
-        )
+        connection.send_error(msg["id"], const.ERR_HOME_ASSISTANT_ERROR, str(err))
     except Exception as err:  # pylint: disable=broad-except
         connection.logger.exception(err)
-        connection.send_message(
-            messages.error_message(msg["id"], const.ERR_UNKNOWN_ERROR, str(err))
-        )
+        connection.send_error(msg["id"], const.ERR_UNKNOWN_ERROR, str(err))
+
+
+@callback
+def _async_get_allowed_states(
+    hass: HomeAssistant, connection: ActiveConnection
+) -> list[State]:
+    if connection.user.permissions.access_all_entities("read"):
+        return hass.states.async_all()
+    entity_perm = connection.user.permissions.check_entity
+    return [
+        state
+        for state in hass.states.async_all()
+        if entity_perm(state.entity_id, "read")
+    ]
 
 
 @callback
@@ -228,17 +230,108 @@ def handle_get_states(
     hass: HomeAssistant, connection: ActiveConnection, msg: dict[str, Any]
 ) -> None:
     """Handle get states command."""
-    if connection.user.permissions.access_all_entities("read"):
-        states = hass.states.async_all()
-    else:
-        entity_perm = connection.user.permissions.check_entity
-        states = [
-            state
-            for state in hass.states.async_all()
-            if entity_perm(state.entity_id, "read")
-        ]
+    states = _async_get_allowed_states(hass, connection)
 
-    connection.send_message(messages.result_message(msg["id"], states))
+    # JSON serialize here so we can recover if it blows up due to the
+    # state machine containing unserializable data. This command is required
+    # to succeed for the UI to show.
+    response = messages.result_message(msg["id"], states)
+    try:
+        connection.send_message(const.JSON_DUMP(response))
+        return
+    except (ValueError, TypeError):
+        connection.logger.error(
+            "Unable to serialize to JSON. Bad data found at %s",
+            format_unserializable_data(
+                find_paths_unserializable_data(response, dump=const.JSON_DUMP)
+            ),
+        )
+    del response
+
+    # If we can't serialize, we'll filter out unserializable states
+    serialized = []
+    for state in states:
+        try:
+            serialized.append(const.JSON_DUMP(state))
+        except (ValueError, TypeError):
+            # Error is already logged above
+            pass
+
+    # We now have partially serialized states. Craft some JSON.
+    response2 = const.JSON_DUMP(messages.result_message(msg["id"], ["TO_REPLACE"]))
+    response2 = response2.replace('"TO_REPLACE"', ", ".join(serialized))
+    connection.send_message(response2)
+
+
+@callback
+@decorators.websocket_command(
+    {
+        vol.Required("type"): "subscribe_entities",
+        vol.Optional("entity_ids"): cv.entity_ids,
+    }
+)
+def handle_subscribe_entities(
+    hass: HomeAssistant, connection: ActiveConnection, msg: dict[str, Any]
+) -> None:
+    """Handle subscribe entities command."""
+    entity_ids = set(msg.get("entity_ids", []))
+
+    @callback
+    def forward_entity_changes(event: Event) -> None:
+        """Forward entity state changed events to websocket."""
+        if not connection.user.permissions.check_entity(
+            event.data["entity_id"], POLICY_READ
+        ):
+            return
+        if entity_ids and event.data["entity_id"] not in entity_ids:
+            return
+
+        connection.send_message(messages.cached_state_diff_message(msg["id"], event))
+
+    # We must never await between sending the states and listening for
+    # state changed events or we will introduce a race condition
+    # where some states are missed
+    states = _async_get_allowed_states(hass, connection)
+    connection.subscriptions[msg["id"]] = hass.bus.async_listen(
+        "state_changed", forward_entity_changes
+    )
+    connection.send_result(msg["id"])
+    data: dict[str, dict[str, dict]] = {
+        messages.ENTITY_EVENT_ADD: {
+            state.entity_id: messages.compressed_state_dict_add(state)
+            for state in states
+            if not entity_ids or state.entity_id in entity_ids
+        }
+    }
+
+    # JSON serialize here so we can recover if it blows up due to the
+    # state machine containing unserializable data. This command is required
+    # to succeed for the UI to show.
+    response = messages.event_message(msg["id"], data)
+    try:
+        connection.send_message(const.JSON_DUMP(response))
+        return
+    except (ValueError, TypeError):
+        connection.logger.error(
+            "Unable to serialize to JSON. Bad data found at %s",
+            format_unserializable_data(
+                find_paths_unserializable_data(response, dump=const.JSON_DUMP)
+            ),
+        )
+    del response
+
+    add_entities = data[messages.ENTITY_EVENT_ADD]
+    cannot_serialize: list[str] = []
+    for entity_id, state_dict in add_entities.items():
+        try:
+            const.JSON_DUMP(state_dict)
+        except (ValueError, TypeError):
+            cannot_serialize.append(entity_id)
+
+    for entity_id in cannot_serialize:
+        del add_entities[entity_id]
+
+    connection.send_message(const.JSON_DUMP(messages.event_message(msg["id"], data)))
 
 
 @decorators.websocket_command({vol.Required("type"): "get_services"})
@@ -248,7 +341,7 @@ async def handle_get_services(
 ) -> None:
     """Handle get services command."""
     descriptions = await async_get_all_descriptions(hass)
-    connection.send_message(messages.result_message(msg["id"], descriptions))
+    connection.send_result(msg["id"], descriptions)
 
 
 @callback
@@ -257,7 +350,7 @@ def handle_get_config(
     hass: HomeAssistant, connection: ActiveConnection, msg: dict[str, Any]
 ) -> None:
     """Handle get config command."""
-    connection.send_message(messages.result_message(msg["id"], hass.config.as_dict()))
+    connection.send_result(msg["id"], hass.config.as_dict())
 
 
 @decorators.websocket_command({vol.Required("type"): "manifest/list"})
@@ -300,7 +393,9 @@ async def handle_integration_setup_info(
         msg["id"],
         [
             {"domain": integration, "seconds": timedelta.total_seconds()}
-            for integration, timedelta in hass.data[DATA_SETUP_TIME].items()
+            for integration, timedelta in cast(
+                dict[str, dt.timedelta], hass.data[DATA_SETUP_TIME]
+            ).items()
         ],
     )
 
@@ -338,7 +433,7 @@ async def handle_render_template(
     if timeout:
         try:
             timed_out = await template_obj.async_render_will_timeout(
-                timeout, strict=msg["strict"]
+                timeout, variables, strict=msg["strict"]
             )
         except TemplateError as ex:
             connection.send_error(msg["id"], const.ERR_TEMPLATE_ERROR, str(ex))
@@ -353,7 +448,9 @@ async def handle_render_template(
             return
 
     @callback
-    def _template_listener(event: Event, updates: list[TrackTemplateResult]) -> None:
+    def _template_listener(
+        event: Event | None, updates: list[TrackTemplateResult]
+    ) -> None:
         nonlocal info
         track_template_result = updates.pop()
         result = track_template_result.result
@@ -407,7 +504,7 @@ def handle_entity_source(
                 if entity_perm(entity_id, "read")
             }
 
-        connection.send_message(messages.result_message(msg["id"], sources))
+        connection.send_result(msg["id"], sources)
         return
 
     sources = {}
@@ -457,7 +554,9 @@ async def handle_subscribe_trigger(
             msg["id"], {"variables": variables, "context": context}
         )
         connection.send_message(
-            json.dumps(message, cls=ExtendedJSONEncoder, allow_nan=False)
+            json.dumps(
+                message, cls=ExtendedJSONEncoder, allow_nan=False, separators=(",", ":")
+            )
         )
 
     connection.subscriptions[msg["id"]] = (
@@ -525,4 +624,60 @@ async def handle_execute_script(
     context = connection.context(msg)
     script_obj = Script(hass, msg["sequence"], f"{const.DOMAIN} script", const.DOMAIN)
     await script_obj.async_run(msg.get("variables"), context=context)
-    connection.send_message(messages.result_message(msg["id"], {"context": context}))
+    connection.send_result(msg["id"], {"context": context})
+
+
+@decorators.websocket_command(
+    {
+        vol.Required("type"): "fire_event",
+        vol.Required("event_type"): str,
+        vol.Optional("event_data"): dict,
+    }
+)
+@decorators.require_admin
+@decorators.async_response
+async def handle_fire_event(
+    hass: HomeAssistant, connection: ActiveConnection, msg: dict[str, Any]
+) -> None:
+    """Handle fire event command."""
+    context = connection.context(msg)
+
+    hass.bus.async_fire(msg["event_type"], msg.get("event_data"), context=context)
+    connection.send_result(msg["id"], {"context": context})
+
+
+@decorators.websocket_command(
+    {
+        vol.Required("type"): "validate_config",
+        vol.Optional("trigger"): cv.match_all,
+        vol.Optional("condition"): cv.match_all,
+        vol.Optional("action"): cv.match_all,
+    }
+)
+@decorators.async_response
+async def handle_validate_config(
+    hass: HomeAssistant, connection: ActiveConnection, msg: dict[str, Any]
+) -> None:
+    """Handle validate config command."""
+    # Circular dep
+    # pylint: disable=import-outside-toplevel
+    from homeassistant.helpers import condition, script, trigger
+
+    result = {}
+
+    for key, schema, validator in (
+        ("trigger", cv.TRIGGER_SCHEMA, trigger.async_validate_trigger_config),
+        ("condition", cv.CONDITION_SCHEMA, condition.async_validate_condition_config),
+        ("action", cv.SCRIPT_SCHEMA, script.async_validate_actions_config),
+    ):
+        if key not in msg:
+            continue
+
+        try:
+            await validator(hass, schema(msg[key]))  # type: ignore[operator]
+        except vol.Invalid as err:
+            result[key] = {"valid": False, "error": str(err)}
+        else:
+            result[key] = {"valid": True, "error": None}
+
+    connection.send_result(msg["id"], result)
