@@ -1,27 +1,25 @@
 """View to accept incoming websocket connection."""
+from __future__ import annotations
+
 import asyncio
+from collections.abc import Callable
 from contextlib import suppress
+import datetime as dt
 import logging
-from typing import Optional
+from typing import Any, Final
 
 from aiohttp import WSMsgType, web
 import async_timeout
 
 from homeassistant.components.http import HomeAssistantView
 from homeassistant.const import EVENT_HOMEASSISTANT_STOP
-from homeassistant.core import callback
+from homeassistant.core import Event, HomeAssistant, callback
 from homeassistant.helpers.event import async_call_later
-from homeassistant.util.json import (
-    find_paths_unserializable_data,
-    format_unserializable_data,
-)
 
 from .auth import AuthPhase, auth_required_message
 from .const import (
     CANCELLATION_ERRORS,
     DATA_CONNECTIONS,
-    ERR_UNKNOWN_ERROR,
-    JSON_DUMP,
     MAX_PENDING_MSG,
     PENDING_MSG_PEAK,
     PENDING_MSG_PEAK_TIME,
@@ -30,83 +28,72 @@ from .const import (
     URL,
 )
 from .error import Disconnect
-from .messages import error_message
+from .messages import message_to_json
 
-# mypy: allow-untyped-calls, allow-untyped-defs, no-check-untyped-defs
+_WS_LOGGER: Final = logging.getLogger(f"{__name__}.connection")
 
 
 class WebsocketAPIView(HomeAssistantView):
     """View to serve a websockets endpoint."""
 
-    name = "websocketapi"
-    url = URL
-    requires_auth = False
+    name: str = "websocketapi"
+    url: str = URL
+    requires_auth: bool = False
 
-    async def get(self, request):
+    async def get(self, request: web.Request) -> web.WebSocketResponse:
         """Handle an incoming websocket connection."""
         return await WebSocketHandler(request.app["hass"], request).async_handle()
 
 
+class WebSocketAdapter(logging.LoggerAdapter):
+    """Add connection id to websocket messages."""
+
+    def process(self, msg: str, kwargs: Any) -> tuple[str, Any]:
+        """Add connid to websocket log messages."""
+        return f'[{self.extra["connid"]}] {msg}', kwargs
+
+
 class WebSocketHandler:
     """Handle an active websocket client connection."""
 
-    def __init__(self, hass, request):
+    def __init__(self, hass: HomeAssistant, request: web.Request) -> None:
         """Initialize an active connection."""
         self.hass = hass
         self.request = request
-        self.wsock: Optional[web.WebSocketResponse] = None
+        self.wsock = web.WebSocketResponse(heartbeat=55)
         self._to_write: asyncio.Queue = asyncio.Queue(maxsize=MAX_PENDING_MSG)
-        self._handle_task = None
-        self._writer_task = None
-        self._logger = logging.getLogger("{}.connection.{}".format(__name__, id(self)))
-        self._peak_checker_unsub = None
+        self._handle_task: asyncio.Task | None = None
+        self._writer_task: asyncio.Task | None = None
+        self._logger = WebSocketAdapter(_WS_LOGGER, {"connid": id(self)})
+        self._peak_checker_unsub: Callable[[], None] | None = None
 
-    async def _writer(self):
+    async def _writer(self) -> None:
         """Write outgoing messages."""
         # Exceptions if Socket disconnected or cancelled by connection handler
         with suppress(RuntimeError, ConnectionResetError, *CANCELLATION_ERRORS):
             while not self.wsock.closed:
-                message = await self._to_write.get()
-                if message is None:
+                if (message := await self._to_write.get()) is None:
                     break
 
                 self._logger.debug("Sending %s", message)
-
-                if isinstance(message, str):
-                    await self.wsock.send_str(message)
-                    continue
-
-                try:
-                    dumped = JSON_DUMP(message)
-                except (ValueError, TypeError):
-                    await self.wsock.send_json(
-                        error_message(
-                            message["id"], ERR_UNKNOWN_ERROR, "Invalid JSON in response"
-                        )
-                    )
-                    self._logger.error(
-                        "Unable to serialize to JSON. Bad data found at %s",
-                        format_unserializable_data(
-                            find_paths_unserializable_data(message, dump=JSON_DUMP)
-                        ),
-                    )
-                    continue
-
-                await self.wsock.send_str(dumped)
+                await self.wsock.send_str(message)
 
         # Clean up the peaker checker when we shut down the writer
-        if self._peak_checker_unsub:
+        if self._peak_checker_unsub is not None:
             self._peak_checker_unsub()
             self._peak_checker_unsub = None
 
     @callback
-    def _send_message(self, message):
+    def _send_message(self, message: str | dict[str, Any]) -> None:
         """Send a message to the client.
 
         Closes connection if the client is not reading the messages.
 
         Async friendly.
         """
+        if not isinstance(message, str):
+            message = message_to_json(message)
+
         try:
             self._to_write.put_nowait(message)
         except asyncio.QueueFull:
@@ -128,7 +115,7 @@ class WebSocketHandler:
             )
 
     @callback
-    def _check_write_peak(self, _):
+    def _check_write_peak(self, _utc_time: dt.datetime) -> None:
         """Check that we are no longer above the write peak."""
         self._peak_checker_unsub = None
 
@@ -143,21 +130,29 @@ class WebSocketHandler:
         self._cancel()
 
     @callback
-    def _cancel(self):
+    def _cancel(self) -> None:
         """Cancel the connection."""
-        self._handle_task.cancel()
-        self._writer_task.cancel()
+        if self._handle_task is not None:
+            self._handle_task.cancel()
+        if self._writer_task is not None:
+            self._writer_task.cancel()
 
-    async def async_handle(self):
+    async def async_handle(self) -> web.WebSocketResponse:
         """Handle a websocket response."""
         request = self.request
-        wsock = self.wsock = web.WebSocketResponse(heartbeat=55)
-        await wsock.prepare(request)
-        self._logger.debug("Connected")
+        wsock = self.wsock
+        try:
+            async with async_timeout.timeout(10):
+                await wsock.prepare(request)
+        except asyncio.TimeoutError:
+            self._logger.warning("Timeout preparing request from %s", request.remote)
+            return wsock
+
+        self._logger.debug("Connected from %s", request.remote)
         self._handle_task = asyncio.current_task()
 
         @callback
-        def handle_hass_stop(event):
+        def handle_hass_stop(event: Event) -> None:
             """Cancel this connection."""
             self._cancel()
 
@@ -165,9 +160,13 @@ class WebSocketHandler:
             EVENT_HOMEASSISTANT_STOP, handle_hass_stop
         )
 
-        self._writer_task = self.hass.async_create_task(self._writer())
+        # As the webserver is now started before the start
+        # event we do not want to block for websocket responses
+        self._writer_task = asyncio.create_task(self._writer())
 
-        auth = AuthPhase(self._logger, self.hass, self._send_message, request)
+        auth = AuthPhase(
+            self._logger, self.hass, self._send_message, self._cancel, request
+        )
         connection = None
         disconnect_warn = None
 
@@ -176,11 +175,11 @@ class WebSocketHandler:
 
             # Auth Phase
             try:
-                with async_timeout.timeout(10):
+                async with async_timeout.timeout(10):
                     msg = await wsock.receive()
-            except asyncio.TimeoutError:
+            except asyncio.TimeoutError as err:
                 disconnect_warn = "Did not receive auth message within 10 seconds"
-                raise Disconnect
+                raise Disconnect from err
 
             if msg.type in (WSMsgType.CLOSE, WSMsgType.CLOSING):
                 raise Disconnect
@@ -191,9 +190,9 @@ class WebSocketHandler:
 
             try:
                 msg_data = msg.json()
-            except ValueError:
+            except ValueError as err:
                 disconnect_warn = "Received invalid JSON."
-                raise Disconnect
+                raise Disconnect from err
 
             self._logger.debug("Received %s", msg_data)
             connection = await auth.async_handle(msg_data)
@@ -237,26 +236,26 @@ class WebSocketHandler:
             unsub_stop()
 
             if connection is not None:
-                connection.async_close()
+                connection.async_handle_close()
 
             try:
                 self._to_write.put_nowait(None)
                 # Make sure all error messages are written before closing
                 await self._writer_task
-            except asyncio.QueueFull:
+                await wsock.close()
+            except asyncio.QueueFull:  # can be raised by put_nowait
                 self._writer_task.cancel()
 
-            await wsock.close()
+            finally:
+                if disconnect_warn is None:
+                    self._logger.debug("Disconnected")
+                else:
+                    self._logger.warning("Disconnected: %s", disconnect_warn)
 
-            if disconnect_warn is None:
-                self._logger.debug("Disconnected")
-            else:
-                self._logger.warning("Disconnected: %s", disconnect_warn)
-
-            if connection is not None:
-                self.hass.data[DATA_CONNECTIONS] -= 1
-            self.hass.helpers.dispatcher.async_dispatcher_send(
-                SIGNAL_WEBSOCKET_DISCONNECTED
-            )
+                if connection is not None:
+                    self.hass.data[DATA_CONNECTIONS] -= 1
+                self.hass.helpers.dispatcher.async_dispatcher_send(
+                    SIGNAL_WEBSOCKET_DISCONNECTED
+                )
 
         return wsock
