@@ -3,11 +3,17 @@ from __future__ import annotations
 
 import asyncio
 from enum import Enum
-from functools import wraps
+from functools import partialmethod, wraps
 import logging
 from typing import Any
 
 import zigpy.exceptions
+from zigpy.zcl.foundation import (
+    CommandSchema,
+    ConfigureReportingResponseRecord,
+    Status,
+    ZCLAttributeDef,
+)
 
 from homeassistant.const import ATTR_COMMAND
 from homeassistant.core import callback
@@ -19,17 +25,20 @@ from ..const import (
     ATTR_ATTRIBUTE_ID,
     ATTR_ATTRIBUTE_NAME,
     ATTR_CLUSTER_ID,
+    ATTR_PARAMS,
     ATTR_TYPE,
     ATTR_UNIQUE_ID,
     ATTR_VALUE,
     CHANNEL_ZDO,
+    REPORT_CONFIG_ATTR_PER_REQ,
     SIGNAL_ATTR_UPDATED,
     ZHA_CHANNEL_MSG,
     ZHA_CHANNEL_MSG_BIND,
     ZHA_CHANNEL_MSG_CFG_RPT,
     ZHA_CHANNEL_MSG_DATA,
+    ZHA_CHANNEL_READS_PER_REQ,
 )
-from ..helpers import LogMixin, safe_read
+from ..helpers import LogMixin, retryable_req, safe_read
 
 _LOGGER = logging.getLogger(__name__)
 
@@ -87,29 +96,37 @@ class ChannelStatus(Enum):
 class ZigbeeChannel(LogMixin):
     """Base channel for a Zigbee cluster."""
 
-    REPORT_CONFIG = ()
+    REPORT_CONFIG: tuple[dict[int | str, tuple[int, int, int | float]]] = ()
     BIND: bool = True
 
+    # Dict of attributes to read on channel initialization.
+    # Dict keys -- attribute ID or names, with bool value indicating whether a cached
+    # attribute read is acceptable.
+    ZCL_INIT_ATTRS: dict[int | str, bool] = {}
+
     def __init__(
         self, cluster: zha_typing.ZigpyClusterType, ch_pool: zha_typing.ChannelPoolType
     ) -> None:
         """Initialize ZigbeeChannel."""
         self._generic_id = f"channel_0x{cluster.cluster_id:04x}"
-        self._channel_name = getattr(cluster, "ep_attribute", self._generic_id)
         self._ch_pool = ch_pool
         self._cluster = cluster
         self._id = f"{ch_pool.id}:0x{cluster.cluster_id:04x}"
         unique_id = ch_pool.unique_id.replace("-", ":")
         self._unique_id = f"{unique_id}:0x{cluster.cluster_id:04x}"
-        self._report_config = self.REPORT_CONFIG
-        if not hasattr(self, "_value_attribute") and len(self._report_config) > 0:
-            attr = self._report_config[0].get("attr")
+        if not hasattr(self, "_value_attribute") and self.REPORT_CONFIG:
+            attr = self.REPORT_CONFIG[0].get("attr")
             if isinstance(attr, str):
-                self.value_attribute = self.cluster.attridx.get(attr)
+                attribute: ZCLAttributeDef = self.cluster.attributes_by_name.get(attr)
+                if attribute is not None:
+                    self.value_attribute = attribute.id
+                else:
+                    self.value_attribute = None
             else:
                 self.value_attribute = attr
         self._status = ChannelStatus.CREATED
         self._cluster.add_listener(self)
+        self.data_cache = {}
 
     @property
     def id(self) -> str:
@@ -134,13 +151,17 @@ class ZigbeeChannel(LogMixin):
     @property
     def name(self) -> str:
         """Return friendly name."""
-        return self._channel_name
+        return self.cluster.ep_attribute or self._generic_id
 
     @property
     def status(self):
         """Return the status of the channel."""
         return self._status
 
+    def __hash__(self) -> int:
+        """Make this a hashable."""
+        return hash(self._unique_id)
+
     @callback
     def async_send_signal(self, signal: str, *args: Any) -> None:
         """Send a signal through hass dispatcher."""
@@ -195,42 +216,42 @@ class ZigbeeChannel(LogMixin):
         if self.cluster.cluster_id >= 0xFC00 and self._ch_pool.manufacturer_code:
             kwargs["manufacturer"] = self._ch_pool.manufacturer_code
 
-        for report in self._report_config:
-            attr = report["attr"]
+        for attr_report in self.REPORT_CONFIG:
+            attr, config = attr_report["attr"], attr_report["config"]
             attr_name = self.cluster.attributes.get(attr, [attr])[0]
-            min_report_int, max_report_int, reportable_change = report["config"]
             event_data[attr_name] = {
-                "min": min_report_int,
-                "max": max_report_int,
+                "min": config[0],
+                "max": config[1],
                 "id": attr,
                 "name": attr_name,
-                "change": reportable_change,
+                "change": config[2],
+                "success": False,
             }
 
+        to_configure = [*self.REPORT_CONFIG]
+        chunk, rest = (
+            to_configure[:REPORT_CONFIG_ATTR_PER_REQ],
+            to_configure[REPORT_CONFIG_ATTR_PER_REQ:],
+        )
+        while chunk:
+            reports = {rec["attr"]: rec["config"] for rec in chunk}
             try:
-                res = await self.cluster.configure_reporting(
-                    attr, min_report_int, max_report_int, reportable_change, **kwargs
-                )
-                self.debug(
-                    "reporting '%s' attr on '%s' cluster: %d/%d/%d: Result: '%s'",
-                    attr_name,
-                    self.cluster.ep_attribute,
-                    min_report_int,
-                    max_report_int,
-                    reportable_change,
-                    res,
-                )
-                event_data[attr_name]["success"] = (
-                    res[0][0].status == 0 or res[0][0].status == 134
-                )
+                res = await self.cluster.configure_reporting_multiple(reports, **kwargs)
+                self._configure_reporting_status(reports, res[0])
+                # if we get a response, then it's a success
+                for attr_stat in event_data.values():
+                    attr_stat["success"] = True
             except (zigpy.exceptions.ZigbeeException, asyncio.TimeoutError) as ex:
                 self.debug(
-                    "failed to set reporting for '%s' attr on '%s' cluster: %s",
-                    attr_name,
+                    "failed to set reporting on '%s' cluster for: %s",
                     self.cluster.ep_attribute,
                     str(ex),
                 )
-                event_data[attr_name]["success"] = False
+                break
+            chunk, rest = (
+                rest[:REPORT_CONFIG_ATTR_PER_REQ],
+                rest[REPORT_CONFIG_ATTR_PER_REQ:],
+            )
 
         async_dispatcher_send(
             self._ch_pool.hass,
@@ -245,37 +266,98 @@ class ZigbeeChannel(LogMixin):
             },
         )
 
+    def _configure_reporting_status(
+        self, attrs: dict[int | str, tuple], res: list | tuple
+    ) -> None:
+        """Parse configure reporting result."""
+        if isinstance(res, (Exception, ConfigureReportingResponseRecord)):
+            # assume default response
+            self.debug(
+                "attr reporting for '%s' on '%s': %s",
+                attrs,
+                self.name,
+                res,
+            )
+            return
+        if res[0].status == Status.SUCCESS and len(res) == 1:
+            self.debug(
+                "Successfully configured reporting for '%s' on '%s' cluster: %s",
+                attrs,
+                self.name,
+                res,
+            )
+            return
+
+        failed = [
+            self.cluster.attributes.get(r.attrid, [r.attrid])[0]
+            for r in res
+            if r.status != Status.SUCCESS
+        ]
+        attrs = {self.cluster.attributes.get(r, [r])[0] for r in attrs}
+        self.debug(
+            "Successfully configured reporting for '%s' on '%s' cluster",
+            attrs - set(failed),
+            self.name,
+        )
+        self.debug(
+            "Failed to configure reporting for '%s' on '%s' cluster: %s",
+            failed,
+            self.name,
+            res,
+        )
+
     async def async_configure(self) -> None:
         """Set cluster binding and attribute reporting."""
         if not self._ch_pool.skip_configuration:
             if self.BIND:
+                self.debug("Performing cluster binding")
                 await self.bind()
             if self.cluster.is_server:
+                self.debug("Configuring cluster attribute reporting")
                 await self.configure_reporting()
             ch_specific_cfg = getattr(self, "async_configure_channel_specific", None)
             if ch_specific_cfg:
+                self.debug("Performing channel specific configuration")
                 await ch_specific_cfg()
             self.debug("finished channel configuration")
         else:
             self.debug("skipping channel configuration")
         self._status = ChannelStatus.CONFIGURED
 
+    @retryable_req(delays=(1, 1, 3))
     async def async_initialize(self, from_cache: bool) -> None:
         """Initialize channel."""
         if not from_cache and self._ch_pool.skip_configuration:
+            self.debug("Skipping channel initialization")
             self._status = ChannelStatus.INITIALIZED
             return
 
         self.debug("initializing channel: from_cache: %s", from_cache)
-        attributes = [cfg["attr"] for cfg in self._report_config]
-        if attributes:
-            await self.get_attributes(attributes, from_cache=from_cache)
+        cached = [a for a, cached in self.ZCL_INIT_ATTRS.items() if cached]
+        uncached = [a for a, cached in self.ZCL_INIT_ATTRS.items() if not cached]
+        uncached.extend([cfg["attr"] for cfg in self.REPORT_CONFIG])
+
+        if cached:
+            self.debug("initializing cached channel attributes: %s", cached)
+            await self._get_attributes(
+                True, cached, from_cache=True, only_cache=from_cache
+            )
+        if uncached:
+            self.debug(
+                "initializing uncached channel attributes: %s - from cache[%s]",
+                uncached,
+                from_cache,
+            )
+            await self._get_attributes(
+                True, uncached, from_cache=from_cache, only_cache=from_cache
+            )
 
         ch_specific_init = getattr(self, "async_initialize_channel_specific", None)
         if ch_specific_init:
+            self.debug("Performing channel specific initialization: %s", uncached)
             await ch_specific_init(from_cache=from_cache)
 
-        self.debug("finished channel configuration")
+        self.debug("finished channel initialization")
         self._status = ChannelStatus.INITIALIZED
 
     @callback
@@ -288,7 +370,7 @@ class ZigbeeChannel(LogMixin):
         self.async_send_signal(
             f"{self.unique_id}_{SIGNAL_ATTR_UPDATED}",
             attrid,
-            self.cluster.attributes.get(attrid, [attrid])[0],
+            self._get_attribute_name(attrid),
             value,
         )
 
@@ -297,20 +379,39 @@ class ZigbeeChannel(LogMixin):
         """Handle ZDO commands on this cluster."""
 
     @callback
-    def zha_send_event(self, command: str, args: int | dict) -> None:
+    def zha_send_event(self, command: str, arg: list | dict | CommandSchema) -> None:
         """Relay events to hass."""
+
+        if isinstance(arg, CommandSchema):
+            args = [a for a in arg if a is not None]
+            params = arg.as_dict()
+        elif isinstance(arg, (list, dict)):
+            # Quirks can directly send lists and dicts to ZHA this way
+            args = arg
+            params = {}
+        else:
+            raise TypeError(f"Unexpected zha_send_event {command!r} argument: {arg!r}")
+
         self._ch_pool.zha_send_event(
             {
                 ATTR_UNIQUE_ID: self.unique_id,
                 ATTR_CLUSTER_ID: self.cluster.cluster_id,
                 ATTR_COMMAND: command,
+                # Maintain backwards compatibility with the old zigpy response format
                 ATTR_ARGS: args,
+                ATTR_PARAMS: params,
             }
         )
 
     async def async_update(self):
         """Retrieve latest state from cluster."""
 
+    def _get_attribute_name(self, attrid: int) -> str | int:
+        if attrid not in self.cluster.attributes:
+            return attrid
+
+        return self.cluster.attributes[attrid].name
+
     async def get_attribute_value(self, attribute, from_cache=True):
         """Get the value for an attribute."""
         manufacturer = None
@@ -321,39 +422,56 @@ class ZigbeeChannel(LogMixin):
             self._cluster,
             [attribute],
             allow_cache=from_cache,
-            only_cache=from_cache and not self._ch_pool.is_mains_powered,
+            only_cache=from_cache,
             manufacturer=manufacturer,
         )
         return result.get(attribute)
 
-    async def get_attributes(self, attributes, from_cache=True):
+    async def _get_attributes(
+        self,
+        raise_exceptions: bool,
+        attributes: list[int | str],
+        from_cache: bool = True,
+        only_cache: bool = True,
+    ) -> dict[int | str, Any]:
         """Get the values for a list of attributes."""
         manufacturer = None
         manufacturer_code = self._ch_pool.manufacturer_code
         if self.cluster.cluster_id >= 0xFC00 and manufacturer_code:
             manufacturer = manufacturer_code
-        try:
-            result, _ = await self.cluster.read_attributes(
-                attributes,
-                allow_cache=from_cache,
-                only_cache=from_cache and not self._ch_pool.is_mains_powered,
-                manufacturer=manufacturer,
-            )
-            return result
-        except (asyncio.TimeoutError, zigpy.exceptions.ZigbeeException) as ex:
-            self.debug(
-                "failed to get attributes '%s' on '%s' cluster: %s",
-                attributes,
-                self.cluster.ep_attribute,
-                str(ex),
-            )
-            return {}
+        chunk = attributes[:ZHA_CHANNEL_READS_PER_REQ]
+        rest = attributes[ZHA_CHANNEL_READS_PER_REQ:]
+        result = {}
+        while chunk:
+            try:
+                self.debug("Reading attributes in chunks: %s", chunk)
+                read, _ = await self.cluster.read_attributes(
+                    attributes,
+                    allow_cache=from_cache,
+                    only_cache=only_cache,
+                    manufacturer=manufacturer,
+                )
+                result.update(read)
+            except (asyncio.TimeoutError, zigpy.exceptions.ZigbeeException) as ex:
+                self.debug(
+                    "failed to get attributes '%s' on '%s' cluster: %s",
+                    chunk,
+                    self.cluster.ep_attribute,
+                    str(ex),
+                )
+                if raise_exceptions:
+                    raise
+            chunk = rest[:ZHA_CHANNEL_READS_PER_REQ]
+            rest = rest[ZHA_CHANNEL_READS_PER_REQ:]
+        return result
 
-    def log(self, level, msg, *args):
+    get_attributes = partialmethod(_get_attributes, False)
+
+    def log(self, level, msg, *args, **kwargs):
         """Log a message."""
         msg = f"[%s:%s]: {msg}"
         args = (self._ch_pool.nwk, self._id) + args
-        _LOGGER.log(level, msg, *args)
+        _LOGGER.log(level, msg, *args, **kwargs)
 
     def __getattr__(self, name):
         """Get attribute or a decorated cluster command."""
@@ -407,11 +525,11 @@ class ZDOChannel(LogMixin):
         """Configure channel."""
         self._status = ChannelStatus.CONFIGURED
 
-    def log(self, level, msg, *args):
+    def log(self, level, msg, *args, **kwargs):
         """Log a message."""
         msg = f"[%s:ZDO](%s): {msg}"
         args = (self._zha_device.nwk, self._zha_device.model) + args
-        _LOGGER.log(level, msg, *args)
+        _LOGGER.log(level, msg, *args, **kwargs)
 
 
 class ClientChannel(ZigbeeChannel):
@@ -420,13 +538,17 @@ class ClientChannel(ZigbeeChannel):
     @callback
     def attribute_updated(self, attrid, value):
         """Handle an attribute updated on this cluster."""
+
+        try:
+            attr_name = self._cluster.attributes[attrid].name
+        except KeyError:
+            attr_name = "Unknown"
+
         self.zha_send_event(
             SIGNAL_ATTR_UPDATED,
             {
                 ATTR_ATTRIBUTE_ID: attrid,
-                ATTR_ATTRIBUTE_NAME: self._cluster.attributes.get(attrid, ["Unknown"])[
-                    0
-                ],
+                ATTR_ATTRIBUTE_NAME: attr_name,
                 ATTR_VALUE: value,
             },
         )
@@ -438,4 +560,4 @@ class ClientChannel(ZigbeeChannel):
             self._cluster.server_commands is not None
             and self._cluster.server_commands.get(command_id) is not None
         ):
-            self.zha_send_event(self._cluster.server_commands.get(command_id)[0], args)
+            self.zha_send_event(self._cluster.server_commands[command_id].name, args)
