@@ -1,17 +1,18 @@
 """Representation of Z-Wave thermostats."""
 from __future__ import annotations
 
-from typing import Any, Callable, cast
+from typing import Any, cast
 
 from zwave_js_server.client import Client as ZwaveClient
-from zwave_js_server.const import (
+from zwave_js_server.const import CommandClass
+from zwave_js_server.const.command_class.thermostat import (
     THERMOSTAT_CURRENT_TEMP_PROPERTY,
+    THERMOSTAT_HUMIDITY_PROPERTY,
     THERMOSTAT_MODE_PROPERTY,
     THERMOSTAT_MODE_SETPOINT_MAP,
     THERMOSTAT_MODES,
     THERMOSTAT_OPERATING_STATE_PROPERTY,
     THERMOSTAT_SETPOINT_PROPERTY,
-    CommandClass,
     ThermostatMode,
     ThermostatOperatingState,
     ThermostatSetpointType,
@@ -27,22 +28,11 @@ from homeassistant.components.climate.const import (
     ATTR_HVAC_MODE,
     ATTR_TARGET_TEMP_HIGH,
     ATTR_TARGET_TEMP_LOW,
-    CURRENT_HVAC_COOL,
-    CURRENT_HVAC_FAN,
-    CURRENT_HVAC_HEAT,
-    CURRENT_HVAC_IDLE,
     DOMAIN as CLIMATE_DOMAIN,
-    HVAC_MODE_COOL,
-    HVAC_MODE_DRY,
-    HVAC_MODE_FAN_ONLY,
-    HVAC_MODE_HEAT,
-    HVAC_MODE_HEAT_COOL,
-    HVAC_MODE_OFF,
     PRESET_NONE,
-    SUPPORT_FAN_MODE,
-    SUPPORT_PRESET_MODE,
-    SUPPORT_TARGET_TEMPERATURE,
-    SUPPORT_TARGET_TEMPERATURE_RANGE,
+    ClimateEntityFeature,
+    HVACAction,
+    HVACMode,
 )
 from homeassistant.config_entries import ConfigEntry
 from homeassistant.const import (
@@ -53,52 +43,57 @@ from homeassistant.const import (
 )
 from homeassistant.core import HomeAssistant, callback
 from homeassistant.helpers.dispatcher import async_dispatcher_connect
-from homeassistant.helpers.temperature import convert_temperature
+from homeassistant.helpers.entity_platform import AddEntitiesCallback
+from homeassistant.util.temperature import convert as convert_temperature
 
-from .const import DATA_CLIENT, DATA_UNSUBSCRIBE, DOMAIN
+from .const import DATA_CLIENT, DOMAIN
 from .discovery import ZwaveDiscoveryInfo
+from .discovery_data_template import DynamicCurrentTempClimateDataTemplate
 from .entity import ZWaveBaseEntity
+from .helpers import get_value_of_zwave_value
 
 # Map Z-Wave HVAC Mode to Home Assistant value
 # Note: We treat "auto" as "heat_cool" as most Z-Wave devices
 # report auto_changeover as auto without schedule support.
-ZW_HVAC_MODE_MAP: dict[int, str] = {
-    ThermostatMode.OFF: HVAC_MODE_OFF,
-    ThermostatMode.HEAT: HVAC_MODE_HEAT,
-    ThermostatMode.COOL: HVAC_MODE_COOL,
+ZW_HVAC_MODE_MAP: dict[int, HVACMode] = {
+    ThermostatMode.OFF: HVACMode.OFF,
+    ThermostatMode.HEAT: HVACMode.HEAT,
+    ThermostatMode.COOL: HVACMode.COOL,
     # Z-Wave auto mode is actually heat/cool in the hass world
-    ThermostatMode.AUTO: HVAC_MODE_HEAT_COOL,
-    ThermostatMode.AUXILIARY: HVAC_MODE_HEAT,
-    ThermostatMode.FAN: HVAC_MODE_FAN_ONLY,
-    ThermostatMode.FURNANCE: HVAC_MODE_HEAT,
-    ThermostatMode.DRY: HVAC_MODE_DRY,
-    ThermostatMode.AUTO_CHANGE_OVER: HVAC_MODE_HEAT_COOL,
-    ThermostatMode.HEATING_ECON: HVAC_MODE_HEAT,
-    ThermostatMode.COOLING_ECON: HVAC_MODE_COOL,
-    ThermostatMode.AWAY: HVAC_MODE_HEAT_COOL,
-    ThermostatMode.FULL_POWER: HVAC_MODE_HEAT,
+    ThermostatMode.AUTO: HVACMode.HEAT_COOL,
+    ThermostatMode.AUXILIARY: HVACMode.HEAT,
+    ThermostatMode.FAN: HVACMode.FAN_ONLY,
+    ThermostatMode.FURNANCE: HVACMode.HEAT,
+    ThermostatMode.DRY: HVACMode.DRY,
+    ThermostatMode.AUTO_CHANGE_OVER: HVACMode.HEAT_COOL,
+    ThermostatMode.HEATING_ECON: HVACMode.HEAT,
+    ThermostatMode.COOLING_ECON: HVACMode.COOL,
+    ThermostatMode.AWAY: HVACMode.HEAT_COOL,
+    ThermostatMode.FULL_POWER: HVACMode.HEAT,
 }
 
-HVAC_CURRENT_MAP: dict[int, str] = {
-    ThermostatOperatingState.IDLE: CURRENT_HVAC_IDLE,
-    ThermostatOperatingState.PENDING_HEAT: CURRENT_HVAC_IDLE,
-    ThermostatOperatingState.HEATING: CURRENT_HVAC_HEAT,
-    ThermostatOperatingState.PENDING_COOL: CURRENT_HVAC_IDLE,
-    ThermostatOperatingState.COOLING: CURRENT_HVAC_COOL,
-    ThermostatOperatingState.FAN_ONLY: CURRENT_HVAC_FAN,
-    ThermostatOperatingState.VENT_ECONOMIZER: CURRENT_HVAC_FAN,
-    ThermostatOperatingState.AUX_HEATING: CURRENT_HVAC_HEAT,
-    ThermostatOperatingState.SECOND_STAGE_HEATING: CURRENT_HVAC_HEAT,
-    ThermostatOperatingState.SECOND_STAGE_COOLING: CURRENT_HVAC_COOL,
-    ThermostatOperatingState.SECOND_STAGE_AUX_HEAT: CURRENT_HVAC_HEAT,
-    ThermostatOperatingState.THIRD_STAGE_AUX_HEAT: CURRENT_HVAC_HEAT,
+HVAC_CURRENT_MAP: dict[int, HVACAction] = {
+    ThermostatOperatingState.IDLE: HVACAction.IDLE,
+    ThermostatOperatingState.PENDING_HEAT: HVACAction.IDLE,
+    ThermostatOperatingState.HEATING: HVACAction.HEATING,
+    ThermostatOperatingState.PENDING_COOL: HVACAction.IDLE,
+    ThermostatOperatingState.COOLING: HVACAction.COOLING,
+    ThermostatOperatingState.FAN_ONLY: HVACAction.FAN,
+    ThermostatOperatingState.VENT_ECONOMIZER: HVACAction.FAN,
+    ThermostatOperatingState.AUX_HEATING: HVACAction.HEATING,
+    ThermostatOperatingState.SECOND_STAGE_HEATING: HVACAction.HEATING,
+    ThermostatOperatingState.SECOND_STAGE_COOLING: HVACAction.COOLING,
+    ThermostatOperatingState.SECOND_STAGE_AUX_HEAT: HVACAction.HEATING,
+    ThermostatOperatingState.THIRD_STAGE_AUX_HEAT: HVACAction.HEATING,
 }
 
 ATTR_FAN_STATE = "fan_state"
 
 
 async def async_setup_entry(
-    hass: HomeAssistant, config_entry: ConfigEntry, async_add_entities: Callable
+    hass: HomeAssistant,
+    config_entry: ConfigEntry,
+    async_add_entities: AddEntitiesCallback,
 ) -> None:
     """Set up Z-Wave climate from config entry."""
     client: ZwaveClient = hass.data[DOMAIN][config_entry.entry_id][DATA_CLIENT]
@@ -107,11 +102,14 @@ async def async_setup_entry(
     def async_add_climate(info: ZwaveDiscoveryInfo) -> None:
         """Add Z-Wave Climate."""
         entities: list[ZWaveBaseEntity] = []
-        entities.append(ZWaveClimate(config_entry, client, info))
+        if info.platform_hint == "dynamic_current_temp":
+            entities.append(DynamicCurrentTempClimate(config_entry, client, info))
+        else:
+            entities.append(ZWaveClimate(config_entry, client, info))
 
         async_add_entities(entities)
 
-    hass.data[DOMAIN][config_entry.entry_id][DATA_UNSUBSCRIBE].append(
+    config_entry.async_on_unload(
         async_dispatcher_connect(
             hass,
             f"{DOMAIN}_{config_entry.entry_id}_add_{CLIMATE_DOMAIN}",
@@ -126,9 +124,9 @@ class ZWaveClimate(ZWaveBaseEntity, ClimateEntity):
     def __init__(
         self, config_entry: ConfigEntry, client: ZwaveClient, info: ZwaveDiscoveryInfo
     ) -> None:
-        """Initialize lock."""
+        """Initialize thermostat."""
         super().__init__(config_entry, client, info)
-        self._hvac_modes: dict[str, int | None] = {}
+        self._hvac_modes: dict[HVACMode, int | None] = {}
         self._hvac_presets: dict[str, int | None] = {}
         self._unit_value: ZwaveValue | None = None
 
@@ -166,7 +164,7 @@ class ZWaveClimate(ZWaveBaseEntity, ClimateEntity):
         if not self._unit_value:
             self._unit_value = self._current_temp
         self._current_humidity = self.get_zwave_value(
-            "Humidity",
+            THERMOSTAT_HUMIDITY_PROPERTY,
             command_class=CommandClass.SENSOR_MULTILEVEL,
             add_to_watched_value_ids=True,
             check_all_endpoints=True,
@@ -184,29 +182,30 @@ class ZWaveClimate(ZWaveBaseEntity, ClimateEntity):
             check_all_endpoints=True,
         )
         self._set_modes_and_presets()
-        self._supported_features = 0
+        self._attr_supported_features = 0
         if len(self._hvac_presets) > 1:
-            self._supported_features |= SUPPORT_PRESET_MODE
+            self._attr_supported_features |= ClimateEntityFeature.PRESET_MODE
         # If any setpoint value exists, we can assume temperature
         # can be set
         if any(self._setpoint_values.values()):
-            self._supported_features |= SUPPORT_TARGET_TEMPERATURE
-        if HVAC_MODE_HEAT_COOL in self.hvac_modes:
-            self._supported_features |= SUPPORT_TARGET_TEMPERATURE_RANGE
+            self._attr_supported_features |= ClimateEntityFeature.TARGET_TEMPERATURE
+        if HVACMode.HEAT_COOL in self.hvac_modes:
+            self._attr_supported_features |= (
+                ClimateEntityFeature.TARGET_TEMPERATURE_RANGE
+            )
         if self._fan_mode:
-            self._supported_features |= SUPPORT_FAN_MODE
+            self._attr_supported_features |= ClimateEntityFeature.FAN_MODE
 
     def _setpoint_value(self, setpoint_type: ThermostatSetpointType) -> ZwaveValue:
         """Optionally return a ZwaveValue for a setpoint."""
-        val = self._setpoint_values[setpoint_type]
-        if val is None:
+        if (val := self._setpoint_values[setpoint_type]) is None:
             raise ValueError("Value requested is not available")
 
         return val
 
     def _set_modes_and_presets(self) -> None:
         """Convert Z-Wave Thermostat modes into Home Assistant modes and presets."""
-        all_modes: dict[str, int | None] = {}
+        all_modes: dict[HVACMode, int | None] = {}
         all_presets: dict[str, int | None] = {PRESET_NONE: None}
 
         # Z-Wave uses one list for both modes and presets.
@@ -220,8 +219,7 @@ class ZWaveClimate(ZWaveBaseEntity, ClimateEntity):
             mode_id = int(mode_id)
             if mode_id in THERMOSTAT_MODES:
                 # treat value as hvac mode
-                hass_mode = ZW_HVAC_MODE_MAP.get(mode_id)
-                if hass_mode:
+                if hass_mode := ZW_HVAC_MODE_MAP.get(mode_id):
                     all_modes[hass_mode] = mode_id
             else:
                 # treat value as hvac preset
@@ -235,7 +233,7 @@ class ZWaveClimate(ZWaveBaseEntity, ClimateEntity):
         if self._current_mode is None:
             # Thermostat(valve) with no support for setting a mode is considered heating-only
             return [ThermostatSetpointType.HEATING]
-        return THERMOSTAT_MODE_SETPOINT_MAP.get(int(self._current_mode.value), [])  # type: ignore
+        return THERMOSTAT_MODE_SETPOINT_MAP.get(int(self._current_mode.value), [])  # type: ignore[no-any-return]
 
     @property
     def temperature_unit(self) -> str:
@@ -254,23 +252,23 @@ class ZWaveClimate(ZWaveBaseEntity, ClimateEntity):
         return PRECISION_TENTHS
 
     @property
-    def hvac_mode(self) -> str:
+    def hvac_mode(self) -> HVACMode:
         """Return hvac operation ie. heat, cool mode."""
         if self._current_mode is None:
             # Thermostat(valve) with no support for setting a mode is considered heating-only
-            return HVAC_MODE_HEAT
+            return HVACMode.HEAT
         if self._current_mode.value is None:
             # guard missing value
-            return HVAC_MODE_HEAT
-        return ZW_HVAC_MODE_MAP.get(int(self._current_mode.value), HVAC_MODE_HEAT_COOL)
+            return HVACMode.HEAT
+        return ZW_HVAC_MODE_MAP.get(int(self._current_mode.value), HVACMode.HEAT_COOL)
 
     @property
-    def hvac_modes(self) -> list[str]:
+    def hvac_modes(self) -> list[HVACMode]:
         """Return the list of available hvac operation modes."""
         return list(self._hvac_modes)
 
     @property
-    def hvac_action(self) -> str | None:
+    def hvac_action(self) -> HVACAction | None:
         """Return the current running hvac operation if supported."""
         if not self._operating_state:
             return None
@@ -282,12 +280,12 @@ class ZWaveClimate(ZWaveBaseEntity, ClimateEntity):
     @property
     def current_humidity(self) -> int | None:
         """Return the current humidity level."""
-        return self._current_humidity.value if self._current_humidity else None
+        return get_value_of_zwave_value(self._current_humidity)
 
     @property
     def current_temperature(self) -> float | None:
         """Return the current temperature."""
-        return self._current_temp.value if self._current_temp else None
+        return get_value_of_zwave_value(self._current_temp)
 
     @property
     def target_temperature(self) -> float | None:
@@ -299,7 +297,7 @@ class ZWaveClimate(ZWaveBaseEntity, ClimateEntity):
             temp = self._setpoint_value(self._current_mode_setpoint_enums[0])
         except (IndexError, ValueError):
             return None
-        return temp.value if temp else None
+        return get_value_of_zwave_value(temp)
 
     @property
     def target_temperature_high(self) -> float | None:
@@ -311,7 +309,7 @@ class ZWaveClimate(ZWaveBaseEntity, ClimateEntity):
             temp = self._setpoint_value(self._current_mode_setpoint_enums[1])
         except (IndexError, ValueError):
             return None
-        return temp.value if temp else None
+        return get_value_of_zwave_value(temp)
 
     @property
     def target_temperature_low(self) -> float | None:
@@ -375,11 +373,6 @@ class ZWaveClimate(ZWaveBaseEntity, ClimateEntity):
 
         return None
 
-    @property
-    def supported_features(self) -> int:
-        """Return the list of supported features."""
-        return self._supported_features
-
     @property
     def min_temp(self) -> float:
         """Return the minimum temperature."""
@@ -432,7 +425,7 @@ class ZWaveClimate(ZWaveBaseEntity, ClimateEntity):
 
     async def async_set_temperature(self, **kwargs: Any) -> None:
         """Set new target temperature."""
-        hvac_mode: str | None = kwargs.get(ATTR_HVAC_MODE)
+        hvac_mode: HVACMode | None = kwargs.get(ATTR_HVAC_MODE)
 
         if hvac_mode is not None:
             await self.async_set_hvac_mode(hvac_mode)
@@ -457,17 +450,16 @@ class ZWaveClimate(ZWaveBaseEntity, ClimateEntity):
             if target_temp_high is not None:
                 await self.info.node.async_set_value(setpoint_high, target_temp_high)
 
-    async def async_set_hvac_mode(self, hvac_mode: str) -> None:
+    async def async_set_hvac_mode(self, hvac_mode: HVACMode) -> None:
         """Set new target hvac mode."""
-        if not self._current_mode:
-            # Thermostat(valve) with no support for setting a mode
-            raise ValueError(
-                f"Thermostat {self.entity_id} does not support setting a mode"
-            )
-        hvac_mode_value = self._hvac_modes.get(hvac_mode)
-        if hvac_mode_value is None:
+        if (hvac_mode_id := self._hvac_modes.get(hvac_mode)) is None:
             raise ValueError(f"Received an invalid hvac mode: {hvac_mode}")
-        await self.info.node.async_set_value(self._current_mode, hvac_mode_value)
+
+        if not self._current_mode:
+            # Thermostat(valve) has no support for setting a mode, so we make it a no-op
+            return
+
+        await self.info.node.async_set_value(self._current_mode, hvac_mode_id)
 
     async def async_set_preset_mode(self, preset_mode: str) -> None:
         """Set new target preset mode."""
@@ -479,3 +471,25 @@ class ZWaveClimate(ZWaveBaseEntity, ClimateEntity):
         if preset_mode_value is None:
             raise ValueError(f"Received an invalid preset mode: {preset_mode}")
         await self.info.node.async_set_value(self._current_mode, preset_mode_value)
+
+
+class DynamicCurrentTempClimate(ZWaveClimate):
+    """Representation of a thermostat that can dynamically use a different Zwave Value for current temp."""
+
+    def __init__(
+        self, config_entry: ConfigEntry, client: ZwaveClient, info: ZwaveDiscoveryInfo
+    ) -> None:
+        """Initialize thermostat."""
+        super().__init__(config_entry, client, info)
+        self.data_template = cast(
+            DynamicCurrentTempClimateDataTemplate, self.info.platform_data_template
+        )
+
+    @property
+    def current_temperature(self) -> float | None:
+        """Return the current temperature."""
+        assert self.info.platform_data
+        val = get_value_of_zwave_value(
+            self.data_template.current_temperature_value(self.info.platform_data)
+        )
+        return val if val is not None else super().current_temperature
