@@ -20,19 +20,10 @@ from zwave_js_server.model.value import Value as ZwaveValue
 from homeassistant.components.cover import (
     ATTR_POSITION,
     ATTR_TILT_POSITION,
-    DEVICE_CLASS_BLIND,
-    DEVICE_CLASS_GARAGE,
-    DEVICE_CLASS_SHUTTER,
-    DEVICE_CLASS_WINDOW,
     DOMAIN as COVER_DOMAIN,
-    SUPPORT_CLOSE,
-    SUPPORT_CLOSE_TILT,
-    SUPPORT_OPEN,
-    SUPPORT_OPEN_TILT,
-    SUPPORT_SET_POSITION,
-    SUPPORT_SET_TILT_POSITION,
-    SUPPORT_STOP,
+    CoverDeviceClass,
     CoverEntity,
+    CoverEntityFeature,
 )
 from homeassistant.config_entries import ConfigEntry
 from homeassistant.core import HomeAssistant, callback
@@ -119,11 +110,11 @@ class ZWaveCover(ZWaveBaseEntity, CoverEntity):
         super().__init__(config_entry, client, info)
 
         # Entity class attributes
-        self._attr_device_class = DEVICE_CLASS_WINDOW
+        self._attr_device_class = CoverDeviceClass.WINDOW
         if self.info.platform_hint in ("window_shutter", "window_shutter_tilt"):
-            self._attr_device_class = DEVICE_CLASS_SHUTTER
+            self._attr_device_class = CoverDeviceClass.SHUTTER
         if self.info.platform_hint == "window_blind":
-            self._attr_device_class = DEVICE_CLASS_BLIND
+            self._attr_device_class = CoverDeviceClass.BLIND
 
     @property
     def is_closed(self) -> bool | None:
@@ -180,16 +171,16 @@ class ZWaveCover(ZWaveBaseEntity, CoverEntity):
 
 
 class ZWaveTiltCover(ZWaveCover):
-    """Representation of a Fibaro Z-Wave cover device."""
+    """Representation of a Z-Wave Cover device with tilt."""
 
     _attr_supported_features = (
-        SUPPORT_OPEN
-        | SUPPORT_CLOSE
-        | SUPPORT_STOP
-        | SUPPORT_SET_POSITION
-        | SUPPORT_OPEN_TILT
-        | SUPPORT_CLOSE_TILT
-        | SUPPORT_SET_TILT_POSITION
+        CoverEntityFeature.OPEN
+        | CoverEntityFeature.CLOSE
+        | CoverEntityFeature.STOP
+        | CoverEntityFeature.SET_POSITION
+        | CoverEntityFeature.OPEN_TILT
+        | CoverEntityFeature.CLOSE_TILT
+        | CoverEntityFeature.SET_TILT_POSITION
     )
 
     def __init__(
@@ -234,8 +225,8 @@ class ZWaveTiltCover(ZWaveCover):
 class ZwaveMotorizedBarrier(ZWaveBaseEntity, CoverEntity):
     """Representation of a Z-Wave motorized barrier device."""
 
-    _attr_supported_features = SUPPORT_OPEN | SUPPORT_CLOSE
-    _attr_device_class = DEVICE_CLASS_GARAGE
+    _attr_supported_features = CoverEntityFeature.OPEN | CoverEntityFeature.CLOSE
+    _attr_device_class = CoverDeviceClass.GARAGE
 
     def __init__(
         self,
