@@ -14,8 +14,8 @@ import aiohttp
 
 from homeassistant.const import __version__ as HA_VERSION
 
-WHOAMI_URL = "https://whoami.home-assistant.io/v1"
-WHOAMI_URL_DEV = "https://whoami-v1-dev.home-assistant.workers.dev/v1"
+WHOAMI_URL = "https://services.home-assistant.io/whoami/v1"
+WHOAMI_URL_DEV = "https://services-dev.home-assistant.workers.dev/whoami/v1"
 
 # Constants from https://github.com/maurycyp/vincenty
 # Earth ellipsoid according to WGS 84
@@ -115,7 +115,7 @@ def vincenty(
         cosSigma = sinU1 * sinU2 + cosU1 * cosU2 * cosLambda
         sigma = math.atan2(sinSigma, cosSigma)
         sinAlpha = cosU1 * cosU2 * sinLambda / sinSigma
-        cosSqAlpha = 1 - sinAlpha ** 2
+        cosSqAlpha = 1 - sinAlpha**2
         try:
             cos2SigmaM = cosSigma - 2 * sinU1 * sinU2 / cosSqAlpha
         except ZeroDivisionError:
@@ -124,14 +124,14 @@ def vincenty(
         LambdaPrev = Lambda
         Lambda = L + (1 - C) * FLATTENING * sinAlpha * (
             sigma
-            + C * sinSigma * (cos2SigmaM + C * cosSigma * (-1 + 2 * cos2SigmaM ** 2))
+            + C * sinSigma * (cos2SigmaM + C * cosSigma * (-1 + 2 * cos2SigmaM**2))
         )
         if abs(Lambda - LambdaPrev) < CONVERGENCE_THRESHOLD:
             break  # successful convergence
     else:
         return None  # failure to converge
 
-    uSq = cosSqAlpha * (AXIS_A ** 2 - AXIS_B ** 2) / (AXIS_B ** 2)
+    uSq = cosSqAlpha * (AXIS_A**2 - AXIS_B**2) / (AXIS_B**2)
     A = 1 + uSq / 16384 * (4096 + uSq * (-768 + uSq * (320 - 175 * uSq)))
     B = uSq / 1024 * (256 + uSq * (-128 + uSq * (74 - 47 * uSq)))
     deltaSigma = (
@@ -142,12 +142,12 @@ def vincenty(
             + B
             / 4
             * (
-                cosSigma * (-1 + 2 * cos2SigmaM ** 2)
+                cosSigma * (-1 + 2 * cos2SigmaM**2)
                 - B
                 / 6
                 * cos2SigmaM
-                * (-3 + 4 * sinSigma ** 2)
-                * (-3 + 4 * cos2SigmaM ** 2)
+                * (-3 + 4 * sinSigma**2)
+                * (-3 + 4 * cos2SigmaM**2)
             )
         )
     )
